@@ -5,17 +5,15 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.connector;
 
 import java.io.IOException;
@@ -57,10 +55,10 @@ import org.apache.tomcat.util.res.StringManager;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: CoyoteAdapter.java 1176592 2011-09-27 20:19:01Z markt $
+ * @version $Id: CoyoteAdapter.java 1200174 2011-11-10 06:12:08Z kkolinko $
  */
 public class CoyoteAdapter implements Adapter {
-    
+
     private static final Log log = LogFactory.getLog(CoyoteAdapter.class);
 
     // -------------------------------------------------------------- Constants
@@ -76,7 +74,7 @@ public class CoyoteAdapter implements Adapter {
     public static final int ADAPTER_NOTES = 1;
 
 
-    protected static final boolean ALLOW_BACKSLASH = 
+    protected static final boolean ALLOW_BACKSLASH =
         Boolean.valueOf(System.getProperty("org.apache.catalina.connector.CoyoteAdapter.ALLOW_BACKSLASH", "false")).booleanValue();
 
 
@@ -136,14 +134,14 @@ public class CoyoteAdapter implements Adapter {
 
     // -------------------------------------------------------- Adapter Methods
 
-    
+
     /**
      * Event method.
-     * 
+     *
      * @return false to indicate an error, expected or not
      */
     @Override
-    public boolean event(org.apache.coyote.Request req, 
+    public boolean event(org.apache.coyote.Request req,
             org.apache.coyote.Response res, SocketStatus status) {
 
         Request request = (Request) req.getNote(ADAPTER_NOTES);
@@ -152,7 +150,7 @@ public class CoyoteAdapter implements Adapter {
         if (request.getWrapper() == null) {
             return false;
         }
-            
+
         boolean error = false;
         boolean read = false;
         try {
@@ -206,7 +204,7 @@ public class CoyoteAdapter implements Adapter {
             }
 
             req.getRequestProcessor().setWorkerThreadName(Thread.currentThread().getName());
-            
+
             // Calling the container
             connector.getService().getContainer().getPipeline().getFirst().event(request, response, request.getEvent());
 
@@ -249,13 +247,17 @@ public class CoyoteAdapter implements Adapter {
             req.getRequestProcessor().setWorkerThreadName(null);
             // Recycle the wrapper request and response
             if (error || response.isClosed() || !request.isComet()) {
+                ((Context) request.getMappingData().context).logAccess(
+                        request, response,
+                        System.currentTimeMillis() - req.getStartTime(),
+                        false);
                 request.recycle();
                 request.setFilterChain(null);
                 response.recycle();
             }
         }
     }
-    
+
     @Override
     public boolean asyncDispatch(org.apache.coyote.Request req,
             org.apache.coyote.Response res, SocketStatus status) throws Exception {
@@ -275,7 +277,7 @@ public class CoyoteAdapter implements Adapter {
                 // Error or timeout - need to tell listeners the request is over
                 // Have to test this first since state may change while in this
                 // method and this is only required if entering this method in
-                // this state 
+                // this state
                 Context ctxt = (Context) request.getMappingData().context;
                 if (ctxt != null) {
                     ctxt.fireRequestDestroyEvent(request);
@@ -343,7 +345,7 @@ public class CoyoteAdapter implements Adapter {
                 request.recycle();
                 response.recycle();
             } else {
-                // Clear converters so that the minimum amount of memory 
+                // Clear converters so that the minimum amount of memory
                 // is used by this processor
                 request.clearEncoders();
                 response.clearEncoders();
@@ -351,12 +353,12 @@ public class CoyoteAdapter implements Adapter {
         }
         return success;
     }
-    
+
     /**
      * Service method.
      */
     @Override
-    public void service(org.apache.coyote.Request req, 
+    public void service(org.apache.coyote.Request req,
                         org.apache.coyote.Response res)
         throws Exception {
 
@@ -391,10 +393,10 @@ public class CoyoteAdapter implements Adapter {
 
         boolean comet = false;
         boolean async = false;
-        
+
         try {
 
-            // Parse and set Catalina and configuration specific 
+            // Parse and set Catalina and configuration specific
             // request parameters
             req.getRequestProcessor().setWorkerThreadName(Thread.currentThread().getName());
             boolean postParseSuccess = postParseRequest(req, request, res, response);
@@ -430,9 +432,12 @@ public class CoyoteAdapter implements Adapter {
             } else if (!comet) {
                 request.finishRequest();
                 response.finishResponse();
-                if (postParseSuccess) {
+                if (postParseSuccess &&
+                        request.getMappingData().context != null) {
                     // Log only if processing was invoked.
                     // If postParseRequest() failed, it has already logged it.
+                    // If context is null this was the start of a comet request
+                    // that failed and has already been logged.
                     ((Context) request.getMappingData().context).logAccess(
                             request, response,
                             System.currentTimeMillis() - req.getStartTime(),
@@ -450,7 +455,7 @@ public class CoyoteAdapter implements Adapter {
                 request.recycle();
                 response.recycle();
             } else {
-                // Clear converters so that the minimum amount of memory 
+                // Clear converters so that the minimum amount of memory
                 // is used by this processor
                 request.clearEncoders();
                 response.clearEncoders();
@@ -466,7 +471,7 @@ public class CoyoteAdapter implements Adapter {
 
         Request request = (Request) req.getNote(ADAPTER_NOTES);
         Response response = (Response) res.getNote(ADAPTER_NOTES);
-        
+
         if (request == null) {
             // Create objects
             request = connector.createRequest();
@@ -486,7 +491,7 @@ public class CoyoteAdapter implements Adapter {
             req.getParameters().setQueryStringEncoding
                 (connector.getURIEncoding());
         }
-        
+
         try {
             // Log at the lowest level available. logAccess() will be
             // automatically called on parent containers.
@@ -514,8 +519,8 @@ public class CoyoteAdapter implements Adapter {
             response.recycle();
         }
     }
-    
-    
+
+
     @Override
     public String getDomain() {
         return connector.getDomain();
@@ -528,13 +533,13 @@ public class CoyoteAdapter implements Adapter {
     /**
      * Parse additional request parameters.
      */
-    protected boolean postParseRequest(org.apache.coyote.Request req, 
+    protected boolean postParseRequest(org.apache.coyote.Request req,
                                        Request request,
-                                       org.apache.coyote.Response res, 
+                                       org.apache.coyote.Response res,
                                        Response response)
             throws Exception {
 
-        // XXX the processor may have set a correct scheme and port prior to this point, 
+        // XXX the processor may have set a correct scheme and port prior to this point,
         // in ajp13 protocols dont make sense to get the port from the connector...
         // otherwise, use connector configuration
         if (! req.scheme().isNull()) {
@@ -547,11 +552,11 @@ public class CoyoteAdapter implements Adapter {
             request.setSecure(connector.getSecure());
         }
 
-        // FIXME: the code below doesnt belongs to here, 
-        // this is only have sense 
+        // FIXME: the code below doesnt belongs to here,
+        // this is only have sense
         // in Http11, not in ajp13..
         // At this point the Host header has been processed.
-        // Override if the proxyPort/proxyHost are set 
+        // Override if the proxyPort/proxyHost are set
         String proxyName = connector.getProxyName();
         int proxyPort = connector.getProxyPort();
         if (proxyPort != 0) {
@@ -564,12 +569,12 @@ public class CoyoteAdapter implements Adapter {
         // Copy the raw URI to the decodedURI
         MessageBytes decodedURI = req.decodedURI();
         decodedURI.duplicate(req.requestURI());
-        
+
         // Parse the path parameters. This will:
         //   - strip out the path parameters
         //   - convert the decodedURI to bytes
         parsePathParameters(req, request);
-        
+
         // URI decoding
         // %xx decoding of the URL
         try {
@@ -628,10 +633,10 @@ public class CoyoteAdapter implements Adapter {
             //reset mapping data, should prolly be done elsewhere
             request.getMappingData().recycle();
         }
-        
+
         boolean mapRequired = true;
         String version = null;
-        
+
         while (mapRequired) {
             if (version != null) {
                 // Once we have a version - that is it
@@ -661,14 +666,14 @@ public class CoyoteAdapter implements Adapter {
                 }
                 return false;
             }
-        
+
             // Now we have the context, we can parse the session ID from the URL
             // (if any). Need to do this before we redirect in case we need to
             // include the session id in the redirect
             String sessionID = null;
             if (request.getServletContext().getEffectiveSessionTrackingModes()
                     .contains(SessionTrackingMode.URL)) {
-                
+
                 // Get the session ID if there was one
                 sessionID = request.getPathParameter(
                         ApplicationSessionCookieConfig.getSessionUriParamName(
@@ -682,9 +687,9 @@ public class CoyoteAdapter implements Adapter {
             // Look for session ID in cookies and SSL session
             parseSessionCookiesId(req, request);
             parseSessionSslId(request);
-            
+
             sessionID = request.getRequestedSessionId();
-            
+
             if (mapRequired) {
                 if (sessionID == null) {
                     // No session means no possibility of needing to remap
@@ -702,7 +707,7 @@ public class CoyoteAdapter implements Adapter {
                                 // Set version so second time through mapping the
                                 // correct context is found
                                 version = ctxt.getWebappVersion();
-                                // Reset mapping 
+                                // Reset mapping
                                 request.getMappingData().recycle();
                                 break;
                             }
@@ -712,7 +717,7 @@ public class CoyoteAdapter implements Adapter {
                         // No matching context found. No need to re-map
                         mapRequired = false;
                     }
-                }                
+                }
             }
 
         }
@@ -741,7 +746,7 @@ public class CoyoteAdapter implements Adapter {
         }
 
         // Filter trace method
-        if (!connector.getAllowTrace() 
+        if (!connector.getAllowTrace()
                 && req.method().equalsIgnoreCase("TRACE")) {
             Wrapper wrapper = request.getWrapper();
             String header = null;
@@ -759,7 +764,7 @@ public class CoyoteAdapter implements Adapter {
                         }
                     }
                 }
-            }                               
+            }
             res.setStatus(405);
             res.addHeader("Allow", header);
             res.setMessage("TRACE method is not allowed");
@@ -776,7 +781,7 @@ public class CoyoteAdapter implements Adapter {
      * of the form /path;name=value;name2=value2/ etc. Currently only really
      * interested in the session ID that will be in this form. Other parameters
      * can safely be ignored.
-     * 
+     *
      * @param req
      * @param request
      */
@@ -831,14 +836,14 @@ public class CoyoteAdapter implements Adapter {
                 // Extract path param from decoded request URI
                 byte[] buf = uriBC.getBuffer();
                 for (int i = 0; i < end - start - pathParamEnd; i++) {
-                    buf[start + semicolon + i] 
+                    buf[start + semicolon + i]
                         = buf[start + i + pathParamEnd];
                 }
                 uriBC.setBytes(buf, start,
                         end - start - pathParamEnd + semicolon);
             } else {
                 if (charset != null) {
-                    pv = new String(uriBC.getBuffer(), start + pathParamStart, 
+                    pv = new String(uriBC.getBuffer(), start + pathParamStart,
                                 (end - start) - pathParamStart, charset);
                 }
                 uriBC.setEnd(start + semicolon);
@@ -856,7 +861,7 @@ public class CoyoteAdapter implements Adapter {
                 int equals = pv.indexOf('=');
                 if (equals > -1) {
                     String name = pv.substring(0, equals);
-                    String value = pv.substring(equals + 1); 
+                    String value = pv.substring(equals + 1);
                     request.addPathParameter(name, value);
                     if (log.isDebugEnabled()) {
                         log.debug(sm.getString("coyoteAdapter.debug", "equals",
@@ -872,8 +877,8 @@ public class CoyoteAdapter implements Adapter {
             semicolon = uriBC.indexOf(';', semicolon);
         }
     }
-    
-    
+
+
     /**
      * Look for SSL session ID if required. Only look for SSL Session ID if it
      * is the only tracking method enabled.
@@ -891,8 +896,8 @@ public class CoyoteAdapter implements Adapter {
             request.setRequestedSessionSSL(true);
         }
     }
-    
-    
+
+
     /**
      * Parse session id in URL.
      */
@@ -905,14 +910,16 @@ public class CoyoteAdapter implements Adapter {
         Context context = (Context) request.getMappingData().context;
         if (context != null && !context.getServletContext()
                 .getEffectiveSessionTrackingModes().contains(
-                        SessionTrackingMode.COOKIE))
+                        SessionTrackingMode.COOKIE)) {
             return;
-        
+        }
+
         // Parse session id from cookies
         Cookies serverCookies = req.getCookies();
         int count = serverCookies.getCookieCount();
-        if (count <= 0)
+        if (count <= 0) {
             return;
+        }
 
         String sessionCookieName =
             ApplicationSessionCookieConfig.getSessionCookieName(context);
@@ -928,9 +935,10 @@ public class CoyoteAdapter implements Adapter {
                         (scookie.getValue().toString());
                     request.setRequestedSessionCookie(true);
                     request.setRequestedSessionURL(false);
-                    if (log.isDebugEnabled())
+                    if (log.isDebugEnabled()) {
                         log.debug(" Requested cookie session id is " +
                             request.getRequestedSessionId());
+                    }
                 } else {
                     if (!request.isRequestedSessionIdValid()) {
                         // Replace the session id until one is valid
@@ -948,7 +956,7 @@ public class CoyoteAdapter implements Adapter {
     /**
      * Character conversion of the URI.
      */
-    protected void convertURI(MessageBytes uri, Request request) 
+    protected void convertURI(MessageBytes uri, Request request)
         throws Exception {
 
         ByteChunk bc = uri.getByteChunk();
@@ -972,7 +980,7 @@ public class CoyoteAdapter implements Adapter {
             if (conv != null) {
                 try {
                     conv.convert(bc, cc, cc.getBuffer().length - cc.getEnd());
-                    uri.setChars(cc.getBuffer(), cc.getStart(), 
+                    uri.setChars(cc.getBuffer(), cc.getStart(),
                                  cc.getLength());
                     return;
                 } catch (IOException e) {
@@ -1000,9 +1008,10 @@ public class CoyoteAdapter implements Adapter {
     protected void convertMB(MessageBytes mb) {
 
         // This is of course only meaningful for bytes
-        if (mb.getType() != MessageBytes.T_BYTES)
+        if (mb.getType() != MessageBytes.T_BYTES) {
             return;
-        
+        }
+
         ByteChunk bc = mb.getByteChunk();
         CharChunk cc = mb.getCharChunk();
         int length = bc.getLength();
@@ -1026,7 +1035,7 @@ public class CoyoteAdapter implements Adapter {
      * This method normalizes "\", "//", "/./" and "/../". This method will
      * return false when trying to go above the root, or if the URI contains
      * a null byte.
-     * 
+     *
      * @param uriMB URI to be normalized
      */
     public static boolean normalize(MessageBytes uriMB) {
@@ -1037,12 +1046,14 @@ public class CoyoteAdapter implements Adapter {
         int end = uriBC.getEnd();
 
         // An empty URL is not acceptable
-        if (start == end)
+        if (start == end) {
             return false;
+        }
 
         // URL * is acceptable
-        if ((end - start == 1) && b[start] == (byte) '*')
-          return true;
+        if ((end - start == 1) && b[start] == (byte) '*') {
+            return true;
+        }
 
         int pos = 0;
         int index = 0;
@@ -1081,8 +1092,8 @@ public class CoyoteAdapter implements Adapter {
         // Note: It is possible to extend the URI by 1 without any side effect
         // as the next character is a non-significant WS.
         if (((end - start) >= 2) && (b[end - 1] == (byte) '.')) {
-            if ((b[end - 2] == (byte) '/') 
-                || ((b[end - 2] == (byte) '.') 
+            if ((b[end - 2] == (byte) '/')
+                || ((b[end - 2] == (byte) '.')
                     && (b[end - 3] == (byte) '/'))) {
                 b[end] = (byte) '/';
                 end++;
@@ -1096,9 +1107,10 @@ public class CoyoteAdapter implements Adapter {
         // Resolve occurrences of "/./" in the normalized path
         while (true) {
             index = uriBC.indexOf("/./", 0, 3, index);
-            if (index < 0)
+            if (index < 0) {
                 break;
-            copyBytes(b, start + index, start + index + 2, 
+            }
+            copyBytes(b, start + index, start + index + 2,
                       end - start - index - 2);
             end = end - 2;
             uriBC.setEnd(end);
@@ -1109,11 +1121,13 @@ public class CoyoteAdapter implements Adapter {
         // Resolve occurrences of "/../" in the normalized path
         while (true) {
             index = uriBC.indexOf("/../", 0, 4, index);
-            if (index < 0)
+            if (index < 0) {
                 break;
+            }
             // Prevent from going outside our context
-            if (index == 0)
+            if (index == 0) {
                 return false;
+            }
             int index2 = -1;
             for (pos = start + index - 1; (pos >= 0) && (index2 < 0); pos --) {
                 if (b[pos] == (byte) '/') {
@@ -1136,9 +1150,9 @@ public class CoyoteAdapter implements Adapter {
      * Check that the URI is normalized following character decoding.
      * <p>
      * This method checks for "\", 0, "//", "/./" and "/../". This method will
-     * return false if sequences that are supposed to be normalized are still 
+     * return false if sequences that are supposed to be normalized are still
      * present in the URI.
-     * 
+     *
      * @param uriMB URI to be checked (should be chars)
      */
     public static boolean checkNormalize(MessageBytes uriMB) {
@@ -1171,8 +1185,8 @@ public class CoyoteAdapter implements Adapter {
 
         // Check for ending with "/." or "/.."
         if (((end - start) >= 2) && (c[end - 1] == '.')) {
-            if ((c[end - 2] == '/') 
-                    || ((c[end - 2] == '.') 
+            if ((c[end - 2] == '/')
+                    || ((c[end - 2] == '.')
                     && (c[end - 3] == '/'))) {
                 return false;
             }
@@ -1197,7 +1211,7 @@ public class CoyoteAdapter implements Adapter {
 
 
     /**
-     * Copy an array of bytes to a different position. Used during 
+     * Copy an array of bytes to a different position. Used during
      * normalization.
      */
     protected static void copyBytes(byte[] b, int dest, int src, int len) {
