@@ -5,17 +5,15 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.connector;
 
 
@@ -49,6 +47,7 @@ import org.apache.catalina.core.ApplicationSessionCookieConfig;
 import org.apache.catalina.security.SecurityUtil;
 import org.apache.catalina.util.CharsetMapper;
 import org.apache.catalina.util.DateTool;
+import org.apache.catalina.util.RequestUtil;
 import org.apache.tomcat.util.buf.CharChunk;
 import org.apache.tomcat.util.buf.UEncoder;
 import org.apache.tomcat.util.http.FastHttpDateFormat;
@@ -62,7 +61,7 @@ import org.apache.tomcat.util.res.StringManager;
  *
  * @author Remy Maucherat
  * @author Craig R. McClanahan
- * @version $Id: Response.java 1088458 2011-04-04 00:57:17Z kkolinko $
+ * @version $Id: Response.java 1200178 2011-11-10 06:14:46Z kkolinko $
  */
 
 public class Response
@@ -129,6 +128,7 @@ public class Response
     /**
      * Return the Connector through which this Request was received.
      */
+    @Deprecated
     public Connector getConnector() {
         return (this.connector);
     }
@@ -158,7 +158,7 @@ public class Response
 
     /**
      * Set the Coyote response.
-     * 
+     *
      * @param coyoteResponse The Coyote response
      */
     public void setCoyoteResponse(org.apache.coyote.Response coyoteResponse) {
@@ -169,6 +169,7 @@ public class Response
     /**
      * Get the Coyote response.
      */
+    @Deprecated
     public org.apache.coyote.Response getCoyoteResponse() {
         return (coyoteResponse);
     }
@@ -189,6 +190,7 @@ public class Response
      *
      * @param context The newly associated Context
      */
+    @Deprecated
     public void setContext(Context context) {
         request.setContext(context);
     }
@@ -223,12 +225,12 @@ public class Response
      */
     protected boolean included = false;
 
-    
+
     /**
      * The characterEncoding flag
      */
     private boolean isCharacterEncodingSet = false;
-    
+
     /**
      * The error flag.
      */
@@ -275,7 +277,7 @@ public class Response
         included = false;
         error = false;
         isCharacterEncodingSet = false;
-        
+
         if (Globals.IS_SECURITY_ENABLED || Connector.RECYCLE_FACADES) {
             if (facade != null) {
                 facade.clear();
@@ -302,7 +304,7 @@ public class Response
     public void clearEncoders() {
         outputBuffer.clearEncoders();
     }
-    
+
 
     // ------------------------------------------------------- Response Methods
 
@@ -334,7 +336,7 @@ public class Response
 
     /**
      * Set the application commit flag.
-     * 
+     *
      * @param appCommitted The new application committed flag value
      */
     public void setAppCommitted(boolean appCommitted) {
@@ -347,7 +349,7 @@ public class Response
      */
     public boolean isAppCommitted() {
         return (this.appCommitted || isCommitted() || isSuspended()
-                || ((getContentLength() > 0) 
+                || ((getContentLength() > 0)
                     && (getContentWritten() >= getContentLength())));
     }
 
@@ -355,6 +357,7 @@ public class Response
     /**
      * Return the "processing inside an include" flag.
      */
+    @Deprecated
     public boolean getIncluded() {
         return included;
     }
@@ -366,6 +369,7 @@ public class Response
      * @param included <code>true</code> if we are currently inside a
      *  RequestDispatcher.include(), else <code>false</code>
      */
+    @Deprecated
     public void setIncluded(boolean included) {
         this.included = included;
     }
@@ -423,6 +427,7 @@ public class Response
     /**
      * Return the output stream associated with this Response.
      */
+    @Deprecated
     public OutputStream getStream() {
         if (outputStream == null) {
             outputStream = new CoyoteOutputStream(outputBuffer);
@@ -433,7 +438,7 @@ public class Response
 
     /**
      * Set the suspended flag.
-     * 
+     *
      * @param suspended The new suspended flag value
      */
     public void setSuspended(boolean suspended) {
@@ -479,7 +484,8 @@ public class Response
      *
      * @exception IOException if an input/output error occurs
      */
-    public ServletOutputStream createOutputStream() 
+    @Deprecated
+    public ServletOutputStream createOutputStream()
         throws IOException {
         // Probably useless
         if (outputStream == null) {
@@ -495,7 +501,7 @@ public class Response
      *
      * @exception IOException if an input/output error occurs
      */
-    public void finishResponse() 
+    public void finishResponse()
         throws IOException {
         // Writing leftover bytes
         outputBuffer.close();
@@ -554,7 +560,7 @@ public class Response
      * @exception IOException if an input/output error occurs
      */
     @Override
-    public void flushBuffer() 
+    public void flushBuffer()
         throws IOException {
         outputBuffer.flush();
     }
@@ -586,12 +592,13 @@ public class Response
      * @exception IOException if an input/output error occurs
      */
     @Override
-    public ServletOutputStream getOutputStream() 
+    public ServletOutputStream getOutputStream()
         throws IOException {
 
-        if (usingWriter)
+        if (usingWriter) {
             throw new IllegalStateException
                 (sm.getString("coyoteResponse.getOutputStream.ise"));
+        }
 
         usingOutputStream = true;
         if (outputStream == null) {
@@ -619,12 +626,13 @@ public class Response
      * @exception IOException if an input/output error occurs
      */
     @Override
-    public PrintWriter getWriter() 
+    public PrintWriter getWriter()
         throws IOException {
 
-        if (usingOutputStream)
+        if (usingOutputStream) {
             throw new IllegalStateException
                 (sm.getString("coyoteResponse.getWriter.ise"));
+        }
 
         if (ENFORCE_ENCODING_IN_GET_WRITER) {
             /*
@@ -671,7 +679,9 @@ public class Response
     public void reset() {
 
         if (included)
+         {
             return;     // Ignore any call from an included servlet
+        }
 
         coyoteResponse.reset();
         outputBuffer.reset();
@@ -692,7 +702,7 @@ public class Response
         resetBuffer(false);
     }
 
-    
+
     /**
      * Reset the data buffer and the using Writer/Stream flags but not any
      * status or header information.
@@ -700,18 +710,19 @@ public class Response
      * @param resetWriterStreamFlags <code>true</code> if the internal
      *        <code>usingWriter</code>, <code>usingOutputStream</code>,
      *        <code>isCharacterEncodingSet</code> flags should also be reset
-     * 
+     *
      * @exception IllegalStateException if the response has already
      *  been committed
      */
     public void resetBuffer(boolean resetWriterStreamFlags) {
 
-        if (isCommitted())
+        if (isCommitted()) {
             throw new IllegalStateException
                 (sm.getString("coyoteResponse.resetBuffer.ise"));
+        }
 
         outputBuffer.reset();
-        
+
         if(resetWriterStreamFlags) {
             usingOutputStream = false;
             usingWriter = false;
@@ -720,7 +731,7 @@ public class Response
 
     }
 
-    
+
     /**
      * Set the buffer size to be used for this Response.
      *
@@ -732,9 +743,10 @@ public class Response
     @Override
     public void setBufferSize(int size) {
 
-        if (isCommitted() || !outputBuffer.isNew())
+        if (isCommitted() || !outputBuffer.isNew()) {
             throw new IllegalStateException
                 (sm.getString("coyoteResponse.setBufferSize.ise"));
+        }
 
         outputBuffer.setBufferSize(size);
 
@@ -749,13 +761,15 @@ public class Response
     @Override
     public void setContentLength(int length) {
 
-        if (isCommitted())
+        if (isCommitted()) {
             return;
+        }
 
         // Ignore any call from an included servlet
-        if (included)
+        if (included) {
             return;
-        
+        }
+
         coyoteResponse.setContentLength(length);
 
     }
@@ -770,12 +784,14 @@ public class Response
     @SuppressWarnings("deprecation") // isSpace (deprecated) cannot be replaced by isWhiteSpace
     public void setContentType(String type) {
 
-        if (isCommitted())
+        if (isCommitted()) {
             return;
+        }
 
         // Ignore any call from an included servlet
-        if (included)
+        if (included) {
             return;
+        }
 
         // Ignore charset if getWriter() has already been called
         if (usingWriter) {
@@ -825,17 +841,20 @@ public class Response
     @Override
     public void setCharacterEncoding(String charset) {
 
-        if (isCommitted())
+        if (isCommitted()) {
             return;
-        
+        }
+
         // Ignore any call from an included servlet
-        if (included)
-            return;     
-        
+        if (included) {
+            return;
+        }
+
         // Ignore any call made after the getWriter has been invoked
         // The default should be used
-        if (usingWriter)
+        if (usingWriter) {
             return;
+        }
 
         coyoteResponse.setCharacterEncoding(charset);
         isCharacterEncodingSet = true;
@@ -851,19 +870,22 @@ public class Response
     @Override
     public void setLocale(Locale locale) {
 
-        if (isCommitted())
+        if (isCommitted()) {
             return;
+        }
 
         // Ignore any call from an included servlet
-        if (included)
+        if (included) {
             return;
+        }
 
         coyoteResponse.setLocale(locale);
 
         // Ignore any call made after the getWriter has been invoked.
         // The default should be used
-        if (usingWriter)
+        if (usingWriter) {
             return;
+        }
 
         if (isCharacterEncodingSet) {
             return;
@@ -956,6 +978,7 @@ public class Response
      * @exception IllegalStateException if this response has already been
      *  committed
      */
+    @Deprecated
     public void reset(int status, String message) {
         reset();
         setStatus(status, message);
@@ -975,8 +998,9 @@ public class Response
     public void addCookie(final Cookie cookie) {
 
         // Ignore any call from an included servlet
-        if (included || isCommitted())
+        if (included || isCommitted()) {
             return;
+        }
 
         final StringBuffer sb = generateCookieString(cookie);
         //if we reached here, no exception, cookie is valid
@@ -987,14 +1011,15 @@ public class Response
     }
 
     /**
-     * Special method for adding a session cookie as we should be overriding 
-     * any previous 
+     * Special method for adding a session cookie as we should be overriding
+     * any previous
      * @param cookie
      */
     public void addSessionCookieInternal(final Cookie cookie) {
-        if (isCommitted())
+        if (isCommitted()) {
             return;
-        
+        }
+
         String name = cookie.getName();
         final String headername = "Set-Cookie";
         final String startsWith = name + "=";
@@ -1013,22 +1038,22 @@ public class Response
         if (!set) {
             addHeader(headername, sb.toString());
         }
-        
-        
+
+
     }
 
     public StringBuffer generateCookieString(final Cookie cookie) {
         final StringBuffer sb = new StringBuffer();
-        //web application code can receive a IllegalArgumentException 
+        //web application code can receive a IllegalArgumentException
         //from the appendCookieValue invocation
         if (SecurityUtil.isPackageProtectionEnabled()) {
             AccessController.doPrivileged(new PrivilegedAction<Void>() {
                 @Override
                 public Void run(){
                     ServerCookie.appendCookieValue
-                        (sb, cookie.getVersion(), cookie.getName(), 
-                         cookie.getValue(), cookie.getPath(), 
-                         cookie.getDomain(), cookie.getComment(), 
+                        (sb, cookie.getVersion(), cookie.getName(),
+                         cookie.getValue(), cookie.getPath(),
+                         cookie.getDomain(), cookie.getComment(),
                          cookie.getMaxAge(), cookie.getSecure(),
                          cookie.isHttpOnly());
                     return null;
@@ -1037,7 +1062,7 @@ public class Response
         } else {
             ServerCookie.appendCookieValue
                 (sb, cookie.getVersion(), cookie.getName(), cookie.getValue(),
-                     cookie.getPath(), cookie.getDomain(), cookie.getComment(), 
+                     cookie.getPath(), cookie.getDomain(), cookie.getComment(),
                      cookie.getMaxAge(), cookie.getSecure(),
                      cookie.isHttpOnly());
         }
@@ -1058,8 +1083,9 @@ public class Response
             return;
         }
 
-        if (isCommitted())
+        if (isCommitted()) {
             return;
+        }
 
         // Ignore any call from an included servlet
         if (included) {
@@ -1090,12 +1116,14 @@ public class Response
             return;
         }
 
-        if (isCommitted())
+        if (isCommitted()) {
             return;
+        }
 
         // Ignore any call from an included servlet
-        if (included)
+        if (included) {
             return;
+        }
 
         coyoteResponse.addHeader(name, value);
 
@@ -1115,12 +1143,14 @@ public class Response
             return;
         }
 
-        if (isCommitted())
+        if (isCommitted()) {
             return;
+        }
 
         // Ignore any call from an included servlet
-        if (included)
+        if (included) {
             return;
+        }
 
         addHeader(name, "" + value);
 
@@ -1194,12 +1224,14 @@ public class Response
      */
     @Override
     public String encodeURL(String url) {
-        
+
         String absolute = toAbsolute(url);
         if (isEncodeable(absolute)) {
-            // W3c spec clearly said 
-            if (url.equalsIgnoreCase("")){
+            // W3c spec clearly said
+            if (url.equalsIgnoreCase("")) {
                 url = absolute;
+            } else if (url.equals(absolute) && !hasPath(url)) {
+                url += '/';
             }
             return (toEncoded(url, request.getSessionInternal().getIdInternal()));
         } else {
@@ -1227,18 +1259,20 @@ public class Response
 
     /**
      * Send an acknowledgment of a request.
-     * 
+     *
      * @exception IOException if an input/output error occurs
      */
     public void sendAcknowledgement()
         throws IOException {
 
-        if (isCommitted())
+        if (isCommitted()) {
             return;
+        }
 
         // Ignore any call from an included servlet
-        if (included)
-            return; 
+        if (included) {
+            return;
+        }
 
         coyoteResponse.acknowledge();
 
@@ -1256,7 +1290,7 @@ public class Response
      * @exception IOException if an input/output error occurs
      */
     @Override
-    public void sendError(int status) 
+    public void sendError(int status)
         throws IOException {
         sendError(status, null);
     }
@@ -1273,21 +1307,23 @@ public class Response
      * @exception IOException if an input/output error occurs
      */
     @Override
-    public void sendError(int status, String message) 
+    public void sendError(int status, String message)
         throws IOException {
 
-        if (isCommitted())
+        if (isCommitted()) {
             throw new IllegalStateException
                 (sm.getString("coyoteResponse.sendError.ise"));
+        }
 
         // Ignore any call from an included servlet
-        if (included)
-            return; 
+        if (included) {
+            return;
+        }
 
         Wrapper wrapper = getRequest().getWrapper();
         if (wrapper != null) {
             wrapper.incrementErrorCount();
-        } 
+        }
 
         setError();
 
@@ -1313,25 +1349,33 @@ public class Response
      * @exception IOException if an input/output error occurs
      */
     @Override
-    public void sendRedirect(String location) 
+    public void sendRedirect(String location)
         throws IOException {
 
-        if (isCommitted())
+        if (isCommitted()) {
             throw new IllegalStateException
                 (sm.getString("coyoteResponse.sendRedirect.ise"));
+        }
 
         // Ignore any call from an included servlet
-        if (included)
-            return; 
+        if (included) {
+            return;
+        }
 
         // Clear any data content that has been buffered
-        resetBuffer();
+        resetBuffer(true);
 
         // Generate a temporary redirect to the specified location
         try {
             String absolute = toAbsolute(location);
             setStatus(SC_FOUND);
             setHeader("Location", absolute);
+            if (getContext().getSendRedirectBody()) {
+                PrintWriter writer = getWriter();
+                writer.print(sm.getString("coyoteResponse.sendRedirect.note",
+                        RequestUtil.filter(absolute)));
+                flushBuffer();
+            }
         } catch (IllegalArgumentException e) {
             setStatus(SC_NOT_FOUND);
         }
@@ -1355,8 +1399,9 @@ public class Response
             return;
         }
 
-        if (isCommitted())
+        if (isCommitted()) {
             return;
+        }
 
         // Ignore any call from an included servlet
         if (included) {
@@ -1387,12 +1432,14 @@ public class Response
             return;
         }
 
-        if (isCommitted())
+        if (isCommitted()) {
             return;
+        }
 
         // Ignore any call from an included servlet
-        if (included)
+        if (included) {
             return;
+        }
 
         coyoteResponse.setHeader(name, value);
 
@@ -1412,12 +1459,14 @@ public class Response
             return;
         }
 
-        if (isCommitted())
+        if (isCommitted()) {
             return;
+        }
 
         // Ignore any call from an included servlet
-        if (included)
+        if (included) {
             return;
+        }
 
         setHeader(name, "" + value);
 
@@ -1449,12 +1498,14 @@ public class Response
     @Deprecated
     public void setStatus(int status, String message) {
 
-        if (isCommitted())
+        if (isCommitted()) {
             return;
+        }
 
         // Ignore any call from an included servlet
-        if (included)
+        if (included) {
             return;
+        }
 
         coyoteResponse.setStatus(status);
         coyoteResponse.setMessage(message);
@@ -1480,25 +1531,30 @@ public class Response
      */
     protected boolean isEncodeable(final String location) {
 
-        if (location == null)
+        if (location == null) {
             return (false);
+        }
 
         // Is this an intra-document reference?
-        if (location.startsWith("#"))
+        if (location.startsWith("#")) {
             return (false);
+        }
 
         // Are we in a valid session that is not using cookies?
         final Request hreq = request;
         final Session session = hreq.getSessionInternal(false);
-        if (session == null)
+        if (session == null) {
             return (false);
-        if (hreq.isRequestedSessionIdFromCookie())
+        }
+        if (hreq.isRequestedSessionIdFromCookie()) {
             return (false);
-        
+        }
+
         // Is URL encoding permitted
         if (!hreq.getServletContext().getEffectiveSessionTrackingModes().
-                contains(SessionTrackingMode.URL))
+                contains(SessionTrackingMode.URL)) {
             return false;
+        }
 
         if (SecurityUtil.isPackageProtectionEnabled()) {
             return (
@@ -1514,7 +1570,7 @@ public class Response
         }
     }
 
-    private boolean doIsEncodeable(Request hreq, Session session, 
+    private boolean doIsEncodeable(Request hreq, Session session,
                                    String location) {
         // Is this a valid absolute URL?
         URL url = null;
@@ -1525,38 +1581,45 @@ public class Response
         }
 
         // Does this URL match down to (and including) the context path?
-        if (!hreq.getScheme().equalsIgnoreCase(url.getProtocol()))
+        if (!hreq.getScheme().equalsIgnoreCase(url.getProtocol())) {
             return (false);
-        if (!hreq.getServerName().equalsIgnoreCase(url.getHost()))
+        }
+        if (!hreq.getServerName().equalsIgnoreCase(url.getHost())) {
             return (false);
+        }
         int serverPort = hreq.getServerPort();
         if (serverPort == -1) {
-            if ("https".equals(hreq.getScheme()))
+            if ("https".equals(hreq.getScheme())) {
                 serverPort = 443;
-            else
+            } else {
                 serverPort = 80;
+            }
         }
         int urlPort = url.getPort();
         if (urlPort == -1) {
-            if ("https".equals(url.getProtocol()))
+            if ("https".equals(url.getProtocol())) {
                 urlPort = 443;
-            else
+            } else {
                 urlPort = 80;
+            }
         }
-        if (serverPort != urlPort)
+        if (serverPort != urlPort) {
             return (false);
+        }
 
         String contextPath = getContext().getPath();
         if (contextPath != null) {
             String file = url.getFile();
-            if ((file == null) || !file.startsWith(contextPath))
+            if ((file == null) || !file.startsWith(contextPath)) {
                 return (false);
+            }
             String tok = ";" +
                     ApplicationSessionCookieConfig.getSessionUriParamName(
                                 request.getContext()) +
                     "=" + session.getIdInternal();
-            if( file.indexOf(tok, contextPath.length()) >= 0 )
+            if( file.indexOf(tok, contextPath.length()) >= 0 ) {
                 return (false);
+            }
         }
 
         // This URL belongs to our web application, so it is encodeable
@@ -1575,14 +1638,32 @@ public class Response
      * @exception IllegalArgumentException if a MalformedURLException is
      *  thrown when converting the relative URL to an absolute one
      */
-    private String toAbsolute(String location) {
+    protected String toAbsolute(String location) {
 
-        if (location == null)
+        if (location == null) {
             return (location);
+        }
 
         boolean leadingSlash = location.startsWith("/");
 
-        if (leadingSlash || !hasScheme(location)) {
+        if (location.startsWith("//")) {
+            // Scheme relative
+            redirectURLCC.recycle();
+            // Add the scheme
+            String scheme = request.getScheme();
+                try {
+                redirectURLCC.append(scheme, 0, scheme.length());
+                redirectURLCC.append(':');
+                redirectURLCC.append(location, 0, location.length());
+                return redirectURLCC.toString();
+            } catch (IOException e) {
+                IllegalArgumentException iae =
+                    new IllegalArgumentException(location);
+                iae.initCause(e);
+                throw iae;
+            }
+
+        } else if (leadingSlash || !hasScheme(location)) {
 
             redirectURLCC.recycle();
 
@@ -1604,18 +1685,18 @@ public class Response
                     String relativePath = request.getDecodedRequestURI();
                     int pos = relativePath.lastIndexOf('/');
                     relativePath = relativePath.substring(0, pos);
-                    
+
                     String encodedURI = null;
                     final String frelativePath = relativePath;
                     if (SecurityUtil.isPackageProtectionEnabled() ){
                         try{
-                            encodedURI = AccessController.doPrivileged( 
-                                new PrivilegedExceptionAction<String>(){                                
+                            encodedURI = AccessController.doPrivileged(
+                                new PrivilegedExceptionAction<String>(){
                                     @Override
                                     public String run() throws IOException{
                                         return urlEncoder.encodeURL(frelativePath);
                                     }
-                           });   
+                           });
                         } catch (PrivilegedActionException pae){
                             IllegalArgumentException iae =
                                 new IllegalArgumentException(location);
@@ -1647,6 +1728,21 @@ public class Response
     }
 
 
+    /**
+     * Determine if an absolute URL has a path component
+     */
+    private boolean hasPath(String uri) {
+        int pos = uri.indexOf("://");
+        if (pos < 0) {
+            return false;
+        }
+        pos = uri.indexOf('/', pos + 3);
+        if (pos < 0) {
+            return false;
+        }
+        return true;
+    }
+
     /**
      * Determine if a URI string has a <code>scheme</code> component.
      */
@@ -1672,8 +1768,9 @@ public class Response
      */
     protected String toEncoded(String url, String sessionId) {
 
-        if ((url == null) || (sessionId == null))
+        if ((url == null) || (sessionId == null)) {
             return (url);
+        }
 
         String path = url;
         String query = "";
