@@ -20,24 +20,28 @@ package org.apache.catalina.core;
 
 
 import java.io.IOException;
+import java.security.AccessController;
+import java.security.PrivilegedAction;
 
+import javax.servlet.DispatcherType;
 import javax.servlet.RequestDispatcher;
 import javax.servlet.ServletContext;
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServletResponse;
 
-import org.apache.catalina.CometEvent;
 import org.apache.catalina.Context;
 import org.apache.catalina.Globals;
 import org.apache.catalina.Wrapper;
+import org.apache.catalina.comet.CometEvent;
 import org.apache.catalina.connector.ClientAbortException;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.Response;
 import org.apache.catalina.deploy.ErrorPage;
-import org.apache.catalina.util.StringManager;
 import org.apache.catalina.valves.ValveBase;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
@@ -49,14 +53,34 @@ import org.apache.juli.logging.LogFactory;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Revision: 781484 $ $Date: 2009-06-03 19:23:03 +0200 (Mi, 03. Jun 2009) $
+ * @version $Id: StandardHostValve.java 1164482 2011-09-02 11:33:05Z markt $
  */
 
-final class StandardHostValve
-    extends ValveBase {
+final class StandardHostValve extends ValveBase {
 
+    private static final Log log = LogFactory.getLog(StandardHostValve.class);
 
-    private static Log log = LogFactory.getLog(StandardHostValve.class);
+    protected static final boolean STRICT_SERVLET_COMPLIANCE;
+
+    protected static final boolean ACCESS_SESSION;
+
+    static {
+        STRICT_SERVLET_COMPLIANCE = Globals.STRICT_SERVLET_COMPLIANCE;
+        
+        String accessSession = System.getProperty(
+                "org.apache.catalina.core.StandardHostValve.ACCESS_SESSION");
+        if (accessSession == null) {
+            ACCESS_SESSION = STRICT_SERVLET_COMPLIANCE;
+        } else {
+            ACCESS_SESSION =
+                Boolean.valueOf(accessSession).booleanValue();
+        }
+    }
+
+    //------------------------------------------------------ Constructor
+    public StandardHostValve() {
+        super(true);
+    }
 
     // ----------------------------------------------------- Instance Variables
 
@@ -81,6 +105,7 @@ final class StandardHostValve
     /**
      * Return descriptive information about this Valve implementation.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -98,11 +123,11 @@ final class StandardHostValve
      *
      * @param request Request to be processed
      * @param response Response to be produced
-     * @param valveContext Valve context used to forward to the next Valve
      *
      * @exception IOException if an input/output error occurred
      * @exception ServletException if a servlet error occurred
      */
+    @Override
     public final void invoke(Request request, Response response)
         throws IOException, ServletException {
 
@@ -119,34 +144,75 @@ final class StandardHostValve
         if( context.getLoader() != null ) {
             // Not started - it should check for availability first
             // This should eventually move to Engine, it's generic.
-            Thread.currentThread().setContextClassLoader
-                    (context.getLoader().getClassLoader());
+            if (Globals.IS_SECURITY_ENABLED) {
+                PrivilegedAction<Void> pa = new PrivilegedSetTccl(
+                        context.getLoader().getClassLoader());
+                AccessController.doPrivileged(pa);                
+            } else {
+                Thread.currentThread().setContextClassLoader
+                        (context.getLoader().getClassLoader());
+            }
+        }
+        if (request.isAsyncSupported()) {
+            request.setAsyncSupported(context.getPipeline().isAsyncSupported());
         }
 
-        // Ask this Context to process this request
-        context.getPipeline().getFirst().invoke(request, response);
+        // Don't fire listeners during async processing
+        // If a request init listener throws an exception, the request is
+        // aborted
+        boolean asyncAtStart = request.isAsync(); 
+        if (asyncAtStart || context.fireRequestInitEvent(request)) {
+
+            // Ask this Context to process this request
+            try {
+                context.getPipeline().getFirst().invoke(request, response);
+            } catch (Throwable t) {
+                ExceptionUtils.handleThrowable(t);
+                request.setAttribute(RequestDispatcher.ERROR_EXCEPTION, t);
+                throwable(request, response, t);
+            }
+    
+            // If the request was async at the start and an error occurred then
+            // the async error handling will kick-in and that will fire the
+            // request destroyed event *after* the error handling has taken
+            // place
+            if (!(request.isAsync() || (asyncAtStart && request.getAttribute(
+                        RequestDispatcher.ERROR_EXCEPTION) != null))) {
+                // Protect against NPEs if context was destroyed during a long
+                // running request.
+                if (context.getState().isAvailable()) {
+                    // Error page processing
+                    response.setSuspended(false);
+    
+                    Throwable t = (Throwable) request.getAttribute(
+                            RequestDispatcher.ERROR_EXCEPTION);
+    
+                    if (t != null) {
+                        throwable(request, response, t);
+                    } else {
+                        status(request, response);
+                    }
+    
+                    context.fireRequestDestroyEvent(request);
+                }
+            }
+        }
 
         // Access a session (if present) to update last accessed time, based on a
         // strict interpretation of the specification
-        if (Globals.STRICT_SERVLET_COMPLIANCE) {
+        if (ACCESS_SESSION) {
             request.getSession(false);
         }
 
-        // Error page processing
-        response.setSuspended(false);
-
-        Throwable t = (Throwable) request.getAttribute(Globals.EXCEPTION_ATTR);
-
-        if (t != null) {
-            throwable(request, response, t);
-        } else {
-            status(request, response);
-        }
-
         // Restore the context classloader
-        Thread.currentThread().setContextClassLoader
-            (StandardHostValve.class.getClassLoader());
-
+        if (Globals.IS_SECURITY_ENABLED) {
+            PrivilegedAction<Void> pa = new PrivilegedSetTccl(
+                    StandardHostValve.class.getClassLoader());
+            AccessController.doPrivileged(pa);                
+        } else {
+            Thread.currentThread().setContextClassLoader
+                    (StandardHostValve.class.getClassLoader());
+        }
     }
 
 
@@ -155,11 +221,12 @@ final class StandardHostValve
      *
      * @param request Request to be processed
      * @param response Response to be produced
-     * @param valveContext Valve context used to forward to the next Valve
+     * @param event the event
      *
      * @exception IOException if an input/output error occurred
      * @exception ServletException if a servlet error occurred
      */
+    @Override
     public final void event(Request request, Response response, CometEvent event)
         throws IOException, ServletException {
 
@@ -177,16 +244,12 @@ final class StandardHostValve
         // Ask this Context to process this request
         context.getPipeline().getFirst().event(request, response, event);
 
-        // Access a session (if present) to update last accessed time, based on a
-        // strict interpretation of the specification
-        if (Globals.STRICT_SERVLET_COMPLIANCE) {
-            request.getSession(false);
-        }
-
+        
         // Error page processing
         response.setSuspended(false);
 
-        Throwable t = (Throwable) request.getAttribute(Globals.EXCEPTION_ATTR);
+        Throwable t = (Throwable) request.getAttribute(
+                RequestDispatcher.ERROR_EXCEPTION);
 
         if (t != null) {
             throwable(request, response, t);
@@ -194,6 +257,12 @@ final class StandardHostValve
             status(request, response);
         }
 
+        // Access a session (if present) to update last accessed time, based on a
+        // strict interpretation of the specification
+        if (ACCESS_SESSION) {
+            request.getSession(false);
+        }
+
         // Restore the context classloader
         Thread.currentThread().setContextClassLoader
             (StandardHostValve.class.getClassLoader());
@@ -201,7 +270,68 @@ final class StandardHostValve
     }
 
 
-    // ------------------------------------------------------ Protected Methods
+    // -------------------------------------------------------- Private Methods
+
+    /**
+     * Handle the HTTP status code (and corresponding message) generated
+     * while processing the specified Request to produce the specified
+     * Response.  Any exceptions that occur during generation of the error
+     * report are logged and swallowed.
+     *
+     * @param request The request being processed
+     * @param response The response being generated
+     */
+    private void status(Request request, Response response) {
+
+        int statusCode = response.getStatus();
+
+        // Handle a custom error page for this status code
+        Context context = request.getContext();
+        if (context == null)
+            return;
+
+        /* Only look for error pages when isError() is set.
+         * isError() is set when response.sendError() is invoked. This
+         * allows custom error pages without relying on default from
+         * web.xml.
+         */
+        if (!response.isError())
+            return;
+
+        ErrorPage errorPage = context.findErrorPage(statusCode);
+        if (errorPage != null) {
+            response.setAppCommitted(false);
+            request.setAttribute(RequestDispatcher.ERROR_STATUS_CODE,
+                              Integer.valueOf(statusCode));
+
+            String message = response.getMessage();
+            if (message == null)
+                message = "";
+            request.setAttribute(RequestDispatcher.ERROR_MESSAGE, message);
+            request.setAttribute
+                (ApplicationFilterFactory.DISPATCHER_REQUEST_PATH_ATTR,
+                 errorPage.getLocation());
+            request.setAttribute(ApplicationFilterFactory.DISPATCHER_TYPE_ATTR,
+                              DispatcherType.ERROR);
+
+
+            Wrapper wrapper = request.getWrapper();
+            if (wrapper != null)
+                request.setAttribute(RequestDispatcher.ERROR_SERVLET_NAME,
+                                  wrapper.getName());
+            request.setAttribute(RequestDispatcher.ERROR_REQUEST_URI,
+                                 request.getRequestURI());
+            if (custom(request, response, errorPage)) {
+                try {
+                    response.flushBuffer();
+                } catch (ClientAbortException e) {
+                    // Ignore
+                } catch (IOException e) {
+                    container.getLogger().warn("Exception Processing " + errorPage, e);
+                }
+            }
+        }
+    }
 
 
     /**
@@ -215,7 +345,7 @@ final class StandardHostValve
      * @param throwable The exception that occurred (which possibly wraps
      *  a root cause exception
      */
-    protected void throwable(Request request, Response response,
+    private void throwable(Request request, Response response,
                              Throwable throwable) {
         Context context = request.getContext();
         if (context == null)
@@ -251,21 +381,20 @@ final class StandardHostValve
                 (ApplicationFilterFactory.DISPATCHER_REQUEST_PATH_ATTR,
                  errorPage.getLocation());
             request.setAttribute(ApplicationFilterFactory.DISPATCHER_TYPE_ATTR,
-                              new Integer(ApplicationFilterFactory.ERROR));
-            request.setAttribute
-                (Globals.STATUS_CODE_ATTR,
-                 new Integer(HttpServletResponse.SC_INTERNAL_SERVER_ERROR));
-            request.setAttribute(Globals.ERROR_MESSAGE_ATTR,
+                              DispatcherType.ERROR);
+            request.setAttribute(RequestDispatcher.ERROR_STATUS_CODE,
+                    new Integer(HttpServletResponse.SC_INTERNAL_SERVER_ERROR));
+            request.setAttribute(RequestDispatcher.ERROR_MESSAGE,
                               throwable.getMessage());
-            request.setAttribute(Globals.EXCEPTION_ATTR,
+            request.setAttribute(RequestDispatcher.ERROR_EXCEPTION,
                               realError);
             Wrapper wrapper = request.getWrapper();
             if (wrapper != null)
-                request.setAttribute(Globals.SERVLET_NAME_ATTR,
+                request.setAttribute(RequestDispatcher.ERROR_SERVLET_NAME,
                                   wrapper.getName());
-            request.setAttribute(Globals.EXCEPTION_PAGE_ATTR,
+            request.setAttribute(RequestDispatcher.ERROR_REQUEST_URI,
                                  request.getRequestURI());
-            request.setAttribute(Globals.EXCEPTION_TYPE_ATTR,
+            request.setAttribute(RequestDispatcher.ERROR_EXCEPTION_TYPE,
                               realError.getClass());
             if (custom(request, response, errorPage)) {
                 try {
@@ -285,71 +414,60 @@ final class StandardHostValve
 
             status(request, response);
         }
-
-
     }
 
 
     /**
-     * Handle the HTTP status code (and corresponding message) generated
-     * while processing the specified Request to produce the specified
-     * Response.  Any exceptions that occur during generation of the error
-     * report are logged and swallowed.
+     * Handle an HTTP status code or Java exception by forwarding control
+     * to the location included in the specified errorPage object.  It is
+     * assumed that the caller has already recorded any request attributes
+     * that are to be forwarded to this page.  Return <code>true</code> if
+     * we successfully utilized the specified error page location, or
+     * <code>false</code> if the default error report should be rendered.
      *
      * @param request The request being processed
      * @param response The response being generated
+     * @param errorPage The errorPage directive we are obeying
      */
-    protected void status(Request request, Response response) {
+    private boolean custom(Request request, Response response,
+                             ErrorPage errorPage) {
 
-        int statusCode = response.getStatus();
+        if (container.getLogger().isDebugEnabled())
+            container.getLogger().debug("Processing " + errorPage);
 
-        // Handle a custom error page for this status code
-        Context context = request.getContext();
-        if (context == null)
-            return;
+        request.setPathInfo(errorPage.getLocation());
 
-        /* Only look for error pages when isError() is set.
-         * isError() is set when response.sendError() is invoked. This
-         * allows custom error pages without relying on default from
-         * web.xml.
-         */
-        if (!response.isError())
-            return;
+        try {
+            // Forward control to the specified location
+            ServletContext servletContext =
+                request.getContext().getServletContext();
+            RequestDispatcher rd =
+                servletContext.getRequestDispatcher(errorPage.getLocation());
 
-        ErrorPage errorPage = context.findErrorPage(statusCode);
-        if (errorPage != null) {
-            response.setAppCommitted(false);
-            request.setAttribute(Globals.STATUS_CODE_ATTR,
-                              new Integer(statusCode));
+            if (response.isCommitted()) {
+                // Response is committed - including the error page is the
+                // best we can do 
+                rd.include(request.getRequest(), response.getResponse());
+            } else {
+                // Reset the response (keeping the real error code and message)
+                response.resetBuffer(true);
 
-            String message = response.getMessage();
-            if (message == null)
-                message = "";
-            request.setAttribute(Globals.ERROR_MESSAGE_ATTR, message);
-            request.setAttribute
-                (ApplicationFilterFactory.DISPATCHER_REQUEST_PATH_ATTR,
-                 errorPage.getLocation());
-            request.setAttribute(ApplicationFilterFactory.DISPATCHER_TYPE_ATTR,
-                              new Integer(ApplicationFilterFactory.ERROR));
+                rd.forward(request.getRequest(), response.getResponse());
 
-
-            Wrapper wrapper = request.getWrapper();
-            if (wrapper != null)
-                request.setAttribute(Globals.SERVLET_NAME_ATTR,
-                                  wrapper.getName());
-            request.setAttribute(Globals.EXCEPTION_PAGE_ATTR,
-                                 request.getRequestURI());
-            if (custom(request, response, errorPage)) {
-                try {
-                    response.flushBuffer();
-                } catch (ClientAbortException e) {
-                    // Ignore
-                } catch (IOException e) {
-                    container.getLogger().warn("Exception Processing " + errorPage, e);
-                }
+                // If we forward, the response is suspended again
+                response.setSuspended(false);
             }
-        }
 
+            // Indicate that we have successfully processed this custom page
+            return (true);
+
+        } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
+            // Report our failure to process this custom page
+            container.getLogger().error("Exception Processing " + errorPage, t);
+            return (false);
+
+        }
     }
 
 
@@ -362,7 +480,7 @@ final class StandardHostValve
      * @param context The Context in which to search
      * @param exception The exception for which to find an ErrorPage
      */
-    protected static ErrorPage findErrorPage
+    private static ErrorPage findErrorPage
         (Context context, Throwable exception) {
 
         if (exception == null)
@@ -383,52 +501,18 @@ final class StandardHostValve
     }
 
 
-    /**
-     * Handle an HTTP status code or Java exception by forwarding control
-     * to the location included in the specified errorPage object.  It is
-     * assumed that the caller has already recorded any request attributes
-     * that are to be forwarded to this page.  Return <code>true</code> if
-     * we successfully utilized the specified error page location, or
-     * <code>false</code> if the default error report should be rendered.
-     *
-     * @param request The request being processed
-     * @param response The response being generated
-     * @param errorPage The errorPage directive we are obeying
-     */
-    protected boolean custom(Request request, Response response,
-                             ErrorPage errorPage) {
+    private static class PrivilegedSetTccl implements PrivilegedAction<Void> {
 
-        if (container.getLogger().isDebugEnabled())
-            container.getLogger().debug("Processing " + errorPage);
-
-        request.setPathInfo(errorPage.getLocation());
-
-        try {
-            // Reset the response (keeping the real error code and message)
-            response.resetBuffer(true);
-
-            // Forward control to the specified location
-            ServletContext servletContext =
-                request.getContext().getServletContext();
-            RequestDispatcher rd =
-                servletContext.getRequestDispatcher(errorPage.getLocation());
-            rd.forward(request.getRequest(), response.getResponse());
-
-            // If we forward, the response is suspended again
-            response.setSuspended(false);
-
-            // Indicate that we have successfully processed this custom page
-            return (true);
-
-        } catch (Throwable t) {
-
-            // Report our failure to process this custom page
-            container.getLogger().error("Exception Processing " + errorPage, t);
-            return (false);
+        private ClassLoader cl;
 
+        PrivilegedSetTccl(ClassLoader cl) {
+            this.cl = cl;
         }
 
+        @Override
+        public Void run() {
+            Thread.currentThread().setContextClassLoader(cl);
+            return null;
+        }
     }
-
-
 }
