@@ -14,11 +14,8 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.core;
 
-
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
 import java.io.IOException;
@@ -29,26 +26,24 @@ import java.net.Socket;
 import java.security.AccessControlException;
 import java.util.Random;
 
-import javax.management.MBeanRegistration;
-import javax.management.MBeanServer;
 import javax.management.ObjectName;
 
 import org.apache.catalina.Context;
-import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.LifecycleListener;
+import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Server;
-import org.apache.catalina.ServerFactory;
 import org.apache.catalina.Service;
 import org.apache.catalina.deploy.NamingResources;
-import org.apache.catalina.util.LifecycleSupport;
-import org.apache.catalina.util.StringManager;
+import org.apache.catalina.mbeans.MBeanFactory;
+import org.apache.catalina.mbeans.MBeanUtils;
+import org.apache.catalina.startup.Catalina;
+import org.apache.catalina.util.LifecycleMBeanBase;
 import org.apache.catalina.util.ServerInfo;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.buf.StringCache;
-import org.apache.tomcat.util.modeler.Registry;
-
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
@@ -56,24 +51,13 @@ import org.apache.tomcat.util.modeler.Registry;
  * (but not required) when deploying and starting Catalina.
  *
  * @author Craig R. McClanahan
- * @version $Id: StandardServer.java 939336 2010-04-29 15:00:41Z kkolinko $
+ * @version $Id: StandardServer.java 1079405 2011-03-08 15:58:53Z markt $
  */
-public final class StandardServer
-    implements Lifecycle, Server, MBeanRegistration 
- {
-    private static Log log = LogFactory.getLog(StandardServer.class);
+public final class StandardServer extends LifecycleMBeanBase implements Server {
+
+    private static final Log log = LogFactory.getLog(StandardServer.class);
    
 
-    // -------------------------------------------------------------- Constants
-
-
-    /**
-     * ServerLifecycleListener classname.
-     */
-    private static String SERVER_LISTENER_CLASS_NAME =
-        "org.apache.catalina.mbeans.ServerLifecycleListener";
-
-
     // ------------------------------------------------------------ Constructor
 
 
@@ -83,7 +67,6 @@ public final class StandardServer
     public StandardServer() {
 
         super();
-        ServerFactory.setServer(this);
 
         globalNamingResources = new NamingResources();
         globalNamingResources.setContainer(this);
@@ -120,12 +103,6 @@ public final class StandardServer
         "org.apache.catalina.core.StandardServer/1.0";
 
 
-    /**
-     * The lifecycle event support for this component.
-     */
-    private LifecycleSupport lifecycle = new LifecycleSupport(this);
-
-
     /**
      * The naming context listener for this web application.
      */
@@ -137,6 +114,11 @@ public final class StandardServer
      */
     private int port = 8005;
 
+    /**
+     * The address on which we wait for shutdown commands.
+     */
+    private String address = "localhost";
+
 
     /**
      * A random number generator that is <strong>only</strong> used if
@@ -164,24 +146,26 @@ public final class StandardServer
         StringManager.getManager(Constants.Package);
 
 
-    /**
-     * Has this component been started?
-     */
-    private boolean started = false;
-
-
-    /**
-     * Has this component been initialized?
-     */
-    private boolean initialized = false;
-
-
     /**
      * The property change support for this component.
      */
     protected PropertyChangeSupport support = new PropertyChangeSupport(this);
 
-    private boolean stopAwait = false;
+    private volatile boolean stopAwait = false;
+    
+    private Catalina catalina = null;
+
+    private ClassLoader parentClassLoader = null;
+
+    /**
+     * Thread that currently is inside our await() method.
+     */
+    private volatile Thread awaitThread = null;
+
+    /**
+     * Server socket that is used to wait for the shutdown command.
+     */
+    private volatile ServerSocket awaitSocket = null;
 
     // ------------------------------------------------------------- Properties
 
@@ -189,6 +173,7 @@ public final class StandardServer
     /**
      * Return the global naming resources context.
      */
+    @Override
     public javax.naming.Context getGlobalNamingContext() {
 
         return (this.globalNamingContext);
@@ -212,6 +197,7 @@ public final class StandardServer
     /**
      * Return the global naming resources.
      */
+    @Override
     public NamingResources getGlobalNamingResources() {
 
         return (this.globalNamingResources);
@@ -224,6 +210,7 @@ public final class StandardServer
      *
      * @param globalNamingResources The new global naming resources
      */
+    @Override
     public void setGlobalNamingResources
         (NamingResources globalNamingResources) {
 
@@ -243,6 +230,7 @@ public final class StandardServer
      * the corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -261,6 +249,7 @@ public final class StandardServer
     /**
      * Return the port number we listen to for shutdown commands.
      */
+    @Override
     public int getPort() {
 
         return (this.port);
@@ -273,6 +262,7 @@ public final class StandardServer
      *
      * @param port The new port number
      */
+    @Override
     public void setPort(int port) {
 
         this.port = port;
@@ -280,9 +270,33 @@ public final class StandardServer
     }
 
 
+    /**
+     * Return the address on which we listen to for shutdown commands.
+     */
+    @Override
+    public String getAddress() {
+
+        return (this.address);
+
+    }
+
+
+    /**
+     * Set the address on which we listen to for shutdown commands.
+     *
+     * @param address The new address
+     */
+    @Override
+    public void setAddress(String address) {
+
+        this.address = address;
+
+    }
+
     /**
      * Return the shutdown command string we are waiting for.
      */
+    @Override
     public String getShutdown() {
 
         return (this.shutdown);
@@ -295,6 +309,7 @@ public final class StandardServer
      *
      * @param shutdown The new shutdown command
      */
+    @Override
     public void setShutdown(String shutdown) {
 
         this.shutdown = shutdown;
@@ -302,6 +317,23 @@ public final class StandardServer
     }
 
 
+    /**
+     * Return the outer Catalina startup/shutdown component if present.
+     */
+    @Override
+    public Catalina getCatalina() {
+        return catalina;
+    }
+    
+    
+    /**
+     * Set the outer Catalina startup/shutdown component if present.
+     */
+    @Override
+    public void setCatalina(Catalina catalina) {
+        this.catalina = catalina;
+    }
+    
     // --------------------------------------------------------- Server Methods
 
 
@@ -310,6 +342,7 @@ public final class StandardServer
      *
      * @param service The Service to be added
      */
+    @Override
     public void addService(Service service) {
 
         service.setServer(this);
@@ -320,19 +353,11 @@ public final class StandardServer
             results[services.length] = service;
             services = results;
 
-            if (initialized) {
+            if (getState().isAvailable()) {
                 try {
-                    service.initialize();
+                    service.start();
                 } catch (LifecycleException e) {
-                    log.error(e);
-                }
-            }
-
-            if (started && (service instanceof Lifecycle)) {
-                try {
-                    ((Lifecycle) service).start();
-                } catch (LifecycleException e) {
-                    ;
+                    // Ignore
                 }
             }
 
@@ -344,6 +369,24 @@ public final class StandardServer
 
     public void stopAwait() {
         stopAwait=true;
+        Thread t = awaitThread;
+        if (t != null) {
+            ServerSocket s = awaitSocket;
+            if (s != null) {
+                awaitSocket = null;
+                try {
+                    s.close();
+                } catch (IOException e) {
+                    // Ignored
+                }
+            }
+            t.interrupt();
+            try {
+                t.join(1000);
+            } catch (InterruptedException e) {
+                // Ignored
+            }
+        }
     }
 
     /**
@@ -351,6 +394,7 @@ public final class StandardServer
      * This keeps the main thread alive - the thread pool listening for http 
      * connections is daemon threads.
      */
+    @Override
     public void await() {
         // Negative values - don't wait on port - tomcat is embedded or we just don't like ports
         if( port == -2 ) {
@@ -358,92 +402,118 @@ public final class StandardServer
             return;
         }
         if( port==-1 ) {
-            while( true ) {
-                try {
-                    Thread.sleep( 10000 );
-                } catch( InterruptedException ex ) {
+            try {
+                awaitThread = Thread.currentThread();
+                while(!stopAwait) {
+                    try {
+                        Thread.sleep( 10000 );
+                    } catch( InterruptedException ex ) {
+                        // continue and check the flag
+                    }
                 }
-                if( stopAwait ) return;
+            } finally {
+                awaitThread = null;
             }
+            return;
         }
-        
+
         // Set up a server socket to wait on
-        ServerSocket serverSocket = null;
         try {
-            serverSocket =
-                new ServerSocket(port, 1,
-                                 InetAddress.getByName("localhost"));
+            awaitSocket = new ServerSocket(port, 1,
+                    InetAddress.getByName(address));
         } catch (IOException e) {
-            log.error("StandardServer.await: create[" + port
+            log.error("StandardServer.await: create[" + address
+                               + ":" + port
                                + "]: ", e);
-            System.exit(1);
+            return;
         }
 
-        // Loop waiting for a connection and a valid command
-        while (true) {
-
-            // Wait for the next connection
-            Socket socket = null;
-            InputStream stream = null;
-            try {
-                socket = serverSocket.accept();
-                socket.setSoTimeout(10 * 1000);  // Ten seconds
-                stream = socket.getInputStream();
-            } catch (AccessControlException ace) {
-                log.warn("StandardServer.accept security exception: "
-                                   + ace.getMessage(), ace);
-                continue;
-            } catch (IOException e) {
-                log.error("StandardServer.await: accept: ", e);
-                System.exit(1);
-            }
-
-            // Read a set of characters from the socket
-            StringBuffer command = new StringBuffer();
-            int expected = 1024; // Cut off to avoid DoS attack
-            while (expected < shutdown.length()) {
-                if (random == null)
-                    random = new Random();
-                expected += (random.nextInt() % 1024);
-            }
-            while (expected > 0) {
-                int ch = -1;
-                try {
-                    ch = stream.read();
-                } catch (IOException e) {
-                    log.warn("StandardServer.await: read: ", e);
-                    ch = -1;
-                }
-                if (ch < 32)  // Control character or EOF terminates loop
-                    break;
-                command.append((char) ch);
-                expected--;
-            }
-
-            // Close the socket now that we are done with it
-            try {
-                socket.close();
-            } catch (IOException e) {
-                ;
-            }
-
-            // Match against our command string
-            boolean match = command.toString().equals(shutdown);
-            if (match) {
-                break;
-            } else
-                log.warn("StandardServer.await: Invalid command '" +
-                                   command.toString() + "' received");
-
-        }
-
-        // Close the server socket and return
         try {
-            serverSocket.close();
-        } catch (IOException e) {
-            ;
-        }
+            awaitThread = Thread.currentThread();
 
+            // Loop waiting for a connection and a valid command
+            while (!stopAwait) {
+                ServerSocket serverSocket = awaitSocket;
+                if (serverSocket == null) {
+                    break;
+                }
+    
+                // Wait for the next connection
+                Socket socket = null;
+                StringBuilder command = new StringBuilder();
+                try {
+                    InputStream stream;
+                    try {
+                        socket = serverSocket.accept();
+                        socket.setSoTimeout(10 * 1000);  // Ten seconds
+                        stream = socket.getInputStream();
+                    } catch (AccessControlException ace) {
+                        log.warn("StandardServer.accept security exception: "
+                                + ace.getMessage(), ace);
+                        continue;
+                    } catch (IOException e) {
+                        if (stopAwait) {
+                            // Wait was aborted with socket.close()
+                            break;
+                        }
+                        log.error("StandardServer.await: accept: ", e);
+                        break;
+                    }
+
+                    // Read a set of characters from the socket
+                    int expected = 1024; // Cut off to avoid DoS attack
+                    while (expected < shutdown.length()) {
+                        if (random == null)
+                            random = new Random();
+                        expected += (random.nextInt() % 1024);
+                    }
+                    while (expected > 0) {
+                        int ch = -1;
+                        try {
+                            ch = stream.read();
+                        } catch (IOException e) {
+                            log.warn("StandardServer.await: read: ", e);
+                            ch = -1;
+                        }
+                        if (ch < 32)  // Control character or EOF terminates loop
+                            break;
+                        command.append((char) ch);
+                        expected--;
+                    }
+                } finally {
+                    // Close the socket now that we are done with it
+                    try {
+                        if (socket != null) {
+                            socket.close();
+                        }
+                    } catch (IOException e) {
+                        // Ignore
+                    }
+                }
+
+                // Match against our command string
+                boolean match = command.toString().equals(shutdown);
+                if (match) {
+                    log.info(sm.getString("standardServer.shutdownViaPort"));
+                    break;
+                } else
+                    log.warn("StandardServer.await: Invalid command '"
+                            + command.toString() + "' received");
+            }
+        } finally {
+            ServerSocket serverSocket = awaitSocket;
+            awaitThread = null;
+            awaitSocket = null;
+
+            // Close the server socket and return
+            if (serverSocket != null) {
+                try {
+                    serverSocket.close();
+                } catch (IOException e) {
+                    // Ignore
+                }
+            }
+        }
     }
 
 
@@ -453,6 +523,7 @@ public final class StandardServer
      *
      * @param name Name of the Service to be returned
      */
+    @Override
     public Service findService(String name) {
 
         if (name == null) {
@@ -473,6 +544,7 @@ public final class StandardServer
     /**
      * Return the set of Services defined within this Server.
      */
+    @Override
     public Service[] findServices() {
 
         return (services);
@@ -497,6 +569,7 @@ public final class StandardServer
      *
      * @param service The Service to be removed
      */
+    @Override
     public void removeService(Service service) {
 
         synchronized (services) {
@@ -509,12 +582,10 @@ public final class StandardServer
             }
             if (j < 0)
                 return;
-            if (services[j] instanceof Lifecycle) {
-                try {
-                    ((Lifecycle) services[j]).stop();
-                } catch (LifecycleException e) {
-                    ;
-                }
+            try {
+                services[j].stop();
+            } catch (LifecycleException e) {
+                // Ignore
             }
             int k = 0;
             Service results[] = new Service[services.length - 1];
@@ -561,9 +632,10 @@ public final class StandardServer
     /**
      * Return a String representation of this component.
      */
+    @Override
     public String toString() {
 
-        StringBuffer sb = new StringBuffer("StandardServer[");
+        StringBuilder sb = new StringBuilder("StandardServer[");
         sb.append(getPort());
         sb.append("]");
         return (sb.toString());
@@ -575,26 +647,17 @@ public final class StandardServer
      * Write the configuration information for this entire <code>Server</code>
      * out to the server.xml configuration file.
      *
-     * @exception javax.management.InstanceNotFoundException if the managed resource object
-     *  cannot be found
-     * @exception javax.management.MBeanException if the initializer of the object throws
-     *  an exception, or persistence is not supported
-     * @exception javax.management.RuntimeOperationsException if an exception is reported
-     *  by the persistence mechanism
+     * @exception   javax.management.InstanceNotFoundException
+     *              if the managed resource object cannot be found
+     * @exception   javax.management.MBeanException
+     *              if the initializer of the object throws an exception, or
+     *              persistence is not supported
+     * @exception   javax.management.RuntimeOperationsException
+     *              if an exception is reported by the persistence mechanism
      */
     public synchronized void storeConfig() throws Exception {
-
-        ObjectName sname = null;    
-        try {
-           sname = new ObjectName("Catalina:type=StoreConfig");
-           if(mserver.isRegistered(sname)) {
-               mserver.invoke(sname, "storeConfig", null, null);            
-           } else
-               log.error("StoreConfig mbean not registered" + sname);
-        } catch (Throwable t) {
-            log.error(t);
-        }
-
+        ObjectName sname = new ObjectName("Catalina:type=StoreConfig");
+        mserver.invoke(sname, "storeConfig", null, null);            
     }
 
 
@@ -621,6 +684,7 @@ public final class StandardServer
            } else
                log.error("StoreConfig mbean not registered" + sname);
         } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
             log.error(t);
         }
  
@@ -642,189 +706,155 @@ public final class StandardServer
     }
 
 
-    // ------------------------------------------------------ Lifecycle Methods
-
-
     /**
-     * Add a LifecycleEvent listener to this component.
-     *
-     * @param listener The listener to add
-     */
-    public void addLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.addLifecycleListener(listener);
-
-    }
-
-
-    /**
-     * Get the lifecycle listeners associated with this lifecycle. If this
-     * Lifecycle has no listeners registered, a zero-length array is returned.
-     */
-    public LifecycleListener[] findLifecycleListeners() {
-
-        return lifecycle.findLifecycleListeners();
-
-    }
-
-
-    /**
-     * Remove a LifecycleEvent listener from this component.
-     *
-     * @param listener The listener to remove
-     */
-    public void removeLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.removeLifecycleListener(listener);
-
-    }
-
-
-    /**
-     * Prepare for the beginning of active use of the public methods of this
-     * component.  This method should be called before any of the public
-     * methods of this component are utilized.  It should also send a
-     * LifecycleEvent of type START_EVENT to any registered listeners.
+     * Start nested components ({@link Service}s) and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
      *  that prevents this component from being used
      */
-    public void start() throws LifecycleException {
+    @Override
+    protected void startInternal() throws LifecycleException {
 
-        // Validate and update our current component state
-        if (started) {
-            log.debug(sm.getString("standardServer.start.started"));
-            return;
-        }
-
-        // Notify our interested LifecycleListeners
-        lifecycle.fireLifecycleEvent(BEFORE_START_EVENT, null);
-
-        lifecycle.fireLifecycleEvent(START_EVENT, null);
-        started = true;
+        fireLifecycleEvent(CONFIGURE_START_EVENT, null);
+        setState(LifecycleState.STARTING);
 
+        globalNamingResources.start();
+        
         // Start our defined Services
         synchronized (services) {
             for (int i = 0; i < services.length; i++) {
-                if (services[i] instanceof Lifecycle)
-                    ((Lifecycle) services[i]).start();
+                services[i].start();
             }
         }
-
-        // Notify our interested LifecycleListeners
-        lifecycle.fireLifecycleEvent(AFTER_START_EVENT, null);
-
     }
 
 
     /**
-     * Gracefully terminate the active use of the public methods of this
-     * component.  This method should be the last one called on a given
-     * instance of this component.  It should also send a LifecycleEvent
-     * of type STOP_EVENT to any registered listeners.
+     * Stop nested components ({@link Service}s) and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
      *
      * @exception LifecycleException if this component detects a fatal error
      *  that needs to be reported
      */
-    public void stop() throws LifecycleException {
-
-        // Validate and update our current component state
-        if (!started)
-            return;
-
-        // Notify our interested LifecycleListeners
-        lifecycle.fireLifecycleEvent(BEFORE_STOP_EVENT, null);
-
-        lifecycle.fireLifecycleEvent(STOP_EVENT, null);
-        started = false;
+    @Override
+    protected void stopInternal() throws LifecycleException {
 
+        setState(LifecycleState.STOPPING);
+        fireLifecycleEvent(CONFIGURE_STOP_EVENT, null);
+        
         // Stop our defined Services
         for (int i = 0; i < services.length; i++) {
-            if (services[i] instanceof Lifecycle)
-                ((Lifecycle) services[i]).stop();
+            services[i].stop();
         }
 
-        // Notify our interested LifecycleListeners
-        lifecycle.fireLifecycleEvent(AFTER_STOP_EVENT, null);
-
-        if (port == -1)
-            stopAwait();
-
+        globalNamingResources.stop();
+        
+        stopAwait();
     }
 
-    public void init() throws Exception {
-        initialize();
-    }
-    
     /**
      * Invoke a pre-startup initialization. This is used to allow connectors
      * to bind to restricted ports under Unix operating environments.
      */
-    public void initialize()
-        throws LifecycleException 
-    {
-        if (initialized) {
-                log.info(sm.getString("standardServer.initialize.initialized"));
-            return;
-        }
-        lifecycle.fireLifecycleEvent(INIT_EVENT, null);
-        initialized = true;
-
-        if( oname==null ) {
-            try {
-                oname=new ObjectName( "Catalina:type=Server");
-                Registry.getRegistry(null, null)
-                    .registerComponent(this, oname, null );
-            } catch (Exception e) {
-                log.error("Error registering ",e);
-            }
-        }
+    @Override
+    protected void initInternal() throws LifecycleException {
         
-        // Register global String cache
-        try {
-            ObjectName oname2 = 
-                new ObjectName(oname.getDomain() + ":type=StringCache");
-            Registry.getRegistry(null, null)
-                .registerComponent(new StringCache(), oname2, null );
-        } catch (Exception e) {
-            log.error("Error registering ",e);
-        }
+        super.initInternal();
 
+        // Register global String cache
+        // Note although the cache is global, if there are multiple Servers
+        // present in the JVM (may happen when embedding) then the same cache
+        // will be registered under multiple names
+        onameStringCache = register(new StringCache(), "type=StringCache");
+
+        // Register the MBeanFactory
+        MBeanFactory factory = new MBeanFactory();
+        factory.setContainer(this);
+        onameMBeanFactory = register(factory, "type=MBeanFactory");
+        
+        // Register the naming resources
+        globalNamingResources.init();
+        
         // Initialize our defined Services
         for (int i = 0; i < services.length; i++) {
-            services[i].initialize();
+            services[i].init();
         }
     }
     
-    protected String type;
-    protected String domain;
-    protected String suffix;
-    protected ObjectName oname;
-    protected MBeanServer mserver;
+    @Override
+    protected void destroyInternal() throws LifecycleException {
+        // Destroy our defined Services
+        for (int i = 0; i < services.length; i++) {
+            services[i].destroy();
+        }
 
-    public ObjectName getObjectName() {
-        return oname;
+        globalNamingResources.destroy();
+        
+        unregister(onameMBeanFactory);
+        
+        unregister(onameStringCache);
+                
+        super.destroyInternal();
     }
 
-    public String getDomain() {
+    /**
+     * Return the parent class loader for this component.
+     */
+    @Override
+    public ClassLoader getParentClassLoader() {
+        if (parentClassLoader != null)
+            return (parentClassLoader);
+        if (catalina != null) {
+            return (catalina.getParentClassLoader());
+        }
+        return (ClassLoader.getSystemClassLoader());
+    }
+
+    /**
+     * Set the parent class loader for this server.
+     *
+     * @param parent The new parent class loader
+     */
+    @Override
+    public void setParentClassLoader(ClassLoader parent) {
+        ClassLoader oldParentClassLoader = this.parentClassLoader;
+        this.parentClassLoader = parent;
+        support.firePropertyChange("parentClassLoader", oldParentClassLoader,
+                                   this.parentClassLoader);
+    }
+
+    
+    private ObjectName onameStringCache;
+    private ObjectName onameMBeanFactory;
+    
+    /**
+     * Obtain the MBean domain for this server. The domain is obtained using
+     * the following search order:
+     * <ol>
+     * <li>Name of first {@link org.apache.catalina.Engine}.</li>
+     * <li>Name of first {@link Service}.</li>
+     * </ol>
+     */
+    @Override
+    protected String getDomainInternal() {
+        
+        String domain = null;
+        
+        Service[] services = findServices();
+        if (services.length > 0) {
+            Service service = services[0];
+            if (service != null) {
+                domain = MBeanUtils.getDomain(service);
+            }
+        }
         return domain;
     }
 
-    public ObjectName preRegister(MBeanServer server,
-                                  ObjectName name) throws Exception {
-        oname=name;
-        mserver=server;
-        domain=name.getDomain();
-        return name;
-    }
-
-    public void postRegister(Boolean registrationDone) {
-    }
-
-    public void preDeregister() throws Exception {
-    }
-
-    public void postDeregister() {
-    }
     
+    @Override
+    protected final String getObjectNameKeyProperties() {
+        return "type=Server";
+    }
+
 }
