@@ -21,11 +21,27 @@ package org.apache.catalina.deploy;
 
 import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
+import java.io.Serializable;
+import java.lang.reflect.InvocationTargetException;
+import java.lang.reflect.Method;
 import java.util.HashMap;
 import java.util.Hashtable;
-import java.io.Serializable;
 
-import org.apache.catalina.ServerFactory;
+import javax.naming.NamingException;
+
+import org.apache.catalina.Container;
+import org.apache.catalina.Context;
+import org.apache.catalina.Engine;
+import org.apache.catalina.LifecycleException;
+import org.apache.catalina.LifecycleState;
+import org.apache.catalina.Server;
+import org.apache.catalina.mbeans.MBeanUtils;
+import org.apache.catalina.util.LifecycleMBeanBase;
+import org.apache.juli.logging.Log;
+import org.apache.juli.logging.LogFactory;
+import org.apache.naming.ContextBindings;
+import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
@@ -33,11 +49,19 @@ import org.apache.catalina.ServerFactory;
  * Naming Context and their associated JNDI context.
  *
  * @author Remy Maucherat
- * @version $Revision: 652623 $ $Date: 2008-05-01 20:58:16 +0200 (Do, 01. Mai 2008) $
+ * @version $Id: NamingResources.java 1201569 2011-11-14 01:36:07Z kkolinko $
  */
 
-public class NamingResources implements Serializable {
+public class NamingResources extends LifecycleMBeanBase implements Serializable {
 
+    private static final long serialVersionUID = 1L;
+    
+    private static final Log log = LogFactory.getLog(NamingResources.class);
+    
+    private static final StringManager sm =
+        StringManager.getManager(Constants.Package);
+
+    private volatile boolean resourceRequireExplicitRegistration = false;
 
     // ----------------------------------------------------------- Constructors
 
@@ -46,6 +70,7 @@ public class NamingResources implements Serializable {
      * Create a new NamingResources instance.
      */
     public NamingResources() {
+        // NOOP
     }
 
 
@@ -62,58 +87,67 @@ public class NamingResources implements Serializable {
      * List of naming entries, keyed by name. The value is the entry type, as
      * declared by the user.
      */
-    private Hashtable entries = new Hashtable();
+    private Hashtable<String, String> entries =
+        new Hashtable<String, String>();
 
 
     /**
      * The EJB resource references for this web application, keyed by name.
      */
-    private HashMap ejbs = new HashMap();
+    private HashMap<String, ContextEjb> ejbs =
+        new HashMap<String, ContextEjb>();
 
 
     /**
      * The environment entries for this web application, keyed by name.
      */
-    private HashMap envs = new HashMap();
+    private HashMap<String, ContextEnvironment> envs =
+        new HashMap<String, ContextEnvironment>();
 
 
     /**
      * The local  EJB resource references for this web application, keyed by
      * name.
      */
-    private HashMap localEjbs = new HashMap();
+    private HashMap<String, ContextLocalEjb> localEjbs =
+        new HashMap<String, ContextLocalEjb>();
 
 
     /**
      * The message destination referencess for this web application,
      * keyed by name.
      */
-    private HashMap mdrs = new HashMap();
+    private HashMap<String, MessageDestinationRef> mdrs =
+        new HashMap<String, MessageDestinationRef>();
 
 
     /**
      * The resource environment references for this web application,
      * keyed by name.
      */
-    private HashMap resourceEnvRefs = new HashMap();
+    private HashMap<String, ContextResourceEnvRef> resourceEnvRefs =
+        new HashMap<String, ContextResourceEnvRef>();
 
 
     /**
      * The resource references for this web application, keyed by name.
      */
-    private HashMap resources = new HashMap();
+    private HashMap<String, ContextResource> resources =
+        new HashMap<String, ContextResource>();
 
 
     /**
      * The resource links for this web application, keyed by name.
      */
-    private HashMap resourceLinks = new HashMap();
+    private HashMap<String, ContextResourceLink> resourceLinks =
+        new HashMap<String, ContextResourceLink>();
 
 
     /**
      * The web service references for this web application, keyed by name.
      */
-    private HashMap services = new HashMap();
+    private HashMap<String, ContextService> services =
+        new HashMap<String, ContextService>();
 
 
     /**
@@ -203,8 +237,7 @@ public class NamingResources implements Serializable {
                 }
             } else if (rl != null) {
                 // Link. Need to look at the global resources
-                NamingResources global =
-                    ServerFactory.getServer().getGlobalNamingResources();
+                NamingResources global = getServer().getGlobalNamingResources();
                 if (global.findEnvironment(rl.getGlobal()) != null) {
                     if (global.findEnvironment(rl.getGlobal()).getOverride()) {
                         removeResourceLink(environment.getName());
@@ -226,8 +259,31 @@ public class NamingResources implements Serializable {
         }
         support.firePropertyChange("environment", null, environment);
 
+        // Register with JMX
+        if (resourceRequireExplicitRegistration) {
+            try {
+                MBeanUtils.createMBean(environment);
+            } catch (Exception e) {
+                log.warn(sm.getString("namingResources.mbeanCreateFail",
+                        environment.getName()), e);
+            }
+        }
     }
 
+    // Container should be an instance of Server or Context. If it is anything
+    // else, return null which will trigger a NPE.
+    private Server getServer() {
+        if (container instanceof Server) {
+            return (Server) container;
+        }
+        if (container instanceof Context) {
+            // Could do this in one go. Lots of casts so split out for clarity
+            Engine engine =
+                (Engine) ((Context) container).getParent().getParent();
+            return engine.getService().getServer();
+        }
+        return null;
+    }
 
     /**
      * Add a local EJB resource reference for this web application.
@@ -304,6 +360,15 @@ public class NamingResources implements Serializable {
         }
         support.firePropertyChange("resource", null, resource);
 
+        // Register with JMX
+        if (resourceRequireExplicitRegistration) {
+            try {
+                MBeanUtils.createMBean(resource);
+            } catch (Exception e) {
+                log.warn(sm.getString("namingResources.mbeanCreateFail",
+                        resource.getName()), e);
+            }
+        }
     }
 
 
@@ -339,7 +404,7 @@ public class NamingResources implements Serializable {
         if (entries.containsKey(resourceLink.getName())) {
             return;
         } else {
-            Object value = resourceLink.getType();
+            String value = resourceLink.getType();
             if (value == null) {
                 value = "";
             }
@@ -352,6 +417,15 @@ public class NamingResources implements Serializable {
         }
         support.firePropertyChange("resourceLink", null, resourceLink);
 
+        // Register with JMX
+        if (resourceRequireExplicitRegistration) {
+            try {
+                MBeanUtils.createMBean(resourceLink);
+            } catch (Exception e) {
+                log.warn(sm.getString("namingResources.mbeanCreateFail",
+                        resourceLink.getName()), e);
+            }
+        }
     }
 
 
@@ -365,7 +439,7 @@ public class NamingResources implements Serializable {
         if (entries.containsKey(service.getName())) {
             return;
         } else {
-            Object value = service.getType();
+            String value = service.getType();
             if (value == null) {
                 value = "";
             }
@@ -390,7 +464,7 @@ public class NamingResources implements Serializable {
     public ContextEjb findEjb(String name) {
 
         synchronized (ejbs) {
-            return ((ContextEjb) ejbs.get(name));
+            return ejbs.get(name);
         }
 
     }
@@ -404,7 +478,7 @@ public class NamingResources implements Serializable {
 
         synchronized (ejbs) {
             ContextEjb results[] = new ContextEjb[ejbs.size()];
-            return ((ContextEjb[]) ejbs.values().toArray(results));
+            return ejbs.values().toArray(results);
         }
 
     }
@@ -419,7 +493,7 @@ public class NamingResources implements Serializable {
     public ContextEnvironment findEnvironment(String name) {
 
         synchronized (envs) {
-            return ((ContextEnvironment) envs.get(name));
+            return envs.get(name);
         }
 
     }
@@ -434,7 +508,7 @@ public class NamingResources implements Serializable {
 
         synchronized (envs) {
             ContextEnvironment results[] = new ContextEnvironment[envs.size()];
-            return ((ContextEnvironment[]) envs.values().toArray(results));
+            return envs.values().toArray(results);
         }
 
     }
@@ -449,7 +523,7 @@ public class NamingResources implements Serializable {
     public ContextLocalEjb findLocalEjb(String name) {
 
         synchronized (localEjbs) {
-            return ((ContextLocalEjb) localEjbs.get(name));
+            return localEjbs.get(name);
         }
 
     }
@@ -463,7 +537,7 @@ public class NamingResources implements Serializable {
 
         synchronized (localEjbs) {
             ContextLocalEjb results[] = new ContextLocalEjb[localEjbs.size()];
-            return ((ContextLocalEjb[]) localEjbs.values().toArray(results));
+            return localEjbs.values().toArray(results);
         }
 
     }
@@ -478,7 +552,7 @@ public class NamingResources implements Serializable {
     public MessageDestinationRef findMessageDestinationRef(String name) {
 
         synchronized (mdrs) {
-            return ((MessageDestinationRef) mdrs.get(name));
+            return mdrs.get(name);
         }
 
     }
@@ -493,7 +567,7 @@ public class NamingResources implements Serializable {
         synchronized (mdrs) {
             MessageDestinationRef results[] =
                 new MessageDestinationRef[mdrs.size()];
-            return ((MessageDestinationRef[]) mdrs.values().toArray(results));
+            return mdrs.values().toArray(results);
         }
 
     }
@@ -508,7 +582,7 @@ public class NamingResources implements Serializable {
     public ContextResource findResource(String name) {
 
         synchronized (resources) {
-            return ((ContextResource) resources.get(name));
+            return resources.get(name);
         }
 
     }
@@ -523,7 +597,7 @@ public class NamingResources implements Serializable {
     public ContextResourceLink findResourceLink(String name) {
 
         synchronized (resourceLinks) {
-            return ((ContextResourceLink) resourceLinks.get(name));
+            return resourceLinks.get(name);
         }
 
     }
@@ -538,8 +612,7 @@ public class NamingResources implements Serializable {
         synchronized (resourceLinks) {
             ContextResourceLink results[] = 
                 new ContextResourceLink[resourceLinks.size()];
-            return ((ContextResourceLink[]) resourceLinks.values()
-                    .toArray(results));
+            return resourceLinks.values().toArray(results);
         }
 
     }
@@ -553,7 +626,7 @@ public class NamingResources implements Serializable {
 
         synchronized (resources) {
             ContextResource results[] = new ContextResource[resources.size()];
-            return ((ContextResource[]) resources.values().toArray(results));
+            return resources.values().toArray(results);
         }
 
     }
@@ -568,7 +641,7 @@ public class NamingResources implements Serializable {
     public ContextResourceEnvRef findResourceEnvRef(String name) {
 
         synchronized (resourceEnvRefs) {
-            return ((ContextResourceEnvRef) resourceEnvRefs.get(name));
+            return resourceEnvRefs.get(name);
         }
 
     }
@@ -583,7 +656,7 @@ public class NamingResources implements Serializable {
 
         synchronized (resourceEnvRefs) {
             ContextResourceEnvRef results[] = new ContextResourceEnvRef[resourceEnvRefs.size()];
-            return ((ContextResourceEnvRef[]) resourceEnvRefs.values().toArray(results));
+            return resourceEnvRefs.values().toArray(results);
         }
 
     }
@@ -598,7 +671,7 @@ public class NamingResources implements Serializable {
     public ContextService findService(String name) {
 
         synchronized (services) {
-            return ((ContextService) services.get(name));
+            return services.get(name);
         }
 
     }
@@ -612,7 +685,7 @@ public class NamingResources implements Serializable {
         
         synchronized (services) {
             ContextService results[] = new ContextService[services.size()];
-            return ((ContextService[]) services.values().toArray(results));
+            return services.values().toArray(results);
         }
         
     }
@@ -621,6 +694,7 @@ public class NamingResources implements Serializable {
     /**
      * Return true if the name specified already exists.
      */
+    @Deprecated
     public boolean exists(String name) {
 
         return (entries.containsKey(name));
@@ -639,7 +713,7 @@ public class NamingResources implements Serializable {
 
         ContextEjb ejb = null;
         synchronized (ejbs) {
-            ejb = (ContextEjb) ejbs.remove(name);
+            ejb = ejbs.remove(name);
         }
         if (ejb != null) {
             support.firePropertyChange("ejb", ejb, null);
@@ -660,13 +734,21 @@ public class NamingResources implements Serializable {
 
         ContextEnvironment environment = null;
         synchronized (envs) {
-            environment = (ContextEnvironment) envs.remove(name);
+            environment = envs.remove(name);
         }
         if (environment != null) {
             support.firePropertyChange("environment", environment, null);
+            // De-register with JMX
+            if (resourceRequireExplicitRegistration) {
+                try {
+                    MBeanUtils.destroyMBean(environment);
+                } catch (Exception e) {
+                    log.warn(sm.getString("namingResources.mbeanDestroyFail",
+                            environment.getName()), e);
+                }
+            }
             environment.setNamingResources(null);
         }
-
     }
 
 
@@ -681,7 +763,7 @@ public class NamingResources implements Serializable {
 
         ContextLocalEjb localEjb = null;
         synchronized (localEjbs) {
-            localEjb = (ContextLocalEjb) localEjbs.remove(name);
+            localEjb = localEjbs.remove(name);
         }
         if (localEjb != null) {
             support.firePropertyChange("localEjb", localEjb, null);
@@ -702,7 +784,7 @@ public class NamingResources implements Serializable {
 
         MessageDestinationRef mdr = null;
         synchronized (mdrs) {
-            mdr = (MessageDestinationRef) mdrs.remove(name);
+            mdr = mdrs.remove(name);
         }
         if (mdr != null) {
             support.firePropertyChange("messageDestinationRef",
@@ -736,13 +818,21 @@ public class NamingResources implements Serializable {
 
         ContextResource resource = null;
         synchronized (resources) {
-            resource = (ContextResource) resources.remove(name);
+            resource = resources.remove(name);
         }
         if (resource != null) {
             support.firePropertyChange("resource", resource, null);
+            // De-register with JMX
+            if (resourceRequireExplicitRegistration) {
+                try {
+                    MBeanUtils.destroyMBean(resource);
+                } catch (Exception e) {
+                    log.warn(sm.getString("namingResources.mbeanDestroyFail",
+                            resource.getName()), e);
+                }
+            }
             resource.setNamingResources(null);
         }
-
     }
 
 
@@ -758,7 +848,7 @@ public class NamingResources implements Serializable {
         ContextResourceEnvRef resourceEnvRef = null;
         synchronized (resourceEnvRefs) {
             resourceEnvRef =
-                (ContextResourceEnvRef) resourceEnvRefs.remove(name);
+                resourceEnvRefs.remove(name);
         }
         if (resourceEnvRef != null) {
             support.firePropertyChange("resourceEnvRef", resourceEnvRef, null);
@@ -779,13 +869,21 @@ public class NamingResources implements Serializable {
 
         ContextResourceLink resourceLink = null;
         synchronized (resourceLinks) {
-            resourceLink = (ContextResourceLink) resourceLinks.remove(name);
+            resourceLink = resourceLinks.remove(name);
         }
         if (resourceLink != null) {
             support.firePropertyChange("resourceLink", resourceLink, null);
+            // De-register with JMX
+            if (resourceRequireExplicitRegistration) {
+                try {
+                    MBeanUtils.destroyMBean(resourceLink);
+                } catch (Exception e) {
+                    log.warn(sm.getString("namingResources.mbeanDestroyFail",
+                            resourceLink.getName()), e);
+                }
+            }
             resourceLink.setNamingResources(null);
         }
-
     }
 
 
@@ -800,7 +898,7 @@ public class NamingResources implements Serializable {
         
         ContextService service = null;
         synchronized (services) {
-            service = (ContextService) services.remove(name);
+            service = services.remove(name);
         }
         if (service != null) {
             support.firePropertyChange("service", service, null);
@@ -810,4 +908,200 @@ public class NamingResources implements Serializable {
     }
 
 
+    // ------------------------------------------------------- Lifecycle methods
+    
+    @Override
+    protected void initInternal() throws LifecycleException {
+        super.initInternal();
+        
+        // Set this before we register currently known naming resources to avoid
+        // timing issues. Duplication registration is not an issue.
+        resourceRequireExplicitRegistration = true;
+        
+        for (ContextResource cr : resources.values()) {
+            try {
+                MBeanUtils.createMBean(cr);
+            } catch (Exception e) {
+                log.warn(sm.getString(
+                        "namingResources.mbeanCreateFail", cr.getName()), e);
+            }
+        }
+        
+        for (ContextEnvironment ce : envs.values()) {
+            try {
+                MBeanUtils.createMBean(ce);
+            } catch (Exception e) {
+                log.warn(sm.getString(
+                        "namingResources.mbeanCreateFail", ce.getName()), e);
+            }
+        }
+        
+        for (ContextResourceLink crl : resourceLinks.values()) {
+            try {
+                MBeanUtils.createMBean(crl);
+            } catch (Exception e) {
+                log.warn(sm.getString(
+                        "namingResources.mbeanCreateFail", crl.getName()), e);
+            }
+        }
+    }
+
+
+    @Override
+    protected void startInternal() throws LifecycleException {
+        fireLifecycleEvent(CONFIGURE_START_EVENT, null);
+        setState(LifecycleState.STARTING);
+    }
+
+
+    @Override
+    protected void stopInternal() throws LifecycleException {
+        cleanUp();
+        setState(LifecycleState.STOPPING);
+        fireLifecycleEvent(CONFIGURE_STOP_EVENT, null);
+    }
+
+    /**
+     * Close those resources that an explicit close may help clean-up faster.
+     */
+    private void cleanUp() {
+        if (resources.size() == 0) {
+            return;
+        }
+        javax.naming.Context ctxt;
+        try {
+            if (container instanceof Server) {
+                ctxt = ((Server) container).getGlobalNamingContext();
+            } else {
+                ctxt = ContextBindings.getClassLoader();
+                ctxt = (javax.naming.Context) ctxt.lookup("comp/env");
+            }
+        } catch (NamingException e) {
+            log.warn(sm.getString("namingResources.cleanupNoContext",
+                    container), e);
+            return;
+        }
+        for (ContextResource cr: resources.values()) {
+            if (cr.getSingleton()) {
+                String closeMethod = cr.getCloseMethod(); 
+                if (closeMethod != null && closeMethod.length() > 0) {
+                    String name = cr.getName();
+                    Object resource;
+                    try {
+                         resource = ctxt.lookup(name);
+                    } catch (NamingException e) {
+                        log.warn(sm.getString(
+                                "namingResources.cleanupNoResource",
+                                cr.getName(), container), e);
+                        continue;
+                    }
+                    cleanUp(resource, name, closeMethod);
+                }
+            }
+        }
+    }
+
+    
+    /**
+     * Clean up a resource by calling the defined close method. For example,
+     * closing a database connection pool will close it's open connections. This
+     * will happen on GC but that leaves db connections open that may cause
+     * issues.
+     * 
+     * @param resource  The resource to close.
+     */
+    private void cleanUp(Object resource, String name, String closeMethod) {
+        // Look for a zero-arg close() method
+        Method m = null;
+        try {
+            m = resource.getClass().getMethod(closeMethod, (Class<?>[]) null);
+        } catch (SecurityException e) {
+            log.debug(sm.getString("namingResources.cleanupCloseSecurity",
+                    closeMethod, name, container));
+            return;
+        } catch (NoSuchMethodException e) {
+            log.debug(sm.getString("namingResources.cleanupNoClose",
+                    name, container, closeMethod));
+            return;
+        }
+        if (m != null) {
+            try {
+                m.invoke(resource, (Object[]) null);
+            } catch (IllegalArgumentException e) {
+                log.warn(sm.getString("namingResources.cleanupCloseFailed",
+                        closeMethod, name, container), e);
+            } catch (IllegalAccessException e) {
+                log.warn(sm.getString("namingResources.cleanupCloseFailed",
+                        closeMethod, name, container), e);
+            } catch (InvocationTargetException e) {
+                Throwable t = ExceptionUtils.unwrapInvocationTargetException(e);
+                ExceptionUtils.handleThrowable(t);
+                log.warn(sm.getString("namingResources.cleanupCloseFailed",
+                        closeMethod, name, container), t);
+            }
+        }
+    }
+
+    @Override
+    protected void destroyInternal() throws LifecycleException {
+
+        // Set this before we de-register currently known naming resources to
+        // avoid timing issues. Duplication de-registration is not an issue.
+        resourceRequireExplicitRegistration = false;
+
+        // Destroy in reverse order to create, although it should not matter
+        for (ContextResourceLink crl : resourceLinks.values()) {
+            try {
+                MBeanUtils.destroyMBean(crl);
+            } catch (Exception e) {
+                log.warn(sm.getString(
+                        "namingResources.mbeanDestroyFail", crl.getName()), e);
+            }
+        }
+        
+        for (ContextEnvironment ce : envs.values()) {
+            try {
+                MBeanUtils.destroyMBean(ce);
+            } catch (Exception e) {
+                log.warn(sm.getString(
+                        "namingResources.mbeanDestroyFail", ce.getName()), e);
+            }
+        }
+        
+        for (ContextResource cr : resources.values()) {
+            try {
+                MBeanUtils.destroyMBean(cr);
+            } catch (Exception e) {
+                log.warn(sm.getString(
+                        "namingResources.mbeanDestroyFail", cr.getName()), e);
+            }
+        }
+        
+        super.destroyInternal();
+    }
+
+
+    @Override
+    protected String getDomainInternal() {
+        // Use the same domain as our associated container if we have one
+        Object c = getContainer();
+        
+        if (c instanceof LifecycleMBeanBase) {
+            return ((LifecycleMBeanBase) c).getDomain();
+        }
+
+        return null;
+    }
+
+
+    @Override
+    protected String getObjectNameKeyProperties() {
+        Object c = getContainer();
+        if (c instanceof Container) {
+            return "type=NamingResources" +
+                    MBeanUtils.getContainerKeyProperties((Container) c);
+        }
+        // Server or just unknown
+        return "type=NamingResources";
+    }
 }
