@@ -37,8 +37,6 @@ import java.net.URLStreamHandlerFactory;
 import java.util.ArrayList;
 import java.util.jar.JarFile;
 
-import javax.management.MBeanRegistration;
-import javax.management.MBeanServer;
 import javax.management.ObjectName;
 import javax.naming.NameClassPair;
 import javax.naming.NamingEnumeration;
@@ -48,19 +46,20 @@ import javax.servlet.ServletContext;
 
 import org.apache.catalina.Container;
 import org.apache.catalina.Context;
-import org.apache.catalina.Engine;
 import org.apache.catalina.Globals;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleException;
-import org.apache.catalina.LifecycleListener;
+import org.apache.catalina.LifecycleState;
 import org.apache.catalina.Loader;
 import org.apache.catalina.core.StandardContext;
-import org.apache.catalina.util.LifecycleSupport;
-import org.apache.catalina.util.StringManager;
+import org.apache.catalina.mbeans.MBeanUtils;
+import org.apache.catalina.util.LifecycleMBeanBase;
 import org.apache.naming.resources.DirContextURLStreamHandler;
 import org.apache.naming.resources.DirContextURLStreamHandlerFactory;
 import org.apache.naming.resources.Resource;
+import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.modeler.Registry;
+import org.apache.tomcat.util.res.StringManager;
 
 
 /**
@@ -78,11 +77,11 @@ import org.apache.tomcat.util.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Remy Maucherat
- * @version $Id: WebappLoader.java 1066487 2011-02-02 14:48:39Z markt $
+ * @version $Id: WebappLoader.java 1201610 2011-11-14 06:52:52Z kkolinko $
  */
 
-public class WebappLoader
-    implements Lifecycle, Loader, PropertyChangeListener, MBeanRegistration  {
+public class WebappLoader extends LifecycleMBeanBase
+    implements Loader, PropertyChangeListener {
 
     // ----------------------------------------------------------- Constructors
 
@@ -145,12 +144,6 @@ public class WebappLoader
         "org.apache.catalina.loader.WebappLoader/1.0";
 
 
-    /**
-     * The lifecycle event support for this component.
-     */
-    protected LifecycleSupport lifecycle = new LifecycleSupport(this);
-
-
     /**
      * The Java class name of the ClassLoader implementation to be used.
      * This class should extend WebappClassLoader, otherwise, a different 
@@ -185,12 +178,6 @@ public class WebappLoader
         StringManager.getManager(Constants.Package);
 
 
-    /**
-     * Has this component been started?
-     */
-    private boolean started = false;
-
-
     /**
      * The property change support for this component.
      */
@@ -206,7 +193,7 @@ public class WebappLoader
     /**
      * Repositories that are set in the loader, for JMX.
      */
-    private ArrayList loaderRepositories = null;
+    private ArrayList<String> loaderRepositories = null;
 
 
     /**
@@ -221,9 +208,10 @@ public class WebappLoader
     /**
      * Return the Java class loader to be used by this Container.
      */
+    @Override
     public ClassLoader getClassLoader() {
 
-        return ((ClassLoader) classLoader);
+        return classLoader;
 
     }
 
@@ -231,6 +219,7 @@ public class WebappLoader
     /**
      * Return the Container with which this Logger has been associated.
      */
+    @Override
     public Container getContainer() {
 
         return (container);
@@ -243,6 +232,7 @@ public class WebappLoader
      *
      * @param container The associated Container
      */
+    @Override
     public void setContainer(Container container) {
 
         // Deregister from the old Container (if any)
@@ -267,6 +257,7 @@ public class WebappLoader
      * Return the "follow standard delegation model" flag used to configure
      * our ClassLoader.
      */
+    @Override
     public boolean getDelegate() {
 
         return (this.delegate);
@@ -280,12 +271,13 @@ public class WebappLoader
      *
      * @param delegate The new flag
      */
+    @Override
     public void setDelegate(boolean delegate) {
 
         boolean oldDelegate = this.delegate;
         this.delegate = delegate;
-        support.firePropertyChange("delegate", new Boolean(oldDelegate),
-                                   new Boolean(this.delegate));
+        support.firePropertyChange("delegate", Boolean.valueOf(oldDelegate),
+                                   Boolean.valueOf(this.delegate));
 
     }
 
@@ -295,6 +287,7 @@ public class WebappLoader
      * the corresponding version number, in the format
      * <code>&lt;description&gt;/&lt;version&gt;</code>.
      */
+    @Override
     public String getInfo() {
 
         return (info);
@@ -327,6 +320,7 @@ public class WebappLoader
     /**
      * Return the reloadable flag for this Loader.
      */
+    @Override
     public boolean getReloadable() {
 
         return (this.reloadable);
@@ -339,14 +333,15 @@ public class WebappLoader
      *
      * @param reloadable The new reloadable flag
      */
+    @Override
     public void setReloadable(boolean reloadable) {
 
         // Process this property change
         boolean oldReloadable = this.reloadable;
         this.reloadable = reloadable;
         support.firePropertyChange("reloadable",
-                                   new Boolean(oldReloadable),
-                                   new Boolean(this.reloadable));
+                                   Boolean.valueOf(oldReloadable),
+                                   Boolean.valueOf(this.reloadable));
 
     }
 
@@ -376,6 +371,7 @@ public class WebappLoader
      *
      * @param listener The listener to add
      */
+    @Override
     public void addPropertyChangeListener(PropertyChangeListener listener) {
 
         support.addPropertyChangeListener(listener);
@@ -388,6 +384,7 @@ public class WebappLoader
      *
      * @param repository Repository to be added
      */
+    @Override
     public void addRepository(String repository) {
 
         if (log.isDebugEnabled())
@@ -403,7 +400,7 @@ public class WebappLoader
         results[repositories.length] = repository;
         repositories = results;
 
-        if (started && (classLoader != null)) {
+        if (getState().isAvailable() && (classLoader != null)) {
             classLoader.addRepository(repository);
             if( loaderRepositories != null ) loaderRepositories.add(repository);
             setClassPath();
@@ -417,6 +414,7 @@ public class WebappLoader
      * invoked inside the classloading context of this container. Unexpected
      * throwables will be caught and logged.
      */
+    @Override
     public void backgroundProcess() {
         if (reloadable && modified()) {
             try {
@@ -443,20 +441,21 @@ public class WebappLoader
      * For security reason, returns a clone of the Array (since 
      * String are immutable).
      */
+    @Override
     public String[] findRepositories() {
 
-        return ((String[])repositories.clone());
+        return repositories.clone();
 
     }
 
     public String[] getRepositories() {
-        return ((String[])repositories.clone());
+        return repositories.clone();
     }
 
     /** Extra repositories for this loader
      */
     public String getRepositoriesString() {
-        StringBuffer sb=new StringBuffer();
+        StringBuilder sb=new StringBuilder();
         for( int i=0; i<repositories.length ; i++ ) {
             sb.append( repositories[i]).append(":");
         }
@@ -472,7 +471,7 @@ public class WebappLoader
 
     public String getLoaderRepositoriesString() {
         String repositories[]=getLoaderRepositories();
-        StringBuffer sb=new StringBuffer();
+        StringBuilder sb=new StringBuilder();
         for( int i=0; i<repositories.length ; i++ ) {
             sb.append( repositories[i]).append(":");
         }
@@ -495,10 +494,9 @@ public class WebappLoader
      * Has the internal repository associated with this Loader been modified,
      * such that the loaded classes should be reloaded?
      */
+    @Override
     public boolean modified() {
-
-        return (classLoader.modified());
-
+        return classLoader != null ? classLoader.modified() : false ;
     }
 
 
@@ -506,7 +504,7 @@ public class WebappLoader
      * Used to periodically signal to the classloader to release JAR resources.
      */
     public void closeJARs(boolean force) {
-        if (classLoader !=null){
+        if (classLoader !=null) {
             classLoader.closeJARs(force);
         }
     }
@@ -517,6 +515,7 @@ public class WebappLoader
      *
      * @param listener The listener to remove
      */
+    @Override
     public void removePropertyChangeListener(PropertyChangeListener listener) {
 
         support.removePropertyChangeListener(listener);
@@ -527,9 +526,10 @@ public class WebappLoader
     /**
      * Return a String representation of this component.
      */
+    @Override
     public String toString() {
 
-        StringBuffer sb = new StringBuffer("WebappLoader[");
+        StringBuilder sb = new StringBuilder("WebappLoader[");
         if (container != null)
             sb.append(container.getName());
         sb.append("]");
@@ -538,109 +538,28 @@ public class WebappLoader
     }
 
 
-    // ------------------------------------------------------ Lifecycle Methods
-
-
     /**
-     * Add a lifecycle event listener to this component.
+     * Start associated {@link ClassLoader} and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#startInternal()}.
      *
-     * @param listener The listener to add
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
      */
-    public void addLifecycleListener(LifecycleListener listener) {
+    @Override
+    protected void startInternal() throws LifecycleException {
 
-        lifecycle.addLifecycleListener(listener);
-
-    }
-
-
-    /**
-     * Get the lifecycle listeners associated with this lifecycle. If this 
-     * Lifecycle has no listeners registered, a zero-length array is returned.
-     */
-    public LifecycleListener[] findLifecycleListeners() {
-
-        return lifecycle.findLifecycleListeners();
-
-    }
-
-
-    /**
-     * Remove a lifecycle event listener from this component.
-     *
-     * @param listener The listener to remove
-     */
-    public void removeLifecycleListener(LifecycleListener listener) {
-
-        lifecycle.removeLifecycleListener(listener);
-
-    }
-
-    private boolean initialized=false;
-
-    public void init() {
-        initialized=true;
-
-        if( oname==null ) {
-            // not registered yet - standalone or API
-            if( container instanceof StandardContext) {
-                // Register ourself. The container must be a webapp
-                try {
-                    StandardContext ctx=(StandardContext)container;
-                    Engine eng=(Engine)ctx.getParent().getParent();
-                    String path = ctx.getPath();
-                    if (path.equals("")) {
-                        path = "/";
-                    }   
-                    oname=new ObjectName(ctx.getEngineName() + ":type=Loader,path=" +
-                                path + ",host=" + ctx.getParent().getName());
-                    Registry.getRegistry(null, null).registerComponent(this, oname, null);
-                    controller=oname;
-                } catch (Exception e) {
-                    log.error("Error registering loader", e );
-                }
-            }
-        }
-
-        if( container == null ) {
-            // JMX created the loader
-            // TODO
-
-        }
-    }
-
-    public void destroy() {
-        if( controller==oname ) {
-            // Self-registration, undo it
-            Registry.getRegistry(null, null).unregisterComponent(oname);
-            oname = null;
-        }
-        initialized = false;
-
-    }
-
-    /**
-     * Start this component, initializing our associated class loader.
-     *
-     * @exception LifecycleException if a lifecycle error occurs
-     */
-    public void start() throws LifecycleException {
-        // Validate and update our current component state
-        if( ! initialized ) init();
-        if (started)
-            throw new LifecycleException
-                (sm.getString("webappLoader.alreadyStarted"));
         if (log.isDebugEnabled())
             log.debug(sm.getString("webappLoader.starting"));
-        lifecycle.fireLifecycleEvent(START_EVENT, null);
-        started = true;
 
         if (container.getResources() == null) {
             log.info("No resources for " + container);
+            setState(LifecycleState.STARTING);
             return;
         }
+        
         // Register a stream handler factory for the JNDI protocol
         URLStreamHandlerFactory streamHandlerFactory =
-            new DirContextURLStreamHandlerFactory();
+                DirContextURLStreamHandlerFactory.getInstance();
         if (first) {
             first = false;
             try {
@@ -649,6 +568,7 @@ public class WebappLoader
                 // Log and continue anyway, this is not critical
                 log.error("Error registering jndi stream handler", e);
             } catch (Throwable t) {
+                ExceptionUtils.handleThrowable(t);
                 // This is likely a dual registration
                 log.info("Dual registration of jndi stream handler: " 
                          + t.getMessage());
@@ -665,12 +585,12 @@ public class WebappLoader
             if (container instanceof StandardContext) {
                 classLoader.setAntiJARLocking(
                         ((StandardContext) container).getAntiJARLocking());
+                classLoader.setClearReferencesStatic(
+                        ((StandardContext) container).getClearReferencesStatic());
                 classLoader.setClearReferencesStopThreads(
                         ((StandardContext) container).getClearReferencesStopThreads());
                 classLoader.setClearReferencesStopTimerThreads(
                         ((StandardContext) container).getClearReferencesStopTimerThreads());
-                classLoader.setClearReferencesThreadLocals(
-                        ((StandardContext) container).getClearReferencesThreadLocals());
                 classLoader.setClearReferencesHttpClientKeepAliveThread(
                         ((StandardContext) container).getClearReferencesHttpClientKeepAliveThread());
             }
@@ -685,48 +605,48 @@ public class WebappLoader
 
             setPermissions();
 
-            if (classLoader instanceof Lifecycle)
-                ((Lifecycle) classLoader).start();
+            ((Lifecycle) classLoader).start();
 
             // Binding the Webapp class loader to the directory context
-            DirContextURLStreamHandler.bind
-                ((ClassLoader) classLoader, this.container.getResources());
+            DirContextURLStreamHandler.bind(classLoader,
+                    this.container.getResources());
 
             StandardContext ctx=(StandardContext)container;
-            Engine eng=(Engine)ctx.getParent().getParent();
-            String path = ctx.getPath();
-            if (path.equals("")) {
-                path = "/";
+            String contextName = ctx.getName();
+            if (!contextName.startsWith("/")) {
+                contextName = "/" + contextName;
             }   
             ObjectName cloname = new ObjectName
-                (ctx.getEngineName() + ":type=WebappClassLoader,path="
-                 + path + ",host=" + ctx.getParent().getName());
+                (MBeanUtils.getDomain(ctx) + ":type=WebappClassLoader,context="
+                 + contextName + ",host=" + ctx.getParent().getName());
             Registry.getRegistry(null, null)
                 .registerComponent(classLoader, cloname, null);
 
         } catch (Throwable t) {
+            t = ExceptionUtils.unwrapInvocationTargetException(t);
+            ExceptionUtils.handleThrowable(t);
             log.error( "LifecycleException ", t );
             throw new LifecycleException("start: ", t);
         }
 
+        setState(LifecycleState.STARTING);
     }
 
 
     /**
-     * Stop this component, finalizing our associated class loader.
+     * Stop associated {@link ClassLoader} and implement the requirements
+     * of {@link org.apache.catalina.util.LifecycleBase#stopInternal()}.
      *
-     * @exception LifecycleException if a lifecycle error occurs
+     * @exception LifecycleException if this component detects a fatal error
+     *  that prevents this component from being used
      */
-    public void stop() throws LifecycleException {
+    @Override
+    protected void stopInternal() throws LifecycleException {
 
-        // Validate and update our current component state
-        if (!started)
-            throw new LifecycleException
-                (sm.getString("webappLoader.notStarted"));
         if (log.isDebugEnabled())
             log.debug(sm.getString("webappLoader.stopping"));
-        lifecycle.fireLifecycleEvent(STOP_EVENT, null);
-        started = false;
+
+        setState(LifecycleState.STOPPING);
 
         // Remove context attributes as appropriate
         if (container instanceof Context) {
@@ -736,29 +656,24 @@ public class WebappLoader
         }
 
         // Throw away our current class loader
-        if (classLoader instanceof Lifecycle)
-            ((Lifecycle) classLoader).stop();
-        DirContextURLStreamHandler.unbind((ClassLoader) classLoader);
+        ((Lifecycle) classLoader).stop();
+        DirContextURLStreamHandler.unbind(classLoader);
 
         try {
             StandardContext ctx=(StandardContext)container;
-            Engine eng=(Engine)ctx.getParent().getParent();
-            String path = ctx.getPath();
-            if (path.equals("")) {
-                path = "/";
-            }
+            String contextName = ctx.getName();
+            if (!contextName.startsWith("/")) {
+                contextName = "/" + contextName;
+            }   
             ObjectName cloname = new ObjectName
-                (ctx.getEngineName() + ":type=WebappClassLoader,path="
-                 + path + ",host=" + ctx.getParent().getName());
+                (MBeanUtils.getDomain(ctx) + ":type=WebappClassLoader,context="
+                 + contextName + ",host=" + ctx.getParent().getName());
             Registry.getRegistry(null, null).unregisterComponent(cloname);
-        } catch (Throwable t) {
-            log.error( "LifecycleException ", t );
+        } catch (Exception e) {
+            log.error("LifecycleException ", e);
         }
 
         classLoader = null;
-
-        destroy();
-
     }
 
 
@@ -770,12 +685,12 @@ public class WebappLoader
      *
      * @param event The property change event that has occurred
      */
+    @Override
     public void propertyChange(PropertyChangeEvent event) {
 
         // Validate the source of this event
         if (!(event.getSource() instanceof Context))
             return;
-        Context context = (Context) event.getSource();
 
         // Process a relevant property change
         if (event.getPropertyName().equals("reloadable")) {
@@ -800,15 +715,15 @@ public class WebappLoader
     private WebappClassLoader createClassLoader()
         throws Exception {
 
-        Class clazz = Class.forName(loaderClass);
+        Class<?> clazz = Class.forName(loaderClass);
         WebappClassLoader classLoader = null;
 
         if (parentClassLoader == null) {
             parentClassLoader = container.getParentClassLoader();
         }
-        Class[] argTypes = { ClassLoader.class };
+        Class<?>[] argTypes = { ClassLoader.class };
         Object[] args = { parentClassLoader };
-        Constructor constr = clazz.getConstructor(argTypes);
+        Constructor<?> constr = clazz.getConstructor(argTypes);
         classLoader = (WebappClassLoader) constr.newInstance(args);
 
         return classLoader;
@@ -832,7 +747,7 @@ public class WebappLoader
 
         // Assigning permissions for the work directory
         File workDir =
-            (File) servletContext.getAttribute(Globals.WORK_DIR_ATTR);
+            (File) servletContext.getAttribute(ServletContext.TEMPDIR);
         if (workDir != null) {
             try {
                 String workDirPath = workDir.getCanonicalPath();
@@ -875,6 +790,7 @@ public class WebappLoader
                         String path = libDir.getCanonicalPath();
                         classLoader.addPermission(path);
                     } catch (IOException e) {
+                        // Ignore
                     }
                 }
 
@@ -887,6 +803,7 @@ public class WebappLoader
                             String path = libDir.getCanonicalPath();
                             classLoader.addPermission(path);
                         } catch (IOException e) {
+                            // Ignore
                         }
                     }
                     if (classesURL != null) {
@@ -895,6 +812,7 @@ public class WebappLoader
                             String path = classesDir.getCanonicalPath();
                             classLoader.addPermission(path);
                         } catch (IOException e) {
+                            // Ignore
                         }
                     }
                 }
@@ -902,6 +820,7 @@ public class WebappLoader
             }
 
         } catch (MalformedURLException e) {
+            // Ignore
         }
 
     }
@@ -921,15 +840,15 @@ public class WebappLoader
         if (servletContext == null)
             return;
 
-        loaderRepositories=new ArrayList();
+        loaderRepositories=new ArrayList<String>();
         // Loading the work directory
         File workDir =
-            (File) servletContext.getAttribute(Globals.WORK_DIR_ATTR);
+            (File) servletContext.getAttribute(ServletContext.TEMPDIR);
         if (workDir == null) {
             log.info("No work dir for " + servletContext);
         }
 
-        if( log.isDebugEnabled()) 
+        if( log.isDebugEnabled() && workDir != null) 
             log.debug(sm.getString("webappLoader.deploy", workDir.getAbsolutePath()));
 
         classLoader.setWorkDir(workDir);
@@ -965,7 +884,11 @@ public class WebappLoader
             } else {
 
                 classRepository = new File(workDir, classesPath);
-                classRepository.mkdirs();
+                if (!classRepository.mkdirs() &&
+                        !classRepository.isDirectory()) {
+                    throw new IOException(
+                            sm.getString("webappLoader.mkdirFailure"));
+                }
                 if (!copyDir(classes, classRepository)) {
                     throw new IOException(
                             sm.getString("webappLoader.copyFailure"));
@@ -1013,7 +936,10 @@ public class WebappLoader
             } else {
                 copyJars = true;
                 destDir = new File(workDir, libPath);
-                destDir.mkdirs();
+                if (!destDir.mkdirs() && !destDir.isDirectory()) {
+                    throw new IOException(
+                            sm.getString("webappLoader.mkdirFailure"));
+                }
             }
 
             // Looking up directory /WEB-INF/lib in the context
@@ -1026,20 +952,20 @@ public class WebappLoader
                 ioe.initCause(e);
                 throw ioe;
             }
-                while (enumeration.hasMoreElements()) {
+            while (enumeration.hasMoreElements()) {
                 NameClassPair ncPair = enumeration.nextElement();
                 String filename = libPath + "/" + ncPair.getName();
-                    if (!filename.endsWith(".jar"))
-                        continue;
+                if (!filename.endsWith(".jar"))
+                    continue;
 
-                    // Copy JAR in the work directory, always (the JAR file
-                    // would get locked otherwise, which would make it
-                    // impossible to update it or remove it at runtime)
+                // Copy JAR in the work directory, always (the JAR file
+                // would get locked otherwise, which would make it
+                // impossible to update it or remove it at runtime)
                 File destFile = new File(destDir, ncPair.getName());
 
-                    if( log.isDebugEnabled())
-                    log.debug(sm.getString("webappLoader.jarDeploy", filename,
-                                     destFile.getAbsolutePath()));
+                if( log.isDebugEnabled())
+                log.debug(sm.getString("webappLoader.jarDeploy", filename,
+                                 destFile.getAbsolutePath()));
 
                 // Bug 45403 - Explicitly call lookup() on the name to check
                 // that the resource is readable. We cannot use resources
@@ -1054,33 +980,33 @@ public class WebappLoader
                     ioe.initCause(e);
                     throw ioe;
                 }
+                
+                if (!(obj instanceof Resource))
+                    continue;
                     
-                    if (!(obj instanceof Resource))
-                        continue;
+                Resource jarResource = (Resource) obj;
                     
-                    Resource jarResource = (Resource) obj;
-                    
-                    if (copyJars) {
-                        if (!copy(jarResource.streamContent(),
+                if (copyJars) {
+                    if (!copy(jarResource.streamContent(),
                               new FileOutputStream(destFile))) {
                         throw new IOException(
                                 sm.getString("webappLoader.copyFailure"));
                     }
                 }
 
-                    try {
-                        JarFile jarFile = new JarFile(destFile);
-                        classLoader.addJar(filename, jarFile, destFile);
-                    } catch (Exception ex) {
-                        // Catch the exception if there is an empty jar file
+                try {
+                    JarFile jarFile = new JarFile(destFile);
+                    classLoader.addJar(filename, jarFile, destFile);
+                } catch (Exception ex) {
+                    // Catch the exception if there is an empty jar file
                     // Should ignore and continue loading other jar files 
-                        // in the dir
-                    }
-                    
-                    loaderRepositories.add( filename );
+                    // in the dir
                 }
+                    
+                loaderRepositories.add( filename );
             }
         }
+    }
 
 
     /**
@@ -1107,18 +1033,16 @@ public class WebappLoader
             }
         }
 
-        StringBuffer classpath = new StringBuffer();
+        StringBuilder classpath = new StringBuilder();
 
         // Assemble the class path information from our class loader chain
         ClassLoader loader = getClassLoader();
-        int layers = 0;
         int n = 0;
         while (loader != null) {
             if (!(loader instanceof URLClassLoader)) {
                 String cp=getClasspath( loader );
                 if( cp==null ) {
                     log.info( "Unknown loader " + loader + " " + loader.getClass());
-                    break;
                 } else {
                     if (n > 0) 
                         classpath.append(File.pathSeparator);
@@ -1149,7 +1073,6 @@ public class WebappLoader
                 n++;
             }
             loader = loader.getParent();
-            layers++;
         }
 
         this.classpath=classpath.toString();
@@ -1174,6 +1097,8 @@ public class WebappLoader
                 return (String)o;
             return null;
         } catch( Exception ex ) {
+            Throwable t = ExceptionUtils.unwrapInvocationTargetException(ex);
+            ExceptionUtils.handleThrowable(t);
             if (log.isDebugEnabled())
                 log.debug("getClasspath ", ex);
         }
@@ -1187,10 +1112,9 @@ public class WebappLoader
 
         try {
 
-            NamingEnumeration enumeration = srcDir.list("");
+            NamingEnumeration<NameClassPair> enumeration = srcDir.list("");
             while (enumeration.hasMoreElements()) {
-                NameClassPair ncPair =
-                    (NameClassPair) enumeration.nextElement();
+                NameClassPair ncPair = enumeration.nextElement();
                 String name = ncPair.getName();
                 Object object = srcDir.lookup(name);
                 File currentFile = new File(destDir, name);
@@ -1204,7 +1128,8 @@ public class WebappLoader
                     if (!copy((InputStream) object, os))
                         return false;
                 } else if (object instanceof DirContext) {
-                    currentFile.mkdir();
+                    if (!currentFile.isDirectory() && !currentFile.mkdir())
+                        return false;
                     if (!copyDir((DirContext) object, currentFile))
                         return false;
                 }
@@ -1246,38 +1171,40 @@ public class WebappLoader
     }
 
 
-    private static org.apache.juli.logging.Log log=
+    private static final org.apache.juli.logging.Log log=
         org.apache.juli.logging.LogFactory.getLog( WebappLoader.class );
 
-    private ObjectName oname;
-    private MBeanServer mserver;
-    private String domain;
-    private ObjectName controller;
 
-    public ObjectName preRegister(MBeanServer server,
-                                  ObjectName name) throws Exception {
-        oname=name;
-        mserver=server;
-        domain=name.getDomain();
-
-        return name;
+    @Override
+    protected String getDomainInternal() {
+        return MBeanUtils.getDomain(container);
     }
 
-    public void postRegister(Boolean registrationDone) {
-    }
 
-    public void preDeregister() throws Exception {
-    }
-
-    public void postDeregister() {
-    }
-
-    public ObjectName getController() {
-        return controller;
-    }
-
-    public void setController(ObjectName controller) {
-        this.controller = controller;
+    @Override
+    protected String getObjectNameKeyProperties() {
+        
+        StringBuilder name = new StringBuilder("type=Loader");
+        
+        if (container instanceof Context) {
+            name.append(",context=");
+            Context context = (Context) container;
+            
+            String contextName = context.getName();
+            if (!contextName.startsWith("/")) {
+                name.append("/");
+            }   
+            name.append(contextName);
+            
+            name.append(",host=");
+            name.append(context.getParent().getName());
+        } else {
+            // Unlikely / impossible? Handle it to be safe
+            name.append(",container=");
+            name.append(container.getName());
+        }
+        
+        return name.toString();
     }
 
 }
