@@ -42,11 +42,13 @@ import org.apache.catalina.Service;
 import org.apache.catalina.User;
 import org.apache.catalina.UserDatabase;
 import org.apache.catalina.Valve;
+import org.apache.catalina.Wrapper;
 import org.apache.catalina.connector.Connector;
 import org.apache.catalina.deploy.ContextEnvironment;
 import org.apache.catalina.deploy.ContextResource;
 import org.apache.catalina.deploy.ContextResourceLink;
 import org.apache.catalina.deploy.NamingResources;
+import org.apache.catalina.util.ContextName;
 import org.apache.catalina.valves.ValveBase;
 import org.apache.coyote.ProtocolHandler;
 import org.apache.coyote.ajp.AjpAprProtocol;
@@ -54,8 +56,7 @@ import org.apache.coyote.ajp.AjpProtocol;
 import org.apache.coyote.http11.Http11AprProtocol;
 import org.apache.coyote.http11.Http11NioProtocol;
 import org.apache.coyote.http11.Http11Protocol;
-import org.apache.juli.logging.Log;
-import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.IntrospectionUtils;
 import org.apache.tomcat.util.modeler.ManagedBean;
 import org.apache.tomcat.util.modeler.Registry;
@@ -66,31 +67,19 @@ import org.apache.tomcat.util.modeler.Registry;
  *
  * @author Craig R. McClanahan
  * @author Amy Roh
- * @version $Id: MBeanUtils.java 939518 2010-04-30 00:08:58Z kkolinko $
+ * @version $Id: MBeanUtils.java 1189423 2011-10-26 20:12:27Z markt $
  */
 
 public class MBeanUtils {
-    private static Log log = LogFactory.getLog(MBeanUtils.class);
 
     // ------------------------------------------------------- Static Variables
 
-
     /**
      * The set of exceptions to the normal rules used by
      * <code>createManagedBean()</code>.  The first element of each pair
      * is a class name, and the second element is the managed bean name.
      */
     private static String exceptions[][] = {
-        { "org.apache.ajp.tomcat4.Ajp13Connector",
-          "Ajp13Connector" },
-        { "org.apache.coyote.tomcat4.Ajp13Connector",
-          "CoyoteConnector" },
-        { "org.apache.catalina.users.JDBCGroup",
-          "Group" },
-        { "org.apache.catalina.users.JDBCRole",
-          "Role" },
-        { "org.apache.catalina.users.JDBCUser",
-          "User" },
         { "org.apache.catalina.users.MemoryGroup",
           "Group" },
         { "org.apache.catalina.users.MemoryRole",
@@ -138,69 +127,6 @@ public class MBeanUtils {
 
     }
 
-
-    /**
-     * Create, register, and return an MBean for this
-     * <code>Connector</code> object.
-     *
-     * @param connector The Connector to be managed
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-    static DynamicMBean createMBean(Connector connector)
-        throws Exception {
-
-        String mname = createManagedName(connector);
-        ManagedBean managed = registry.findManagedBean(mname);
-        if (managed == null) {
-            Exception e = new Exception("ManagedBean is not found with "+mname);
-            throw new MBeanException(e);
-        }
-        String domain = managed.getDomain();
-        if (domain == null)
-            domain = mserver.getDefaultDomain();
-        DynamicMBean mbean = managed.createMBean(connector);
-        ObjectName oname = createObjectName(domain, connector);
-        if( mserver.isRegistered( oname ))  {
-            mserver.unregisterMBean(oname);
-        }
-        mserver.registerMBean(mbean, oname);
-        return (mbean);
-
-    }
-
-
-    /**
-     * Create, register, and return an MBean for this
-     * <code>Context</code> object.
-     *
-     * @param context The Context to be managed
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-    static DynamicMBean createMBean(Context context)
-        throws Exception {
-
-        String mname = createManagedName(context);
-        ManagedBean managed = registry.findManagedBean(mname);
-        if (managed == null) {
-            Exception e = new Exception("ManagedBean is not found with "+mname);
-            throw new MBeanException(e);
-        }
-        String domain = managed.getDomain();
-        if (domain == null)
-            domain = mserver.getDefaultDomain();
-        DynamicMBean mbean = managed.createMBean(context);
-        ObjectName oname = createObjectName(domain, context);
-        if( mserver.isRegistered(oname)) {
-            log.debug("Already registered " + oname);
-            mserver.unregisterMBean(oname);
-        }
-        mserver.registerMBean(mbean, oname);
-        return (mbean);
-
-    }
-
     
     /**
      * Create, register, and return an MBean for this
@@ -210,7 +136,7 @@ public class MBeanUtils {
      *
      * @exception Exception if an MBean cannot be created or registered
      */
-    static DynamicMBean createMBean(ContextEnvironment environment)
+    public static DynamicMBean createMBean(ContextEnvironment environment)
         throws Exception {
 
         String mname = createManagedName(environment);
@@ -241,7 +167,7 @@ public class MBeanUtils {
      *
      * @exception Exception if an MBean cannot be created or registered
      */
-    static DynamicMBean createMBean(ContextResource resource)
+    public static DynamicMBean createMBean(ContextResource resource)
         throws Exception {
 
         String mname = createManagedName(resource);
@@ -272,7 +198,7 @@ public class MBeanUtils {
      *
      * @exception Exception if an MBean cannot be created or registered
      */
-    static DynamicMBean createMBean(ContextResourceLink resourceLink)
+    public static DynamicMBean createMBean(ContextResourceLink resourceLink)
         throws Exception {
 
         String mname = createManagedName(resourceLink);
@@ -294,36 +220,6 @@ public class MBeanUtils {
 
     }    
  
-    /**
-     * Create, register, and return an MBean for this
-     * <code>Engine</code> object.
-     *
-     * @param engine The Engine to be managed
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-    static DynamicMBean createMBean(Engine engine)
-        throws Exception {
-
-        String mname = createManagedName(engine);
-        ManagedBean managed = registry.findManagedBean(mname);
-        if (managed == null) {
-            Exception e = new Exception("ManagedBean is not found with "+mname);
-            throw new MBeanException(e);
-        }
-        String domain = managed.getDomain();
-        if (domain == null)
-            domain = mserver.getDefaultDomain();
-        DynamicMBean mbean = managed.createMBean(engine);
-        ObjectName oname = createObjectName(domain, engine);
-        if( mserver.isRegistered( oname ))  {
-            mserver.unregisterMBean(oname);
-        }
-        mserver.registerMBean(mbean, oname);
-        return (mbean);
-
-    }
-
 
     /**
      * Create, register, and return an MBean for this
@@ -356,37 +252,6 @@ public class MBeanUtils {
     }
 
 
-    /**
-     * Create, register, and return an MBean for this
-     * <code>Host</code> object.
-     *
-     * @param host The Host to be managed
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-    static DynamicMBean createMBean(Host host)
-        throws Exception {
-
-        String mname = createManagedName(host);
-        ManagedBean managed = registry.findManagedBean(mname);
-        if (managed == null) {
-            Exception e = new Exception("ManagedBean is not found with "+mname);
-            throw new MBeanException(e);
-        }
-        String domain = managed.getDomain();
-        if (domain == null)
-            domain = mserver.getDefaultDomain();
-        DynamicMBean mbean = managed.createMBean(host);
-        ObjectName oname = createObjectName(domain, host);
-        if( mserver.isRegistered( oname ))  {
-            mserver.unregisterMBean(oname);
-        }
-        mserver.registerMBean(mbean, oname);
-        return (mbean);
-
-    }
-
-
     /**
      * Create, register, and return an MBean for this
      * <code>Loader</code> object.
@@ -418,36 +283,6 @@ public class MBeanUtils {
 
     }
 
-    /**
-     * Create, register, and return an MBean for this
-     * <code>Manager</code> object.
-     *
-     * @param manager The Manager to be managed
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-    static DynamicMBean createMBean(Manager manager)
-        throws Exception {
-
-        String mname = createManagedName(manager);
-        ManagedBean managed = registry.findManagedBean(mname);
-        if (managed == null) {
-            Exception e = new Exception("ManagedBean is not found with "+mname);
-            throw new MBeanException(e);
-        }
-        String domain = managed.getDomain();
-        if (domain == null)
-            domain = mserver.getDefaultDomain();
-        DynamicMBean mbean = managed.createMBean(manager);
-        ObjectName oname = createObjectName(domain, manager);
-        if( mserver.isRegistered( oname ))  {
-            mserver.unregisterMBean(oname);
-        }
-        mserver.registerMBean(mbean, oname);
-        return (mbean);
-
-    }
-
 
     /**
      * Create, register, and return an MBean for this
@@ -511,37 +346,6 @@ public class MBeanUtils {
     }
 
     
-    /**
-     * Create, register, and return an MBean for this
-     * <code>Realm</code> object.
-     *
-     * @param realm The Realm to be managed
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-    static DynamicMBean createMBean(Realm realm)
-        throws Exception {
-
-        String mname = createManagedName(realm);
-        ManagedBean managed = registry.findManagedBean(mname);
-        if (managed == null) {
-            Exception e = new Exception("ManagedBean is not found with "+mname);
-            throw new MBeanException(e);
-        }
-        String domain = managed.getDomain();
-        if (domain == null)
-            domain = mserver.getDefaultDomain();
-        DynamicMBean mbean = managed.createMBean(realm);
-        ObjectName oname = createObjectName(domain, realm);
-        if( mserver.isRegistered( oname ))  {
-            mserver.unregisterMBean(oname);
-        }
-        mserver.registerMBean(mbean, oname);
-        return (mbean);
-
-    }
-
-
     /**
      * Create, register, and return an MBean for this
      * <code>Role</code> object.
@@ -573,68 +377,6 @@ public class MBeanUtils {
     }
 
 
-    /**
-     * Create, register, and return an MBean for this
-     * <code>Server</code> object.
-     *
-     * @param server The Server to be managed
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-    static DynamicMBean createMBean(Server server)
-        throws Exception {
-
-        String mname = createManagedName(server);
-        ManagedBean managed = registry.findManagedBean(mname);
-        if (managed == null) {
-            Exception e = new Exception("ManagedBean is not found with "+mname);
-            throw new MBeanException(e);
-        }
-        String domain = managed.getDomain();
-        if (domain == null)
-            domain = mserver.getDefaultDomain();
-        DynamicMBean mbean = managed.createMBean(server);
-        ObjectName oname = createObjectName(domain, server);
-        if( mserver.isRegistered( oname ))  {
-            mserver.unregisterMBean(oname);
-        }
-        mserver.registerMBean(mbean, oname);
-        return (mbean);
-
-    }
-
-
-    /**
-     * Create, register, and return an MBean for this
-     * <code>Service</code> object.
-     *
-     * @param service The Service to be managed
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-    static DynamicMBean createMBean(Service service)
-        throws Exception {
-
-        String mname = createManagedName(service);
-        ManagedBean managed = registry.findManagedBean(mname);
-        if (managed == null) {
-            Exception e = new Exception("ManagedBean is not found with "+mname);
-            throw new MBeanException(e);
-        }
-        String domain = managed.getDomain();
-        if (domain == null)
-            domain = mserver.getDefaultDomain();
-        DynamicMBean mbean = managed.createMBean(service);
-        ObjectName oname = createObjectName(domain, service);
-        if( mserver.isRegistered( oname ))  {
-            mserver.unregisterMBean(oname);
-        }
-        mserver.registerMBean(mbean, oname);
-        return (mbean);
-
-    }
-
-
     /**
      * Create, register, and return an MBean for this
      * <code>User</code> object.
@@ -697,36 +439,6 @@ public class MBeanUtils {
     }
 
 
-    /**
-     * Create, register, and return an MBean for this
-     * <code>Valve</code> object.
-     *
-     * @param valve The Valve to be managed
-     *
-     * @exception Exception if an MBean cannot be created or registered
-     */
-    static DynamicMBean createMBean(Valve valve)
-        throws Exception {
-
-        String mname = createManagedName(valve);
-        ManagedBean managed = registry.findManagedBean(mname);
-        if (managed == null) {
-            Exception e = new Exception("ManagedBean is not found with "+mname);
-            throw new MBeanException(e);
-        }
-        String domain = managed.getDomain();
-        if (domain == null)
-            domain = mserver.getDefaultDomain();
-        DynamicMBean mbean = managed.createMBean(valve);
-        ObjectName oname = createObjectName(domain, valve);
-        if( mserver.isRegistered( oname ))  {
-            mserver.unregisterMBean(oname);
-        }
-        mserver.registerMBean(mbean, oname);
-        return (mbean);
-
-    }
-
     /**
      * Create an <code>ObjectName</code> for this
      * <code>Connector</code> object.
@@ -742,12 +454,11 @@ public class MBeanUtils {
 
         ObjectName name = null;
         try {
-            Object addressObj =
-                IntrospectionUtils.getProperty(connector, "address");
+            Object addressObj = IntrospectionUtils.getProperty(connector, "address");            
             Integer port = (Integer)
                 IntrospectionUtils.getProperty(connector, "port");
 
-            StringBuffer sb = new StringBuffer(domain);
+            StringBuilder sb = new StringBuilder(domain);
             sb.append(":type=Connector");
             sb.append(",port=");
             sb.append(port);
@@ -785,13 +496,9 @@ public class MBeanUtils {
 
         ObjectName name = null;
         Host host = (Host)context.getParent();
-        Service service = ((Engine)host.getParent()).getService();
-        String path = context.getPath();
-        if (path.length() < 1)
-            path = "/";
-        // FIXME 
+        ContextName cn = new ContextName(context.getName());
         name = new ObjectName(domain + ":j2eeType=WebModule,name=//" +
-                              host.getName()+ path +
+                              host.getName()+ cn.getDisplayName() +
                               ",J2EEApplication=none,J2EEServer=none");
     
         return (name);
@@ -819,14 +526,11 @@ public class MBeanUtils {
             name = new ObjectName(domain + ":type=Environment" + 
                         ",resourcetype=Global,name=" + environment.getName());
         } else if (container instanceof Context) {        
-            String path = ((Context)container).getPath();
-            if (path.length() < 1)
-                path = "/";
-            Host host = (Host) ((Context)container).getParent();
-            Engine engine = (Engine) host.getParent();
-            Service service = engine.getService();
+            Context context = ((Context)container);
+            ContextName cn = new ContextName(context.getName());
+            Container host = context.getParent();
             name = new ObjectName(domain + ":type=Environment" + 
-                        ",resourcetype=Context,path=" + path + 
+                        ",resourcetype=Context,context=" + cn.getDisplayName() + 
                         ",host=" + host.getName() +
                         ",name=" + environment.getName());
         }        
@@ -857,14 +561,11 @@ public class MBeanUtils {
                         ",resourcetype=Global,class=" + resource.getType() + 
                         ",name=" + quotedResourceName);
         } else if (container instanceof Context) {                    
-            String path = ((Context)container).getPath();
-            if (path.length() < 1)
-                path = "/";
-            Host host = (Host) ((Context)container).getParent();
-            Engine engine = (Engine) host.getParent();
-            Service service = engine.getService();
+            Context context = ((Context)container);
+            ContextName cn = new ContextName(context.getName());
+            Container host = context.getParent();
             name = new ObjectName(domain + ":type=Resource" +
-                        ",resourcetype=Context,path=" + path + 
+                        ",resourcetype=Context,context=" + cn.getDisplayName() + 
                         ",host=" + host.getName() +
                         ",class=" + resource.getType() +
                         ",name=" + quotedResourceName);
@@ -898,14 +599,11 @@ public class MBeanUtils {
                         ",resourcetype=Global" + 
                         ",name=" + quotedResourceLinkName);
         } else if (container instanceof Context) {                    
-            String path = ((Context)container).getPath();
-            if (path.length() < 1)
-                path = "/";
-            Host host = (Host) ((Context)container).getParent();
-            Engine engine = (Engine) host.getParent();
-            Service service = engine.getService();
+            Context context = ((Context)container);
+            ContextName cn = new ContextName(context.getName());
+            Container host = context.getParent();
             name = new ObjectName(domain + ":type=ResourceLink" +
-                        ",resourcetype=Context,path=" + path + 
+                        ",resourcetype=Context,context=" + cn.getDisplayName() + 
                         ",host=" + host.getName() +
                         ",name=" + quotedResourceLinkName);
         }
@@ -913,9 +611,8 @@ public class MBeanUtils {
         return (name);
 
     }
-    
-    
- 
+
+
     /**
      * Create an <code>ObjectName</code> for this
      * <code>Engine</code> object.
@@ -925,8 +622,7 @@ public class MBeanUtils {
      *
      * @exception MalformedObjectNameException if a name cannot be created
      */
-    static ObjectName createObjectName(String domain,
-                                              Engine engine)
+    static ObjectName createObjectName(String domain, Engine engine)
         throws MalformedObjectNameException {
 
         ObjectName name = null;
@@ -972,8 +668,6 @@ public class MBeanUtils {
         throws MalformedObjectNameException {
 
         ObjectName name = null;
-        Engine engine = (Engine)host.getParent();
-        Service service = engine.getService();
         name = new ObjectName(domain + ":type=Host,host=" +
                               host.getName());
         return (name);
@@ -998,23 +692,16 @@ public class MBeanUtils {
         Container container = loader.getContainer();
 
         if (container instanceof Engine) {
-            Service service = ((Engine)container).getService();
             name = new ObjectName(domain + ":type=Loader");
         } else if (container instanceof Host) {
-            Engine engine = (Engine) container.getParent();
-            Service service = engine.getService();
             name = new ObjectName(domain + ":type=Loader,host=" +
                               container.getName());
         } else if (container instanceof Context) {
-            String path = ((Context)container).getPath();
-            if (path.length() < 1) {
-                path = "/";
-            }
-            Host host = (Host) container.getParent();
-            Engine engine = (Engine) host.getParent();
-            Service service = engine.getService();
-            name = new ObjectName(domain + ":type=Loader,path=" + path +
-                              ",host=" + host.getName());
+            Context context = ((Context)container);
+            ContextName cn = new ContextName(context.getName());
+            Container host = context.getParent();
+            name = new ObjectName(domain + ":type=Loader,context=" +
+                    cn.getDisplayName() + ",host=" + host.getName());
         }
 
         return (name);
@@ -1039,23 +726,16 @@ public class MBeanUtils {
         Container container = manager.getContainer();
 
         if (container instanceof Engine) {
-            Service service = ((Engine)container).getService();
             name = new ObjectName(domain + ":type=Manager");
         } else if (container instanceof Host) {
-            Engine engine = (Engine) container.getParent();
-            Service service = engine.getService();
             name = new ObjectName(domain + ":type=Manager,host=" +
                               container.getName());
         } else if (container instanceof Context) {
-            String path = ((Context)container).getPath();
-            if (path.length() < 1) {
-                path = "/";
-            }
-            Host host = (Host) container.getParent();
-            Engine engine = (Engine) host.getParent();
-            Service service = engine.getService();
-            name = new ObjectName(domain + ":type=Manager,path=" + path +
-                              ",host=" + host.getName());
+            Context context = ((Context)container);
+            ContextName cn = new ContextName(context.getName());
+            Container host = context.getParent();
+            name = new ObjectName(domain + ":type=Manager,context=" +
+                    cn.getDisplayName() + ",host=" + host.getName());
         }
 
         return (name);
@@ -1082,14 +762,11 @@ public class MBeanUtils {
             name = new ObjectName(domain + ":type=NamingResources" + 
                         ",resourcetype=Global");
         } else if (container instanceof Context) {        
-            String path = ((Context)container).getPath();
-            if (path.length() < 1)
-                path = "/";
-            Host host = (Host) ((Context)container).getParent();
-            Engine engine = (Engine) host.getParent();
-            Service service = engine.getService();
+            Context context = ((Context)container);
+            ContextName cn = new ContextName(context.getName());
+            Container host = context.getParent();
             name = new ObjectName(domain + ":type=NamingResources" + 
-                        ",resourcetype=Context,path=" + path + 
+                        ",resourcetype=Context,context=" + cn.getDisplayName() + 
                         ",host=" + host.getName());
         }
         
@@ -1107,8 +784,7 @@ public class MBeanUtils {
      *
      * @exception MalformedObjectNameException if a name cannot be created
      */
-    static ObjectName createObjectName(String domain,
-                                              MBeanFactory factory)
+    static ObjectName createObjectName(String domain, MBeanFactory factory)
         throws MalformedObjectNameException {
 
         ObjectName name = new ObjectName(domain + ":type=MBeanFactory");
@@ -1135,23 +811,16 @@ public class MBeanUtils {
         Container container = realm.getContainer();
 
         if (container instanceof Engine) {
-            Service service = ((Engine)container).getService();
             name = new ObjectName(domain + ":type=Realm");
         } else if (container instanceof Host) {
-            Engine engine = (Engine) container.getParent();
-            Service service = engine.getService();
             name = new ObjectName(domain + ":type=Realm,host=" +
                               container.getName());
         } else if (container instanceof Context) {
-            String path = ((Context)container).getPath();
-            if (path.length() < 1) {
-                path = "/";
-            }
-            Host host = (Host) container.getParent();
-            Engine engine = (Engine) host.getParent();
-            Service service = engine.getService();
-            name = new ObjectName(domain + ":type=Realm,path=" + path +
-                              ",host=" + host.getName());
+            Context context = ((Context)container);
+            ContextName cn = new ContextName(context.getName());
+            Container host = context.getParent();
+            name = new ObjectName(domain + ":type=Realm,context=" +
+                    cn.getDisplayName() + ",host=" + host.getName());
         }
 
         return (name);
@@ -1190,8 +859,7 @@ public class MBeanUtils {
      *
      * @exception MalformedObjectNameException if a name cannot be created
      */
-    static ObjectName createObjectName(String domain,
-                                              Server server)
+    static ObjectName createObjectName(String domain, Server server)
         throws MalformedObjectNameException {
 
         ObjectName name = null;
@@ -1298,7 +966,6 @@ public class MBeanUtils {
                                valve);
         }        
         if (container instanceof Engine) {
-            Service service = ((Engine)container).getService();
             String local="";
             int seq = getSeq(local);
             String ext="";
@@ -1308,7 +975,6 @@ public class MBeanUtils {
             name = new ObjectName(domain + ":type=Valve,name=" + className + 
                                     ext + local );
         } else if (container instanceof Host) {
-            Service service = ((Engine)container.getParent()).getService();
             String local=",host=" +container.getName();
             int seq = getSeq(local);
             String ext="";
@@ -1318,13 +984,10 @@ public class MBeanUtils {
             name = new ObjectName(domain + ":type=Valve,name=" + className + 
                                     ext + local );
         } else if (container instanceof Context) {
-            String path = ((Context)container).getPath();
-            if (path.length() < 1) {
-                path = "/";
-            }
-            Host host = (Host) container.getParent();
-            Service service = ((Engine) host.getParent()).getService();
-            String local=",path=" + path + ",host=" +
+            Context context = ((Context)container);
+            ContextName cn = new ContextName(context.getName());
+            Container host = context.getParent();
+            String local=",context=" + cn.getDisplayName() + ",host=" +
                     host.getName();
             int seq = getSeq(local);
             String ext="";
@@ -1339,9 +1002,9 @@ public class MBeanUtils {
 
     }
 
-    static Hashtable seq=new Hashtable();
+    static Hashtable<String,int[]> seq = new Hashtable<String,int[]>();
     static int getSeq( String key ) {
-        int i[]=(int [])seq.get( key );
+        int i[]=seq.get( key );
         if (i == null ) {
             i=new int[1];
             i[0]=0;
@@ -1356,11 +1019,11 @@ public class MBeanUtils {
      * Create and configure (if necessary) and return the registry of
      * managed object descriptions.
      */
-    public synchronized static Registry createRegistry() {
+    public static synchronized Registry createRegistry() {
 
         if (registry == null) {
             registry = Registry.getRegistry(null, null);
-            ClassLoader cl=ServerLifecycleListener.class.getClassLoader();
+            ClassLoader cl = MBeanUtils.class.getClassLoader();
 
             registry.loadDescriptors("org.apache.catalina.mbeans",  cl);
             registry.loadDescriptors("org.apache.catalina.authenticator", cl);
@@ -1386,15 +1049,10 @@ public class MBeanUtils {
      * <code>MBeanServer</code> with which we will be
      * registering our <code>DynamicMBean</code> implementations.
      */
-    public synchronized static MBeanServer createServer() {
+    public static synchronized MBeanServer createServer() {
 
         if (mserver == null) {
-            try {
-                mserver = Registry.getRegistry(null, null).getMBeanServer();
-            } catch (Throwable t) {
-                t.printStackTrace(System.out);
-                System.exit(1);
-            }
+            mserver = Registry.getRegistry(null, null).getMBeanServer();
         }
         return (mserver);
 
@@ -1417,7 +1075,6 @@ public class MBeanUtils {
         if (domain == null)
             domain = mserver.getDefaultDomain();
         ObjectName oname = createObjectName(domain, connector);
-        connector.setService(null);
         if( mserver.isRegistered( oname ))  {
             mserver.unregisterMBean(oname);
         }
@@ -1473,7 +1130,7 @@ public class MBeanUtils {
      *
      * @exception Exception if an MBean cannot be deregistered
      */
-    static void destroyMBean(ContextEnvironment environment)
+    public static void destroyMBean(ContextEnvironment environment)
         throws Exception {
 
         String mname = createManagedName(environment);
@@ -1499,7 +1156,7 @@ public class MBeanUtils {
      *
      * @exception Exception if an MBean cannot be deregistered
      */
-    static void destroyMBean(ContextResource resource)
+    public static void destroyMBean(ContextResource resource)
         throws Exception {
 
         // If this is a user database resource need to destroy groups, roles,
@@ -1531,7 +1188,7 @@ public class MBeanUtils {
      *
      * @exception Exception if an MBean cannot be deregistered
      */
-    static void destroyMBean(ContextResourceLink resourceLink)
+    public static void destroyMBean(ContextResourceLink resourceLink)
         throws Exception {
 
         String mname = createManagedName(resourceLink);
@@ -1928,7 +1585,7 @@ public class MBeanUtils {
         try {
             ((Contained)valve).setContainer(null);
         } catch (Throwable t) {
-        ;
+            ExceptionUtils.handleThrowable(t);
         }
         if( mserver.isRegistered(oname) ) {
             mserver.unregisterMBean(oname);
@@ -1936,4 +1593,105 @@ public class MBeanUtils {
 
     }
 
+    
+    /**
+     * Determine the name of the domain to register MBeans for from a given
+     * Service.
+     * 
+     * @param service 
+     */
+    public static String getDomain(Service service) {
+        
+        // Null service -> return null
+        if (service == null) {
+            return null;
+        }
+        
+        String domain = null;
+        
+        Container engine = service.getContainer();
+        
+        // Use the engine name first
+        if (engine != null) {
+            domain = engine.getName();
+        }
+        
+        // No engine or no engine name, use the service name 
+        if (domain == null) {
+            domain = service.getName();
+        }
+        
+        // No service name, use null
+        return domain;
+    }
+    
+
+    /**
+     * Determine the name of the domain to register MBeans for from a given
+     * Container.
+     * 
+     * @param container
+     */
+    public static String getDomain(Container container) {
+        
+        String domain = null;
+        
+        Container c = container;
+        
+        while (!(c instanceof Engine) && c != null) {
+            c = c.getParent();
+        }
+        
+        if (c != null) {
+            domain = c.getName();
+        }
+        
+        return domain;
+    }
+
+    
+    /**
+     * Calculate the key properties string to be added to an object's
+     * {@link ObjectName} to indicate that it is associated with that container.
+     * 
+     * @param container The container the object is associated with 
+     * @return          A string suitable for appending to the ObjectName
+     */
+    public static String getContainerKeyProperties(Container container) {
+        
+        Container c = container;
+        StringBuilder keyProperties = new StringBuilder();
+        int containerCount = 0;
+        
+        // Work up container hierarchy, add a component to the name for
+        // each container
+        while (!(c instanceof Engine)) {
+            if (c instanceof Wrapper) {
+                keyProperties.append(",servlet=");
+                keyProperties.append(c.getName());
+            } else if (c instanceof Context) {
+                keyProperties.append(",context=");
+                ContextName cn = new ContextName(c.getName());
+                keyProperties.append(cn.getDisplayName());
+            } else if (c instanceof Host) {
+                keyProperties.append(",host=");
+                keyProperties.append(c.getName());
+            } else if (c == null) {
+                // May happen in unit testing and/or some embedding scenarios
+                keyProperties.append(",container");
+                keyProperties.append(containerCount++);
+                keyProperties.append("=null");
+                break;
+            } else {
+                // Should never happen...
+                keyProperties.append(",container");
+                keyProperties.append(containerCount++);
+                keyProperties.append('=');
+                keyProperties.append(c.getName());
+            }
+            c = c.getParent();
+        }
+
+        return keyProperties.toString();
+    }
 }
