@@ -23,6 +23,7 @@ import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
+import java.nio.charset.Charset;
 import java.security.MessageDigest;
 import java.security.NoSuchAlgorithmException;
 import java.security.Principal;
@@ -52,6 +53,7 @@ import org.apache.catalina.util.LifecycleMBeanBase;
 import org.apache.catalina.util.MD5Encoder;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.buf.B2CConverter;
 import org.apache.tomcat.util.buf.HexUtils;
 import org.apache.tomcat.util.res.StringManager;
 import org.ietf.jgss.GSSContext;
@@ -65,7 +67,7 @@ import org.ietf.jgss.GSSName;
  * location) are identical to those currently supported by Tomcat 3.X.
  *
  * @author Craig R. McClanahan
- * @version $Id: RealmBase.java 1132702 2011-06-06 16:53:58Z markt $
+ * @version $Id: RealmBase.java 1158177 2011-08-16 10:04:00Z markt $
  */
 
 public abstract class RealmBase extends LifecycleMBeanBase implements Realm {
@@ -244,6 +246,14 @@ public abstract class RealmBase extends LifecycleMBeanBase implements Realm {
         digestEncoding = charset;
     }
 
+    protected Charset getDigestCharset() throws UnsupportedEncodingException {
+        if (digestEncoding == null) {
+            return Charset.defaultCharset();
+        } else {
+            return B2CConverter.getCharset(getDigestEncoding());
+        }
+    }
+
     /**
      * Return descriptive information about this Realm implementation and
      * the corresponding version number, in the format
@@ -349,7 +359,7 @@ public abstract class RealmBase extends LifecycleMBeanBase implements Realm {
      *
      * @param username Username of the Principal to look up
      * @param clientDigest Digest which has been submitted by the client
-     * @param nOnce Unique (or supposedly unique) token which has been used
+     * @param nonce Unique (or supposedly unique) token which has been used
      * for this request
      * @param realm Realm name
      * @param md5a2 Second MD5 digest used to calculate the digest :
@@ -357,7 +367,7 @@ public abstract class RealmBase extends LifecycleMBeanBase implements Realm {
      */
     @Override
     public Principal authenticate(String username, String clientDigest,
-                                  String nOnce, String nc, String cnonce,
+                                  String nonce, String nc, String cnonce,
                                   String qop, String realm,
                                   String md5a2) {
 
@@ -366,22 +376,18 @@ public abstract class RealmBase extends LifecycleMBeanBase implements Realm {
             return null;
         String serverDigestValue;
         if (qop == null) {
-            serverDigestValue = md5a1 + ":" + nOnce + ":" + md5a2;
+            serverDigestValue = md5a1 + ":" + nonce + ":" + md5a2;
         } else {
-            serverDigestValue = md5a1 + ":" + nOnce + ":" + nc + ":" +
+            serverDigestValue = md5a1 + ":" + nonce + ":" + nc + ":" +
                     cnonce + ":" + qop + ":" + md5a2;
         }
 
         byte[] valueBytes = null;
-        if(getDigestEncoding() == null) {
-            valueBytes = serverDigestValue.getBytes();
-        } else {
-            try {
-                valueBytes = serverDigestValue.getBytes(getDigestEncoding());
-            } catch (UnsupportedEncodingException uee) {
-                log.error("Illegal digestEncoding: " + getDigestEncoding(), uee);
-                throw new IllegalArgumentException(uee.getMessage());
-            }
+        try {
+            valueBytes = serverDigestValue.getBytes(getDigestCharset());
+        } catch (UnsupportedEncodingException uee) {
+            log.error("Illegal digestEncoding: " + getDigestEncoding(), uee);
+            throw new IllegalArgumentException(uee.getMessage());
         }
 
         String serverDigest = null;
@@ -392,7 +398,7 @@ public abstract class RealmBase extends LifecycleMBeanBase implements Realm {
 
         if (log.isDebugEnabled()) {
             log.debug("Digest : " + clientDigest + " Username:" + username 
-                    + " ClientSigest:" + clientDigest + " nOnce:" + nOnce 
+                    + " ClientSigest:" + clientDigest + " nonce:" + nonce 
                     + " nc:" + nc + " cnonce:" + cnonce + " qop:" + qop 
                     + " realm:" + realm + "md5a2:" + md5a2 
                     + " Server digest:" + serverDigest);
@@ -1130,15 +1136,11 @@ public abstract class RealmBase extends LifecycleMBeanBase implements Realm {
                 md.reset();
     
                 byte[] bytes = null;
-                if(getDigestEncoding() == null) {
-                    bytes = credentials.getBytes();
-                } else {
-                    try {
-                        bytes = credentials.getBytes(getDigestEncoding());
-                    } catch (UnsupportedEncodingException uee) {
-                        log.error("Illegal digestEncoding: " + getDigestEncoding(), uee);
-                        throw new IllegalArgumentException(uee.getMessage());
-                    }
+                try {
+                    bytes = credentials.getBytes(getDigestCharset());
+                } catch (UnsupportedEncodingException uee) {
+                    log.error("Illegal digestEncoding: " + getDigestEncoding(), uee);
+                    throw new IllegalArgumentException(uee.getMessage());
                 }
                 md.update(bytes);
 
@@ -1177,15 +1179,11 @@ public abstract class RealmBase extends LifecycleMBeanBase implements Realm {
             + getPassword(username);
 
         byte[] valueBytes = null;
-        if(getDigestEncoding() == null) {
-            valueBytes = digestValue.getBytes();
-        } else {
-            try {
-                valueBytes = digestValue.getBytes(getDigestEncoding());
-            } catch (UnsupportedEncodingException uee) {
-                log.error("Illegal digestEncoding: " + getDigestEncoding(), uee);
-                throw new IllegalArgumentException(uee.getMessage());
-            }
+        try {
+            valueBytes = digestValue.getBytes(getDigestCharset());
+        } catch (UnsupportedEncodingException uee) {
+            log.error("Illegal digestEncoding: " + getDigestEncoding(), uee);
+            throw new IllegalArgumentException(uee.getMessage());
         }
 
         byte[] digest = null;
