@@ -25,21 +25,25 @@ import java.net.URLDecoder;
 import java.util.Collection;
 import java.util.Date;
 import java.util.Enumeration;
+import java.util.Locale;
+
 import javax.servlet.RequestDispatcher;
 import javax.servlet.ServletContext;
 import javax.servlet.ServletException;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
+
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.util.RequestUtil;
 import org.apache.coyote.Constants;
+import org.apache.tomcat.util.buf.B2CConverter;
 
 /**
  * An implementation of SSIExternalResolver that is used with servlets.
  * 
  * @author Dan Sandberg
  * @author David Becker
- * @version $Revision: 739532 $, $Date: 2009-01-31 10:52:13 +0100 (Sa, 31. Jan 2009) $
+ * @version $Id: SSIServletExternalResolver.java 1140463 2011-06-28 07:22:28Z markt $
  */
 public class SSIServletExternalResolver implements SSIExternalResolver {
     protected final String VARIABLE_NAMES[] = {"AUTH_TYPE", "CONTENT_LENGTH",
@@ -71,6 +75,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
     }
 
 
+    @Override
     public void log(String message, Throwable throwable) {
         //We can't assume that Servlet.log( message, null )
         //is the same as Servlet.log( message ), since API
@@ -83,7 +88,8 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
     }
 
 
-    public void addVariableNames(Collection variableNames) {
+    @Override
+    public void addVariableNames(Collection<String> variableNames) {
         for (int i = 0; i < VARIABLE_NAMES.length; i++) {
             String variableName = VARIABLE_NAMES[i];
             String variableValue = getVariableValue(variableName);
@@ -91,9 +97,9 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
                 variableNames.add(variableName);
             }
         }
-        Enumeration e = req.getAttributeNames();
+        Enumeration<String> e = req.getAttributeNames();
         while (e.hasMoreElements()) {
-            String name = (String)e.nextElement();
+            String name = e.nextElement();
             if (!isNameReserved(name)) {
                 variableNames.add(name);
             }
@@ -106,9 +112,9 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
         if (!isNameReserved(targetName)) {
             object = req.getAttribute(targetName);
             if (object == null) {
-                Enumeration e = req.getAttributeNames();
+                Enumeration<String> e = req.getAttributeNames();
                 while (e.hasMoreElements()) {
-                    String name = (String)e.nextElement();
+                    String name = e.nextElement();
                     if (targetName.equalsIgnoreCase(name)
                             && !isNameReserved(name)) {
                         object = req.getAttribute(name);
@@ -129,6 +135,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
     }
 
 
+    @Override
     public void setVariableValue(String name, String value) {
         if (!isNameReserved(name)) {
             req.setAttribute(name, value);
@@ -136,6 +143,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
     }
 
 
+    @Override
     public String getVariableValue(String name) {
         String retVal = null;
         Object object = getReqAttributeIgnoreCase(name);
@@ -150,12 +158,11 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
 
     protected String getCGIVariable(String name) {
         String retVal = null;
-        String[] nameParts = name.toUpperCase().split("_");
+        String[] nameParts = name.toUpperCase(Locale.ENGLISH).split("_");
         int requiredParts = 2;
         if (nameParts.length == 1) {
             if (nameParts[0].equals("PATH")) {
                 requiredParts = 1;
-                retVal = null; // Not implemented
             }
         }
         else if (nameParts[0].equals("AUTH")) {
@@ -193,14 +200,14 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
                     accept = "Accept-Language";
                 }
                 if (accept != null) {
-                    Enumeration acceptHeaders = req.getHeaders(accept);
+                    Enumeration<String> acceptHeaders = req.getHeaders(accept);
                     if (acceptHeaders != null)
                         if (acceptHeaders.hasMoreElements()) {
-                            StringBuffer rv = new StringBuffer(
-                                    (String) acceptHeaders.nextElement());
+                            StringBuilder rv = new StringBuilder(
+                                    acceptHeaders.nextElement());
                             while (acceptHeaders.hasMoreElements()) {
                                 rv.append(", ");
-                                rv.append((String) acceptHeaders.nextElement());
+                                rv.append(acceptHeaders.nextElement());
                             }
                         retVal = rv.toString();
                     }
@@ -278,7 +285,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
             } else if (nameParts[1].equals("HOST")) {
                 retVal = req.getRemoteHost();
             } else if (nameParts[1].equals("IDENT")) {
-                retVal = null; // Not implemented
+                // Not implemented
             } else if (nameParts[1].equals("PORT")) {
                 retVal = Integer.toString( req.getRemotePort());
             } else if (nameParts[1].equals("USER")) {
@@ -291,7 +298,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
             else if (nameParts[1].equals("URI")) {
                 // If this is an error page, get the original URI
                 retVal = (String) req.getAttribute(
-                        "javax.servlet.forward.request_uri");
+                        RequestDispatcher.FORWARD_REQUEST_URI);
                 if (retVal == null) retVal=req.getRequestURI();
             }
         } else if (nameParts[0].equals("SCRIPT")) {
@@ -313,7 +320,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
             } else if (nameParts[1].equals("PROTOCOL")) {
                 retVal = req.getProtocol();
             } else if (nameParts[1].equals("SOFTWARE")) {
-                StringBuffer rv = new StringBuffer(context.getServerInfo());
+                StringBuilder rv = new StringBuilder(context.getServerInfo());
                 rv.append(" ");
                 rv.append(System.getProperty("java.vm.name"));
                 rv.append("/");
@@ -329,6 +336,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
             return retVal;
     }
 
+    @Override
     public Date getCurrentDate() {
         return new Date();
     }
@@ -347,7 +355,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
         String retVal = null;
         int lastSlash = servletPath.lastIndexOf('/');
         if (lastSlash >= 0) {
-            //cut off file namee
+            //cut off file name
             retVal = servletPath.substring(0, lastSlash + 1);
         }
         return retVal;
@@ -403,34 +411,34 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
         if (!virtualPath.startsWith("/") && !virtualPath.startsWith("\\")) {
             return new ServletContextAndPath(context,
                     getAbsolutePath(virtualPath));
-        } else {
-            String normalized = RequestUtil.normalize(virtualPath);
-            if (isVirtualWebappRelative) {
-                return new ServletContextAndPath(context, normalized);
-            } else {
-                ServletContext normContext = context.getContext(normalized);
-                if (normContext == null) {
-                    throw new IOException("Couldn't get context for path: "
-                            + normalized);
-                }
-                //If it's the root context, then there is no context element
-                // to remove,
-                // ie:
-                // '/file1.shtml' vs '/appName1/file1.shtml'
-                if (!isRootContext(normContext)) {
-                    String noContext = getPathWithoutContext(
-                            normContext.getContextPath(), normalized);
-                    if (noContext == null) {
-                        throw new IOException(
-                                "Couldn't remove context from path: "
-                                        + normalized);
-                    }
-                    return new ServletContextAndPath(normContext, noContext);
-                } else {
-                    return new ServletContextAndPath(normContext, normalized);
-                }
-            }
         }
+
+        String normalized = RequestUtil.normalize(virtualPath);
+        if (isVirtualWebappRelative) {
+            return new ServletContextAndPath(context, normalized);
+        }
+
+        ServletContext normContext = context.getContext(normalized);
+        if (normContext == null) {
+            throw new IOException("Couldn't get context for path: "
+                    + normalized);
+        }
+        //If it's the root context, then there is no context element
+        // to remove,
+        // ie:
+        // '/file1.shtml' vs '/appName1/file1.shtml'
+        if (!isRootContext(normContext)) {
+            String noContext = getPathWithoutContext(
+                    normContext.getContextPath(), normalized);
+            if (noContext == null) {
+                throw new IOException(
+                        "Couldn't remove context from path: "
+                                + normalized);
+            }
+            return new ServletContextAndPath(normContext, noContext);
+        }
+
+        return new ServletContextAndPath(normContext, normalized);
     }
 
 
@@ -478,6 +486,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
     }
 
 
+    @Override
     public long getFileLastModified(String path, boolean virtual)
             throws IOException {
         long lastModified = 0;
@@ -491,6 +500,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
     }
 
 
+    @Override
     public long getFileSize(String path, boolean virtual) throws IOException {
         long fileSize = -1;
         try {
@@ -506,6 +516,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
     //We are making lots of unnecessary copies of the included data here. If
     //someone ever complains that this is slow, we should connect the included
     // stream to the print writer that SSICommand uses.
+    @Override
     public String getFileText(String originalPath, boolean virtual)
             throws IOException {
         try {
@@ -532,7 +543,8 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
             if (inputEncoding == null) {
                 retVal = new String( bytes );
             } else {
-                retVal = new String (bytes, inputEncoding);
+                retVal = new String (bytes,
+                        B2CConverter.getCharset(inputEncoding));
             }
 
             //make an assumption that an empty response is a failure. This is
@@ -550,7 +562,7 @@ public class SSIServletExternalResolver implements SSIExternalResolver {
         }
     }
 
-    protected class ServletContextAndPath {
+    protected static class ServletContextAndPath {
         protected ServletContext servletContext;
         protected String path;
 
