@@ -19,14 +19,14 @@ package org.apache.coyote.http11.filters;
 
 import java.io.EOFException;
 import java.io.IOException;
-
-import org.apache.tomcat.util.buf.ByteChunk;
-import org.apache.tomcat.util.buf.HexUtils;
+import java.nio.charset.Charset;
 
 import org.apache.coyote.InputBuffer;
 import org.apache.coyote.Request;
 import org.apache.coyote.http11.Constants;
 import org.apache.coyote.http11.InputFilter;
+import org.apache.tomcat.util.buf.ByteChunk;
+import org.apache.tomcat.util.buf.HexUtils;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.http.MimeHeaders;
 
@@ -51,7 +51,8 @@ public class ChunkedInputFilter implements InputFilter {
 
 
     static {
-        ENCODING.setBytes(ENCODING_NAME.getBytes(), 0, ENCODING_NAME.length());
+        ENCODING.setBytes(ENCODING_NAME.getBytes(Charset.defaultCharset()), 0,
+                ENCODING_NAME.length());
     }
 
 
@@ -103,15 +104,7 @@ public class ChunkedInputFilter implements InputFilter {
     /**
      * Byte chunk used to store trailing headers.
      */
-    protected ByteChunk trailingHeaders;
-
-    {
-        trailingHeaders = new ByteChunk();
-        if (org.apache.coyote.Constants.MAX_TRAILER_SIZE > 0) {
-            trailingHeaders.setLimit(org.apache.coyote.Constants.MAX_TRAILER_SIZE);
-        }
-    }
-
+    protected ByteChunk trailingHeaders = new ByteChunk();
 
     /**
      * Flag set to true if the next call to doRead() must parse a CRLF pair
@@ -125,8 +118,10 @@ public class ChunkedInputFilter implements InputFilter {
      */
     private Request request;
     
-    // ------------------------------------------------------------- Properties
-
+    // ----------------------------------------------------------- Constructors
+    public ChunkedInputFilter(int maxTrailerSize) {
+        this.trailingHeaders.setLimit(maxTrailerSize);
+    }
 
     // ---------------------------------------------------- InputBuffer Methods
 
@@ -140,6 +135,7 @@ public class ChunkedInputFilter implements InputFilter {
      * whichever is greater. If the filter does not do request body length
      * control, the returned value should be -1.
      */
+    @Override
     public int doRead(ByteChunk chunk, Request req)
         throws IOException {
 
@@ -148,7 +144,7 @@ public class ChunkedInputFilter implements InputFilter {
 
         if(needCRLFParse) {
             needCRLFParse = false;
-            parseCRLF(false);
+            parseCRLF();
         }
 
         if (remaining <= 0) {
@@ -164,10 +160,7 @@ public class ChunkedInputFilter implements InputFilter {
         int result = 0;
 
         if (pos >= lastValid) {
-            if (readBytes() < 0) {
-                throw new IOException(
-                        "Unexpected end of stream whilst reading request body");
-            }
+            readBytes();
         }
 
         if (remaining > (lastValid - pos)) {
@@ -186,7 +179,7 @@ public class ChunkedInputFilter implements InputFilter {
                 //so we defer it to the next call BZ 11117
                 needCRLFParse = true;
             } else {
-                parseCRLF(false); //parse the CRLF immediately
+                parseCRLF(); //parse the CRLF immediately
             }
         }
 
@@ -201,6 +194,7 @@ public class ChunkedInputFilter implements InputFilter {
     /**
      * Read the content length from the request.
      */
+    @Override
     public void setRequest(Request request) {
         this.request = request;
     }
@@ -209,11 +203,13 @@ public class ChunkedInputFilter implements InputFilter {
     /**
      * End the current request.
      */
+    @Override
     public long end()
         throws IOException {
 
         // Consume extra bytes : parse the stream until the end chunk is found
         while (doRead(readChunk, null) >= 0) {
+            // NOOP: Just consume the input
         }
 
         // Return the number of extra bytes which were consumed
@@ -225,6 +221,7 @@ public class ChunkedInputFilter implements InputFilter {
     /**
      * Amount of bytes still available in a buffer.
      */
+    @Override
     public int available() {
         return (lastValid - pos);
     }
@@ -233,6 +230,7 @@ public class ChunkedInputFilter implements InputFilter {
     /**
      * Set the next buffer in the filter pipeline.
      */
+    @Override
     public void setBuffer(InputBuffer buffer) {
         this.buffer = buffer;
     }
@@ -241,12 +239,12 @@ public class ChunkedInputFilter implements InputFilter {
     /**
      * Make the filter ready to process the next request.
      */
+    @Override
     public void recycle() {
         remaining = 0;
         pos = 0;
         lastValid = 0;
         endChunk = false;
-        needCRLFParse = false;
         trailingHeaders.recycle();
     }
 
@@ -255,6 +253,7 @@ public class ChunkedInputFilter implements InputFilter {
      * Return the name of the associated encoding; Here, the value is 
      * "identity".
      */
+    @Override
     public ByteChunk getEncodingName() {
         return ENCODING;
     }
@@ -303,18 +302,18 @@ public class ChunkedInputFilter implements InputFilter {
                     return false;
             }
 
-            if (buf[pos] == Constants.CR || buf[pos] == Constants.LF) {
-                parseCRLF(false);
+            if (buf[pos] == Constants.CR) {
+                // FIXME: Improve parsing to check for CRLF 
+            } else if (buf[pos] == Constants.LF) {
                 eol = true;
             } else if (buf[pos] == Constants.SEMI_COLON) {
                 trailer = true;
             } else if (!trailer) { 
                 //don't read data after the trailer
-                int charValue = HexUtils.getDec(buf[pos]);
-                if (charValue != -1) {
+                if (HexUtils.getDec(buf[pos]) != -1) {
                     readDigit = true;
                     result *= 16;
-                    result += charValue;
+                    result += HexUtils.getDec(buf[pos]);
                 } else {
                     //we shouldn't allow invalid, non hex characters
                     //in the chunked header
@@ -322,10 +321,7 @@ public class ChunkedInputFilter implements InputFilter {
                 }
             }
 
-            // Parsing the CRLF increments pos
-            if (!eol) {
-                pos++;
-            }
+            pos++;
 
         }
 
@@ -346,22 +342,9 @@ public class ChunkedInputFilter implements InputFilter {
 
     /**
      * Parse CRLF at end of chunk.
-     * @deprecated  Use {@link #parseCRLF(boolean)}
      */
-    @Deprecated
-    protected boolean parseCRLF() throws IOException {
-        parseCRLF(false);
-        return true;
-    }
-
-    /**
-     * Parse CRLF at end of chunk.
-     *
-     * @param   tolerant    Should tolerant parsing (LF and CRLF) be used? This
-     *                      is recommended (RFC2616, section 19.3) for message
-     *                      headers.
-     */
-    protected void parseCRLF(boolean tolerant) throws IOException {
+    protected boolean parseCRLF()
+        throws IOException {
 
         boolean eol = false;
         boolean crfound = false;
@@ -377,9 +360,7 @@ public class ChunkedInputFilter implements InputFilter {
                 if (crfound) throw new IOException("Invalid CRLF, two CR characters encountered.");
                 crfound = true;
             } else if (buf[pos] == Constants.LF) {
-                if (!tolerant && !crfound) {
-                    throw new IOException("Invalid CRLF, no CR character encountered.");
-                }
+                if (!crfound) throw new IOException("Invalid CRLF, no CR character encountered.");
                 eol = true;
             } else {
                 throw new IOException("Invalid CRLF");
@@ -388,19 +369,21 @@ public class ChunkedInputFilter implements InputFilter {
             pos++;
 
         }
+
+        return true;
+
     }
 
 
     /**
      * Parse end chunk data.
      */
-    protected boolean parseEndChunk() throws IOException {
+    protected void parseEndChunk() throws IOException {
 
-        // Handle optional trailer headers
+        // Handle option trailer headers
         while (parseHeader()) {
             // Loop until we run out of headers
         }
-        return true;
     }
 
     
@@ -409,19 +392,26 @@ public class ChunkedInputFilter implements InputFilter {
         MimeHeaders headers = request.getMimeHeaders();
 
         byte chr = 0;
+        while (true) {
+            // Read new bytes if needed
+            if (pos >= lastValid) {
+                if (readBytes() <0)
+                    throw new EOFException("Unexpected end of stream whilst reading trailer headers for chunked request");
+            }
 
-        // Read new bytes if needed
-        if (pos >= lastValid) {
-            if (readBytes() <0)
-                throw new EOFException("Unexpected end of stream whilst reading trailer headers for chunked request");
-        }
+            chr = buf[pos];
+    
+            if ((chr == Constants.CR) || (chr == Constants.LF)) {
+                if (chr == Constants.LF) {
+                    pos++;
+                    return false;
+                }
+            } else {
+                break;
+            }
+    
+            pos++;
     
-        chr = buf[pos];
-
-        // CRLF terminates the request
-        if (chr == Constants.CR || chr == Constants.LF) {
-            parseCRLF(false);
-            return false;
         }
     
         // Mark the current buffer position
@@ -501,8 +491,9 @@ public class ChunkedInputFilter implements InputFilter {
                 }
     
                 chr = buf[pos];
-                if (chr == Constants.CR || chr == Constants.LF) {
-                    parseCRLF(true);
+                if (chr == Constants.CR) {
+                    // Skip
+                } else if (chr == Constants.LF) {
                     eol = true;
                 } else if (chr == Constants.SP) {
                     trailingHeaders.append(chr);
@@ -511,9 +502,8 @@ public class ChunkedInputFilter implements InputFilter {
                     lastSignificantChar = trailingHeaders.getEnd();
                 }
     
-                if (!eol) {
-                    pos++;
-                }
+                pos++;
+    
             }
     
             // Checking the first character of the new line. If the character
