@@ -14,7 +14,6 @@
  *  See the License for the specific language governing permissions and
  *  limitations under the License.
  */
-
 package org.apache.tomcat.util.http;
 
 import java.io.PrintWriter;
@@ -36,9 +35,9 @@ public final class Cookies { // extends MultiMap {
 
     private static final org.apache.juli.logging.Log log=
         org.apache.juli.logging.LogFactory.getLog(Cookies.class );
-    
+
     // expected average number of cookies per request
-    public static final int INITIAL_SIZE=4; 
+    public static final int INITIAL_SIZE=4;
     ServerCookie scookies[]=new ServerCookie[INITIAL_SIZE];
     int cookieCount=0;
     boolean unprocessed=true;
@@ -61,8 +60,9 @@ public final class Cookies { // extends MultiMap {
      */
     public void recycle() {
             for( int i=0; i< cookieCount; i++ ) {
-            if( scookies[i]!=null )
+            if( scookies[i]!=null ) {
                 scookies[i].recycle();
+            }
         }
         cookieCount=0;
         unprocessed=true;
@@ -84,7 +84,7 @@ public final class Cookies { // extends MultiMap {
     }
 
     // -------------------- Indexed access --------------------
-    
+
     public ServerCookie getCookie( int idx ) {
         if( unprocessed ) {
             getCookieCount(); // will also update the cookies
@@ -112,7 +112,7 @@ public final class Cookies { // extends MultiMap {
             System.arraycopy( scookies, 0, scookiesTmp, 0, cookieCount);
             scookies=scookiesTmp;
         }
-        
+
         ServerCookie c = scookies[cookieCount];
         if( c==null ) {
             c= new ServerCookie();
@@ -123,20 +123,24 @@ public final class Cookies { // extends MultiMap {
     }
 
 
-    // code from CookieTools 
+    // code from CookieTools
 
     /** Add all Cookie found in the headers of a request.
      */
     public  void processCookies( MimeHeaders headers ) {
         if( headers==null )
+         {
             return;// nothing to process
+        }
         // process each "cookie" header
         int pos=0;
         while( pos>=0 ) {
             // Cookie2: version ? not needed
             pos=headers.findHeader( "Cookie", pos );
             // no more cookie headers headers
-            if( pos<0 ) break;
+            if( pos<0 ) {
+                break;
+            }
 
             MessageBytes cookieValue=headers.getValue( pos );
             if( cookieValue==null || cookieValue.isNull() ) {
@@ -151,8 +155,9 @@ public final class Cookies { // extends MultiMap {
                         e);
                 cookieValue.toBytes();
             }
-            if(log.isDebugEnabled())
+            if(log.isDebugEnabled()) {
                 log.debug("Cookies: Parsing b[]: " + cookieValue.toString());
+            }
             ByteChunk bc=cookieValue.getByteChunk();
             processCookieHeader( bc.getBytes(),
                                  bc.getOffset(),
@@ -175,7 +180,7 @@ public final class Cookies { // extends MultiMap {
         }
         return true;
     }
-    
+
 
     /**
      * Returns true if the byte is a whitespace character as
@@ -186,7 +191,7 @@ public final class Cookies { // extends MultiMap {
         // This switch statement is slightly slower
         // for my vm than the if statement.
         // Java(TM) 2 Runtime Environment, Standard Edition (build 1.5.0_07-164)
-        /* 
+        /*
         switch (c) {
         case ' ':;
         case '\t':;
@@ -198,10 +203,11 @@ public final class Cookies { // extends MultiMap {
             return false;
         }
         */
-       if (c == ' ' || c == '\t' || c == '\n' || c == '\r' || c == '\f')
-           return true;
-       else
-           return false;
+       if (c == ' ' || c == '\t' || c == '\n' || c == '\r' || c == '\f') {
+        return true;
+    } else {
+        return false;
+    }
     }
 
     /**
@@ -219,7 +225,7 @@ public final class Cookies { // extends MultiMap {
         int end = bc.getEnd();
         int dest = src;
         byte[] buffer = bc.getBuffer();
-        
+
         while (src < end) {
             if (buffer[src] == '\\' && src < end && buffer[src+1]  == '"') {
                 src++;
@@ -238,7 +244,9 @@ public final class Cookies { // extends MultiMap {
      * JVK
      */
     protected final void processCookieHeader(byte bytes[], int off, int len){
-        if( len<=0 || bytes==null ) return;
+        if( len<=0 || bytes==null ) {
+            return;
+        }
         int end=off+len;
         int pos=off;
         int nameStart=0;
@@ -255,15 +263,16 @@ public final class Cookies { // extends MultiMap {
             isQuoted = false;
 
             // Skip whitespace and non-token characters (separators)
-            while (pos < end && 
+            while (pos < end &&
                    (CookieSupport.isHttpSeparator((char) bytes[pos]) &&
                            !CookieSupport.ALLOW_HTTP_SEPARATORS_IN_V0 ||
                     CookieSupport.isV0Separator((char) bytes[pos]) ||
-                    isWhiteSpace(bytes[pos]))) 
-                {pos++; } 
+                    isWhiteSpace(bytes[pos])))
+                {pos++; }
 
-            if (pos >= end)
+            if (pos >= end) {
                 return;
+            }
 
             // Detect Special cookies
             if (bytes[pos] == '$') {
@@ -271,27 +280,28 @@ public final class Cookies { // extends MultiMap {
                 pos++;
             }
 
-            // Get the cookie/attribute name. This must be a token            
-            valueEnd = valueStart = nameStart = pos; 
+            // Get the cookie/attribute name. This must be a token
+            valueEnd = valueStart = nameStart = pos;
             pos = nameEnd = getTokenEndPosition(bytes,pos,end,version,true);
 
             // Skip whitespace
-            while (pos < end && isWhiteSpace(bytes[pos])) {pos++; } 
-         
+            while (pos < end && isWhiteSpace(bytes[pos])) {pos++; }
+
 
             // Check for an '=' -- This could also be a name-only
             // cookie at the end of the cookie header, so if we
             // are past the end of the header, but we have a name
             // skip to the name-only part.
-            if (pos < (end - 1) && bytes[pos] == '=') {                
+            if (pos < (end - 1) && bytes[pos] == '=') {
 
                 // Skip whitespace
                 do {
                     pos++;
-                } while (pos < end && isWhiteSpace(bytes[pos])); 
+                } while (pos < end && isWhiteSpace(bytes[pos]));
 
-                if (pos >= end)
+                if (pos >= end) {
                     return;
+                }
 
                 // Determine what type of value this is, quoted value,
                 // token, name-only with an '=', or other (bad)
@@ -299,18 +309,19 @@ public final class Cookies { // extends MultiMap {
                 case '"': // Quoted Value
                     isQuoted = true;
                     valueStart=pos + 1; // strip "
-                    // getQuotedValue returns the position before 
+                    // getQuotedValue returns the position before
                     // at the last quote. This must be dealt with
                     // when the bytes are copied into the cookie
-                    valueEnd=getQuotedValueEndPosition(bytes, 
+                    valueEnd=getQuotedValueEndPosition(bytes,
                                                        valueStart, end);
                     // We need pos to advance
-                    pos = valueEnd; 
-                    // Handles cases where the quoted value is 
-                    // unterminated and at the end of the header, 
+                    pos = valueEnd;
+                    // Handles cases where the quoted value is
+                    // unterminated and at the end of the header,
                     // e.g. [myname="value]
-                    if (pos >= end)
+                    if (pos >= end) {
                         return;
+                    }
                     break;
                 case ';':
                 case ',':
@@ -339,15 +350,15 @@ public final class Cookies { // extends MultiMap {
                         // not valid.
                         log.info("Cookies: Invalid cookie. " +
                                 "Value not a token or quoted value");
-                        while (pos < end && bytes[pos] != ';' && 
-                               bytes[pos] != ',') 
+                        while (pos < end && bytes[pos] != ';' &&
+                               bytes[pos] != ',')
                             {pos++; }
                         pos++;
-                        // Make sure no special avpairs can be attributed to 
+                        // Make sure no special avpairs can be attributed to
                         // the previous cookie by setting the current cookie
                         // to null
                         sc = null;
-                        continue;                        
+                        continue;
                     }
                 }
             } else {
@@ -356,30 +367,30 @@ public final class Cookies { // extends MultiMap {
                 pos = nameEnd;
 
             }
-          
+
             // We should have an avpair or name-only cookie at this
             // point. Perform some basic checks to make sure we are
             // in a good state.
-  
+
             // Skip whitespace
             while (pos < end && isWhiteSpace(bytes[pos])) {pos++; }
 
 
             // Make sure that after the cookie we have a separator. This
             // is only important if this is not the last cookie pair
-            while (pos < end && bytes[pos] != ';' && bytes[pos] != ',') { 
+            while (pos < end && bytes[pos] != ';' && bytes[pos] != ',') {
                 pos++;
             }
-            
+
             pos++;
 
-            // All checks passed. Add the cookie, start with the 
+            // All checks passed. Add the cookie, start with the
             // special avpairs first
             if (isSpecial) {
                 isSpecial = false;
                 // $Version must be the first avpair in the cookie header
                 // (sc must be null)
-                if (equals( "Version", bytes, nameStart, nameEnd) && 
+                if (equals( "Version", bytes, nameStart, nameEnd) &&
                     sc == null) {
                     // Set version
                     if( bytes[valueStart] =='1' && valueEnd == (valueStart+1)) {
@@ -388,8 +399,8 @@ public final class Cookies { // extends MultiMap {
                         // unknown version (Versioning is not very strict)
                     }
                     continue;
-                } 
-                
+                }
+
                 // We need an active cookie for Path/Port/etc.
                 if (sc == null) {
                     continue;
@@ -401,22 +412,22 @@ public final class Cookies { // extends MultiMap {
                                            valueStart,
                                            valueEnd-valueStart);
                     continue;
-                } 
+                }
 
                 if (equals( "Path", bytes, nameStart, nameEnd)) {
                     sc.getPath().setBytes( bytes,
                                            valueStart,
                                            valueEnd-valueStart);
                     continue;
-                } 
+                }
 
                 // v2 cookie attributes - skip them
                 if (equals( "Port", bytes, nameStart, nameEnd)) {
                     continue;
-                } 
+                }
                 if (equals( "CommentURL", bytes, nameStart, nameEnd)) {
                     continue;
-                } 
+                }
 
                 // Unknown cookie, complain
                 log.info("Cookies: Unknown Special Cookie");
@@ -431,7 +442,7 @@ public final class Cookies { // extends MultiMap {
                 sc.setVersion( version );
                 sc.getName().setBytes( bytes, nameStart,
                                        nameEnd-nameStart);
-                
+
                 if (valueStart != -1) { // Normal AVPair
                     sc.getValue().setBytes( bytes, valueStart,
                             valueEnd-valueStart);
@@ -441,7 +452,7 @@ public final class Cookies { // extends MultiMap {
                     }
                 } else {
                     // Name Only
-                    sc.getValue().setString(""); 
+                    sc.getValue().setString("");
                 }
                 continue;
             }
@@ -466,13 +477,14 @@ public final class Cookies { // extends MultiMap {
                          CookieSupport.ALLOW_EQUALS_IN_VALUE)) {
             pos++;
         }
-        
-        if (pos > end)
+
+        if (pos > end) {
             return end;
+        }
         return pos;
     }
 
-    /** 
+    /**
      * Given a starting position after an initial quote character, this gets
      * the position of the end quote. This escapes anything after a '\' char
      * JVK RFC 2616
@@ -481,7 +493,7 @@ public final class Cookies { // extends MultiMap {
         int pos = off;
         while (pos < end) {
             if (bytes[pos] == '"') {
-                return pos;                
+                return pos;
             } else if (bytes[pos] == '\\' && pos < (end - 1)) {
                 pos+=2;
             } else {
