@@ -35,7 +35,6 @@ import javax.management.MBeanOperationInfo;
 import javax.management.ReflectionException;
 import javax.management.RuntimeOperationsException;
 import javax.management.ServiceNotFoundException;
-//import javax.management.modelmbean.InvalidTargetObjectTypeException;
 
 
 /**
@@ -43,15 +42,17 @@ import javax.management.ServiceNotFoundException;
  * descriptor.</p>
  *
  * @author Craig R. McClanahan
- * @version $Revision: 610929 $ $Date: 2008-01-10 22:04:31 +0100 (Do, 10. Jan 2008) $
+ * @version $Id: ManagedBean.java 1176157 2011-09-27 01:21:08Z kkolinko $
  */
 
-public class ManagedBean implements java.io.Serializable
-{
+public class ManagedBean implements java.io.Serializable {
+
+    private static final long serialVersionUID = 1L;
+    
     private static final String BASE_MBEAN = "org.apache.tomcat.util.modeler.BaseModelMBean";
     // ----------------------------------------------------- Instance Variables
-    static final Object[] NO_ARGS_PARAM=new Object[0];
-    static final Class[] NO_ARGS_PARAM_SIG=new Class[0];
+    static final Object[] NO_ARGS_PARAM = new Object[0];
+    static final Class<?>[] NO_ARGS_PARAM_SIG = new Class[0];
 
 
     /**
@@ -59,10 +60,12 @@ public class ManagedBean implements java.io.Serializable
      * to this <code>ManagedBean</code> instance.
      */
     transient MBeanInfo info = null;
-    // Map<AttributeInfo>
-    private Map attributes = new HashMap();
-    //Map<OperationInfo>
-    private Map operations = new HashMap();
+
+    private Map<String,AttributeInfo> attributes =
+        new HashMap<String,AttributeInfo>();
+
+    private Map<String,OperationInfo> operations =
+        new HashMap<String,OperationInfo>();
     
     protected String className = BASE_MBEAN;
     //protected ConstructorInfo constructors[] = new ConstructorInfo[0];
@@ -302,8 +305,8 @@ public class ManagedBean implements java.io.Serializable
      *
      * @exception InstanceNotFoundException if the managed resource
      *  object cannot be found
-     * @exception InvalidTargetObjectTypeException if our MBean cannot
-     *  handle object references (should never happen)
+     * @exception javax.management.modelmbean.InvalidTargetObjectTypeException
+     *  if our MBean cannot handle object references (should never happen)
      * @exception MBeanException if a problem occurs instantiating the
      *  <code>ModelMBean</code> instance
      * @exception RuntimeOperationsException if a JMX runtime error occurs
@@ -330,8 +333,8 @@ public class ManagedBean implements java.io.Serializable
      *
      * @exception InstanceNotFoundException if the managed resource
      *  object cannot be found
-     * @exception InvalidTargetObjectTypeException if our MBean cannot
-     *  handle object references (should never happen)
+     * @exception javax.management.modelmbean.InvalidTargetObjectTypeException
+     *  if our MBean cannot handle object references (should never happen)
      * @exception MBeanException if a problem occurs instantiating the
      *  <code>ModelMBean</code> instance
      * @exception RuntimeOperationsException if a JMX runtime error occurs
@@ -347,7 +350,7 @@ public class ManagedBean implements java.io.Serializable
             // Skip introspection
             mbean = new BaseModelMBean();
         } else {
-            Class clazz = null;
+            Class<?> clazz = null;
             Exception ex = null;
             try {
                 clazz = Class.forName(getClassName());
@@ -458,9 +461,10 @@ public class ManagedBean implements java.io.Serializable
     /**
      * Return a string representation of this managed bean.
      */
+    @Override
     public String toString() {
 
-        StringBuffer sb = new StringBuffer("ManagedBean[");
+        StringBuilder sb = new StringBuilder("ManagedBean[");
         sb.append("name=");
         sb.append(name);
         sb.append(", className=");
@@ -479,148 +483,141 @@ public class ManagedBean implements java.io.Serializable
     }
 
     Method getGetter(String aname, BaseModelMBean mbean, Object resource) 
-            throws AttributeNotFoundException, MBeanException, ReflectionException {
-        // TODO: do we need caching ? JMX is for management, it's not supposed to require lots of performance.
-        Method m=null; // (Method)getAttMap.get( name );
+            throws AttributeNotFoundException, ReflectionException {
 
-        if( m==null ) {
-            AttributeInfo attrInfo = (AttributeInfo)attributes.get(aname);
-            // Look up the actual operation to be used
-            if (attrInfo == null)
-                throw new AttributeNotFoundException(" Cannot find attribute " + aname + " for " + resource);
-            
-            String getMethod = attrInfo.getGetMethod();
-            if (getMethod == null)
-                throw new AttributeNotFoundException("Cannot find attribute " + aname + " get method name");
+        Method m = null;
 
-            Object object = null;
-            NoSuchMethodException exception = null;
-            try {
-                object = mbean;
-                m = object.getClass().getMethod(getMethod, NO_ARGS_PARAM_SIG);
-            } catch (NoSuchMethodException e) {
-                exception = e;;
-            }
-            if( m== null && resource != null ) {
-                try {
-                    object = resource;
-                    m = object.getClass().getMethod(getMethod, NO_ARGS_PARAM_SIG);
-                    exception=null;
-                } catch (NoSuchMethodException e) {
-                    exception = e;
-                }
-            }
-            if( exception != null )
-                throw new ReflectionException(exception,
-                                              "Cannot find getter method " + getMethod);
-            //getAttMap.put( name, m );
+        AttributeInfo attrInfo = attributes.get(aname);
+        // Look up the actual operation to be used
+        if (attrInfo == null)
+            throw new AttributeNotFoundException(" Cannot find attribute " + aname + " for " + resource);
+        
+        String getMethod = attrInfo.getGetMethod();
+        if (getMethod == null)
+            throw new AttributeNotFoundException("Cannot find attribute " + aname + " get method name");
+
+        Object object = null;
+        NoSuchMethodException exception = null;
+        try {
+            object = mbean;
+            m = object.getClass().getMethod(getMethod, NO_ARGS_PARAM_SIG);
+        } catch (NoSuchMethodException e) {
+            exception = e;
         }
+        if( m== null && resource != null ) {
+            try {
+                object = resource;
+                m = object.getClass().getMethod(getMethod, NO_ARGS_PARAM_SIG);
+                exception=null;
+            } catch (NoSuchMethodException e) {
+                exception = e;
+            }
+        }
+        if( exception != null )
+            throw new ReflectionException(exception,
+                                          "Cannot find getter method " + getMethod);
 
         return m;
     }
 
     public Method getSetter(String aname, BaseModelMBean bean, Object resource) 
-            throws AttributeNotFoundException, MBeanException, ReflectionException {
-        // Cache may be needed for getters, but it is a really bad idea for setters, this is far 
-        // less frequent.
-        Method m=null;//(Method)setAttMap.get( name );
+            throws AttributeNotFoundException, ReflectionException {
 
-        if( m==null ) {
-            AttributeInfo attrInfo = (AttributeInfo)attributes.get(aname);
-            if (attrInfo == null)
-                throw new AttributeNotFoundException(" Cannot find attribute " + aname);
+        Method m = null;
 
-            // Look up the actual operation to be used
-            String setMethod = attrInfo.getSetMethod();
-            if (setMethod == null)
-                throw new AttributeNotFoundException("Cannot find attribute " + aname + " set method name");
+        AttributeInfo attrInfo = attributes.get(aname);
+        if (attrInfo == null)
+            throw new AttributeNotFoundException(" Cannot find attribute " + aname);
 
-            String argType=attrInfo.getType();
+        // Look up the actual operation to be used
+        String setMethod = attrInfo.getSetMethod();
+        if (setMethod == null)
+            throw new AttributeNotFoundException("Cannot find attribute " + aname + " set method name");
 
-            Class signature[] = new Class[] { BaseModelMBean.getAttributeClass( argType ) };
+        String argType=attrInfo.getType();
 
-            Object object = null;
-            NoSuchMethodException exception = null;
-            try {
-                object = bean;
-                m = object.getClass().getMethod(setMethod, signature);
-            } catch (NoSuchMethodException e) {
-                exception = e;;
-            }
-            if( m== null && resource != null ) {
-                try {
-                    object = resource;
-                    m = object.getClass().getMethod(setMethod, signature);
-                    exception=null;
-                } catch (NoSuchMethodException e) {
-                    exception = e;
-                }
-            }
-            if( exception != null )
-                throw new ReflectionException(exception,
-                                              "Cannot find setter method " + setMethod +
-                        " " + resource);
-            //setAttMap.put( name, m );
+        Class<?> signature[] =
+            new Class[] { BaseModelMBean.getAttributeClass( argType ) };
+
+        Object object = null;
+        NoSuchMethodException exception = null;
+        try {
+            object = bean;
+            m = object.getClass().getMethod(setMethod, signature);
+        } catch (NoSuchMethodException e) {
+            exception = e;
         }
+        if( m== null && resource != null ) {
+            try {
+                object = resource;
+                m = object.getClass().getMethod(setMethod, signature);
+                exception=null;
+            } catch (NoSuchMethodException e) {
+                exception = e;
+            }
+        }
+        if( exception != null )
+            throw new ReflectionException(exception,
+                                          "Cannot find setter method " + setMethod +
+                    " " + resource);
 
         return m;
     }
 
     public Method getInvoke(String aname, Object[] params, String[] signature, BaseModelMBean bean, Object resource) 
             throws MBeanException, ReflectionException {
+
         Method method = null;
-        if (method == null) {
-            if (params == null)
-                params = new Object[0];
-            if (signature == null)
-                signature = new String[0];
-            if (params.length != signature.length)
-                throw new RuntimeOperationsException(
-                        new IllegalArgumentException(
-                                "Inconsistent arguments and signature"),
-                        "Inconsistent arguments and signature");
+        
+        if (params == null)
+            params = new Object[0];
+        if (signature == null)
+            signature = new String[0];
+        if (params.length != signature.length)
+            throw new RuntimeOperationsException(
+                    new IllegalArgumentException(
+                            "Inconsistent arguments and signature"),
+                    "Inconsistent arguments and signature");
 
-            // Acquire the ModelMBeanOperationInfo information for
-            // the requested operation
-            OperationInfo opInfo = (OperationInfo)operations.get(aname);
-            if (opInfo == null)
-                throw new MBeanException(new ServiceNotFoundException(
-                        "Cannot find operation " + aname),
-                        "Cannot find operation " + aname);
+        // Acquire the ModelMBeanOperationInfo information for
+        // the requested operation
+        OperationInfo opInfo = operations.get(aname);
+        if (opInfo == null)
+            throw new MBeanException(new ServiceNotFoundException(
+                    "Cannot find operation " + aname),
+                    "Cannot find operation " + aname);
 
-            // Prepare the signature required by Java reflection APIs
-            // FIXME - should we use the signature from opInfo?
-            Class types[] = new Class[signature.length];
-            for (int i = 0; i < signature.length; i++) {
-                types[i] = BaseModelMBean.getAttributeClass(signature[i]);
-            }
-
-            // Locate the method to be invoked, either in this MBean itself
-            // or in the corresponding managed resource
-            // FIXME - Accessible methods in superinterfaces?
-            Object object = null;
-            Exception exception = null;
-            try {
-                object = bean;
-                method = object.getClass().getMethod(aname, types);
-            } catch (NoSuchMethodException e) {
-                exception = e;
-                ;
-            }
-            try {
-                if ((method == null) && (resource != null)) {
-                    object = resource;
-                    method = object.getClass().getMethod(aname, types);
-                }
-            } catch (NoSuchMethodException e) {
-                exception = e;
-            }
-            if (method == null) {
-                throw new ReflectionException(exception, "Cannot find method "
-                        + aname + " with this signature");
-            }
-            // invokeAttMap.put(mkey, method);
+        // Prepare the signature required by Java reflection APIs
+        // FIXME - should we use the signature from opInfo?
+        Class<?> types[] = new Class[signature.length];
+        for (int i = 0; i < signature.length; i++) {
+            types[i] = BaseModelMBean.getAttributeClass(signature[i]);
         }
+
+        // Locate the method to be invoked, either in this MBean itself
+        // or in the corresponding managed resource
+        // FIXME - Accessible methods in superinterfaces?
+        Object object = null;
+        Exception exception = null;
+        try {
+            object = bean;
+            method = object.getClass().getMethod(aname, types);
+        } catch (NoSuchMethodException e) {
+            exception = e;
+        }
+        try {
+            if ((method == null) && (resource != null)) {
+                object = resource;
+                method = object.getClass().getMethod(aname, types);
+            }
+        } catch (NoSuchMethodException e) {
+            exception = e;
+        }
+        if (method == null) {
+            throw new ReflectionException(exception, "Cannot find method "
+                    + aname + " with this signature");
+        }
+
         return method;
     }
 
