@@ -31,10 +31,10 @@ import org.apache.tomcat.jdbc.pool.PoolProperties.InterceptorDefinition;
 
 /**
  *
- * The DataSource proxy lets us implements methods that don't exist in the current 
+ * The DataSource proxy lets us implements methods that don't exist in the current
  * compiler JDK but might be methods that are part of a future JDK DataSource interface.
  * <br/>
- * It's a trick to work around compiler issues when implementing interfaces. For example, 
+ * It's a trick to work around compiler issues when implementing interfaces. For example,
  * I could put in Java 6 methods of javax.sql.DataSource here, and compile it with JDK 1.5
  * and still be able to run under Java 6 without getting NoSuchMethodException.
  *
@@ -44,15 +44,15 @@ import org.apache.tomcat.jdbc.pool.PoolProperties.InterceptorDefinition;
 
 public class DataSourceProxy implements PoolConfiguration {
     private static final Log log = LogFactory.getLog(DataSourceProxy.class);
-    
+
     protected volatile ConnectionPool pool = null;
-    
+
     protected PoolConfiguration poolProperties = null;
 
     public DataSourceProxy() {
         this(new PoolProperties());
     }
-    
+
     public DataSourceProxy(PoolConfiguration poolProperties) {
         if (poolProperties == null) throw new NullPointerException("PoolConfiguration can not be null.");
         this.poolProperties = poolProperties;
@@ -99,7 +99,7 @@ public class DataSourceProxy implements PoolConfiguration {
             return pCreatePool();
         }
     }
-    
+
     /**
      * Sets up the connection pool, by creating a pooling driver.
      * @return Driver
@@ -123,10 +123,10 @@ public class DataSourceProxy implements PoolConfiguration {
             return createPool().getConnection();
         return pool.getConnection();
     }
-    
+
     /**
      * Invokes an sync operation to retrieve the connection.
-     * @return a Future containing a reference to the connection when it becomes available 
+     * @return a Future containing a reference to the connection when it becomes available
      * @throws SQLException
      */
     public Future<Connection> getConnectionAsync() throws SQLException {
@@ -134,9 +134,9 @@ public class DataSourceProxy implements PoolConfiguration {
             return createPool().getConnectionAsync();
         return pool.getConnectionAsync();
     }
-    
+
     /**
-     * {@link javax.sql.XADataSource#getXAConnection()} 
+     * {@link javax.sql.XADataSource#getXAConnection()}
      */
     public XAConnection getXAConnection() throws SQLException {
         Connection con = getConnection();
@@ -147,9 +147,9 @@ public class DataSourceProxy implements PoolConfiguration {
             throw new SQLException("Connection from pool does not implement javax.sql.XAConnection");
         }
     }
-    
+
     /**
-     * {@link javax.sql.XADataSource#getXAConnection(String, String)} 
+     * {@link javax.sql.XADataSource#getXAConnection(String, String)}
      */
     public XAConnection getXAConnection(String username, String password) throws SQLException {
         Connection con = getConnection(username, password);
@@ -176,12 +176,12 @@ public class DataSourceProxy implements PoolConfiguration {
                                                 String password) throws SQLException {
         return (javax.sql.PooledConnection) getConnection();
     }
-    
+
     public ConnectionPool getPool() {
         return pool;
     }
 
-    
+
     public void close() {
         close(false);
     }
@@ -205,7 +205,8 @@ public class DataSourceProxy implements PoolConfiguration {
         else return p.getSize();
     }
 
-    
+
+    @Override
     public String toString() {
         return super.toString()+"{"+getPoolProperties()+"}";
     }
@@ -214,194 +215,217 @@ public class DataSourceProxy implements PoolConfiguration {
 /*-----------------------------------------------------------------------*/
 //      PROPERTIES WHEN NOT USED WITH FACTORY
 /*------------------------------------------------------------------------*/
-   
-    /** 
+
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public String getPoolName() {
         return pool.getName();
     }
-   
-   
+
+
     public void setPoolProperties(PoolConfiguration poolProperties) {
         this.poolProperties = poolProperties;
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setDriverClassName(String driverClassName) {
         this.poolProperties.setDriverClassName(driverClassName);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setInitialSize(int initialSize) {
         this.poolProperties.setInitialSize(initialSize);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setInitSQL(String initSQL) {
         this.poolProperties.setInitSQL(initSQL);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setLogAbandoned(boolean logAbandoned) {
         this.poolProperties.setLogAbandoned(logAbandoned);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setMaxActive(int maxActive) {
         this.poolProperties.setMaxActive(maxActive);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setMaxIdle(int maxIdle) {
         this.poolProperties.setMaxIdle(maxIdle);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setMaxWait(int maxWait) {
         this.poolProperties.setMaxWait(maxWait);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setMinEvictableIdleTimeMillis(int minEvictableIdleTimeMillis) {
         this.poolProperties.setMinEvictableIdleTimeMillis(minEvictableIdleTimeMillis);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setMinIdle(int minIdle) {
         this.poolProperties.setMinIdle(minIdle);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setNumTestsPerEvictionRun(int numTestsPerEvictionRun) {
         this.poolProperties.setNumTestsPerEvictionRun(numTestsPerEvictionRun);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setPassword(String password) {
         this.poolProperties.setPassword(password);
         this.poolProperties.getDbProperties().setProperty("password",this.poolProperties.getPassword());
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setRemoveAbandoned(boolean removeAbandoned) {
         this.poolProperties.setRemoveAbandoned(removeAbandoned);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setRemoveAbandonedTimeout(int removeAbandonedTimeout) {
         this.poolProperties.setRemoveAbandonedTimeout(removeAbandonedTimeout);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setTestOnBorrow(boolean testOnBorrow) {
         this.poolProperties.setTestOnBorrow(testOnBorrow);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setTestOnConnect(boolean testOnConnect) {
         this.poolProperties.setTestOnConnect(testOnConnect);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setTestOnReturn(boolean testOnReturn) {
         this.poolProperties.setTestOnReturn(testOnReturn);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setTestWhileIdle(boolean testWhileIdle) {
         this.poolProperties.setTestWhileIdle(testWhileIdle);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setTimeBetweenEvictionRunsMillis(int timeBetweenEvictionRunsMillis) {
         this.poolProperties.setTimeBetweenEvictionRunsMillis(timeBetweenEvictionRunsMillis);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setUrl(String url) {
         this.poolProperties.setUrl(url);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setUsername(String username) {
         this.poolProperties.setUsername(username);
         this.poolProperties.getDbProperties().setProperty("user",getPoolProperties().getUsername());
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setValidationInterval(long validationInterval) {
         this.poolProperties.setValidationInterval(validationInterval);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setValidationQuery(String validationQuery) {
         this.poolProperties.setValidationQuery(validationQuery);
     }
@@ -409,71 +433,80 @@ public class DataSourceProxy implements PoolConfiguration {
     /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setValidatorClassName(String className) {
         this.poolProperties.setValidatorClassName(className);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setJdbcInterceptors(String interceptors) {
         this.getPoolProperties().setJdbcInterceptors(interceptors);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setJmxEnabled(boolean enabled) {
         this.getPoolProperties().setJmxEnabled(enabled);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setFairQueue(boolean fairQueue) {
         this.getPoolProperties().setFairQueue(fairQueue);
     }
-    
-    /** 
+
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setUseLock(boolean useLock) {
         this.getPoolProperties().setUseLock(useLock);
     }
-    
-    /** 
+
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setDefaultCatalog(String catalog) {
         this.getPoolProperties().setDefaultCatalog(catalog);
     }
-    
-    /** 
+
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setDefaultAutoCommit(Boolean autocommit) {
         this.getPoolProperties().setDefaultAutoCommit(autocommit);
     }
-    
-    /** 
+
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setDefaultTransactionIsolation(int defaultTransactionIsolation) {
         this.getPoolProperties().setDefaultTransactionIsolation(defaultTransactionIsolation);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setConnectionProperties(String properties) {
         try {
             java.util.Properties prop = DataSourceFactory
@@ -490,11 +523,12 @@ public class DataSourceProxy implements PoolConfiguration {
             throw new RuntimeException(x);
         }
     }
-    
-    /** 
+
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setUseEquals(boolean useEquals) {
         this.getPoolProperties().setUseEquals(useEquals);
     }
@@ -538,25 +572,27 @@ public class DataSourceProxy implements PoolConfiguration {
             poolProperties.setMaxWait(1000 * i);
         }
 
-    }    
-    
-    
+    }
+
+
     /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getSuspectTimeout() {
-        return getPoolProperties().getSuspectTimeout(); 
+        return getPoolProperties().getSuspectTimeout();
     }
 
     /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setSuspectTimeout(int seconds) {
         getPoolProperties().setSuspectTimeout(seconds);
     }
-    
+
   //===============================================================================
 //  Expose JMX attributes through Tomcat's dynamic reflection
 //===============================================================================
@@ -612,7 +648,7 @@ public class DataSourceProxy implements PoolConfiguration {
             throw new RuntimeException(x);
         }
     }
-    
+
     /**
      * @return number of connections in use by the application
      * {@link DataSource#getActive()}
@@ -652,136 +688,152 @@ public class DataSourceProxy implements PoolConfiguration {
         }catch (SQLException x) {
             throw new RuntimeException(x);
         }
-    }    
+    }
     //=========================================================
     //  PROPERTIES / CONFIGURATION
-    //=========================================================    
-    
-    /** 
+    //=========================================================
+
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public String getConnectionProperties() {
         return getPoolProperties().getConnectionProperties();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public Properties getDbProperties() {
         return getPoolProperties().getDbProperties();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public String getDefaultCatalog() {
         return getPoolProperties().getDefaultCatalog();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getDefaultTransactionIsolation() {
         return getPoolProperties().getDefaultTransactionIsolation();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public String getDriverClassName() {
         return getPoolProperties().getDriverClassName();
     }
 
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getInitialSize() {
         return getPoolProperties().getInitialSize();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public String getInitSQL() {
         return getPoolProperties().getInitSQL();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public String getJdbcInterceptors() {
         return getPoolProperties().getJdbcInterceptors();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getMaxActive() {
         return getPoolProperties().getMaxActive();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getMaxIdle() {
         return getPoolProperties().getMaxIdle();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getMaxWait() {
         return getPoolProperties().getMaxWait();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getMinEvictableIdleTimeMillis() {
         return getPoolProperties().getMinEvictableIdleTimeMillis();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getMinIdle() {
         return getPoolProperties().getMinIdle();
     }
-    
-    /** 
+
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public long getMaxAge() {
         return getPoolProperties().getMaxAge();
-    }    
+    }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public String getName() {
         return getPoolProperties().getName();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getNumTestsPerEvictionRun() {
         return getPoolProperties().getNumTestsPerEvictionRun();
     }
@@ -789,55 +841,62 @@ public class DataSourceProxy implements PoolConfiguration {
     /**
      * @return DOES NOT RETURN THE PASSWORD, IT WOULD SHOW UP IN JMX
      */
+    @Override
     public String getPassword() {
         return "Password not available as DataSource/JMX operation.";
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getRemoveAbandonedTimeout() {
         return getPoolProperties().getRemoveAbandonedTimeout();
     }
 
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getTimeBetweenEvictionRunsMillis() {
         return getPoolProperties().getTimeBetweenEvictionRunsMillis();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public String getUrl() {
         return getPoolProperties().getUrl();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public String getUsername() {
         return getPoolProperties().getUsername();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public long getValidationInterval() {
         return getPoolProperties().getValidationInterval();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public String getValidationQuery() {
         return getPoolProperties().getValidationQuery();
     }
@@ -845,7 +904,8 @@ public class DataSourceProxy implements PoolConfiguration {
     /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public String getValidatorClassName() {
         return getPoolProperties().getValidatorClassName();
     }
@@ -853,253 +913,285 @@ public class DataSourceProxy implements PoolConfiguration {
     /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public Validator getValidator() {
         return getPoolProperties().getValidator();
     }
-    
-    /** 
+
+    /**
      * {@inheritDoc}
      */
+    @Override
     public void setValidator(Validator validator) {
         getPoolProperties().setValidator(validator);
     }
 
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public boolean isAccessToUnderlyingConnectionAllowed() {
         return getPoolProperties().isAccessToUnderlyingConnectionAllowed();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public Boolean isDefaultAutoCommit() {
         return getPoolProperties().isDefaultAutoCommit();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public Boolean isDefaultReadOnly() {
         return getPoolProperties().isDefaultReadOnly();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public boolean isLogAbandoned() {
         return getPoolProperties().isLogAbandoned();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public boolean isPoolSweeperEnabled() {
         return getPoolProperties().isPoolSweeperEnabled();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public boolean isRemoveAbandoned() {
         return getPoolProperties().isRemoveAbandoned();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getAbandonWhenPercentageFull() {
         return getPoolProperties().getAbandonWhenPercentageFull();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public boolean isTestOnBorrow() {
         return getPoolProperties().isTestOnBorrow();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public boolean isTestOnConnect() {
         return getPoolProperties().isTestOnConnect();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public boolean isTestOnReturn() {
         return getPoolProperties().isTestOnReturn();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public boolean isTestWhileIdle() {
         return getPoolProperties().isTestWhileIdle();
     }
 
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public Boolean getDefaultAutoCommit() {
         return getPoolProperties().getDefaultAutoCommit();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public Boolean getDefaultReadOnly() {
         return getPoolProperties().getDefaultReadOnly();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public InterceptorDefinition[] getJdbcInterceptorsAsArray() {
         return getPoolProperties().getJdbcInterceptorsAsArray();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public boolean getUseLock() {
         return getPoolProperties().getUseLock();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public boolean isFairQueue() {
         return getPoolProperties().isFairQueue();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public boolean isJmxEnabled() {
         return getPoolProperties().isJmxEnabled();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public boolean isUseEquals() {
         return getPoolProperties().isUseEquals();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setAbandonWhenPercentageFull(int percentage) {
         getPoolProperties().setAbandonWhenPercentageFull(percentage);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setAccessToUnderlyingConnectionAllowed(boolean accessToUnderlyingConnectionAllowed) {
         getPoolProperties().setAccessToUnderlyingConnectionAllowed(accessToUnderlyingConnectionAllowed);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setDbProperties(Properties dbProperties) {
         getPoolProperties().setDbProperties(dbProperties);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setDefaultReadOnly(Boolean defaultReadOnly) {
         getPoolProperties().setDefaultReadOnly(defaultReadOnly);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setMaxAge(long maxAge) {
         getPoolProperties().setMaxAge(maxAge);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setName(String name) {
         getPoolProperties().setName(name);
     }
-    
-    /** 
+
+    /**
      * {@inheritDoc}
      */
+    @Override
     public void setDataSource(Object ds) {
         getPoolProperties().setDataSource(ds);
     }
-    
-    /** 
+
+    /**
      * {@inheritDoc}
      */
+    @Override
     public Object getDataSource() {
         return getPoolProperties().getDataSource();
     }
-    
-    
-    /** 
+
+
+    /**
      * {@inheritDoc}
      */
+    @Override
     public void setDataSourceJNDI(String jndiDS) {
         getPoolProperties().setDataSourceJNDI(jndiDS);
     }
-    
-    /** 
+
+    /**
      * {@inheritDoc}
      */
+    @Override
     public String getDataSourceJNDI() {
         return getPoolProperties().getDataSourceJNDI();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
+    @Override
     public boolean isAlternateUsernameAllowed() {
         return getPoolProperties().isAlternateUsernameAllowed();
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
+    @Override
     public void setAlternateUsernameAllowed(boolean alternateUsernameAllowed) {
         getPoolProperties().setAlternateUsernameAllowed(alternateUsernameAllowed);
     }
-    
+
 }
