@@ -47,11 +47,11 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
      * sequence for JMX notifications
      */
     protected AtomicInteger sequence = new AtomicInteger(0);
-    
+
     /**
      * Listeners that are local and interested in our notifications, no need for JMX
      */
-    protected ConcurrentLinkedQueue<NotificationListener> listeners = new ConcurrentLinkedQueue<NotificationListener>(); 
+    protected ConcurrentLinkedQueue<NotificationListener> listeners = new ConcurrentLinkedQueue<NotificationListener>();
 
     public ConnectionPool(org.apache.tomcat.jdbc.pool.ConnectionPool pool) {
         super();
@@ -61,11 +61,11 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
     public org.apache.tomcat.jdbc.pool.ConnectionPool getPool() {
         return pool;
     }
-    
+
     public PoolConfiguration getPoolProperties() {
         return pool.getPoolProperties();
     }
-    
+
     //=================================================================
     //       NOTIFICATION INFO
     //=================================================================
@@ -77,23 +77,23 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
     public static final String SUSPECT_ABANDONED_NOTIFICATION = "SUSPECT CONNETION ABANDONED";
 
     @Override
-    public MBeanNotificationInfo[] getNotificationInfo() { 
+    public MBeanNotificationInfo[] getNotificationInfo() {
         MBeanNotificationInfo[] pres = super.getNotificationInfo();
         MBeanNotificationInfo[] loc = getDefaultNotificationInfo();
         MBeanNotificationInfo[] aug = new MBeanNotificationInfo[pres.length + loc.length];
         if (pres.length>0) System.arraycopy(pres, 0, aug, 0, pres.length);
-        if (loc.length >0) System.arraycopy(loc, 0, aug, pres.length, loc.length);    
-        return aug; 
-    } 
-    
+        if (loc.length >0) System.arraycopy(loc, 0, aug, pres.length, loc.length);
+        return aug;
+    }
+
     public static MBeanNotificationInfo[] getDefaultNotificationInfo() {
-        String[] types = new String[] {NOTIFY_INIT, NOTIFY_CONNECT, NOTIFY_ABANDON, SLOW_QUERY_NOTIFICATION, FAILED_QUERY_NOTIFICATION, SUSPECT_ABANDONED_NOTIFICATION}; 
-        String name = Notification.class.getName(); 
-        String description = "A connection pool error condition was met."; 
-        MBeanNotificationInfo info = new MBeanNotificationInfo(types, name, description); 
+        String[] types = new String[] {NOTIFY_INIT, NOTIFY_CONNECT, NOTIFY_ABANDON, SLOW_QUERY_NOTIFICATION, FAILED_QUERY_NOTIFICATION, SUSPECT_ABANDONED_NOTIFICATION};
+        String name = Notification.class.getName();
+        String description = "A connection pool error condition was met.";
+        MBeanNotificationInfo info = new MBeanNotificationInfo(types, name, description);
         return new MBeanNotificationInfo[] {info};
     }
-    
+
     /**
      * Return true if the notification was sent successfully, false otherwise.
      * @param type
@@ -119,41 +119,47 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
             }
             return false;
         }
-        
+
     }
-    
+
     public void addListener(NotificationListener list) {
         listeners.add(list);
     }
-    
+
     public boolean removeListener(NotificationListener list) {
         return listeners.remove(list);
     }
-    
+
     //=================================================================
     //       POOL STATS
     //=================================================================
 
+    @Override
     public int getSize() {
         return pool.getSize();
     }
 
+    @Override
     public int getIdle() {
         return pool.getIdle();
     }
 
+    @Override
     public int getActive() {
         return pool.getActive();
     }
-    
+
+    @Override
     public int getNumIdle() {
         return getIdle();
     }
-    
+
+    @Override
     public int getNumActive() {
         return getActive();
     }
-    
+
+    @Override
     public int getWaitCount() {
         return pool.getWaitCount();
     }
@@ -161,14 +167,17 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
     //=================================================================
     //       POOL OPERATIONS
     //=================================================================
+    @Override
     public void checkIdle() {
         pool.checkIdle();
     }
 
+    @Override
     public void checkAbandoned() {
         pool.checkAbandoned();
     }
 
+    @Override
     public void testIdle() {
         pool.testAllIdle();
     }
@@ -177,70 +186,86 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
     //=================================================================
     //=========================================================
     //  PROPERTIES / CONFIGURATION
-    //=========================================================    
+    //=========================================================
 
 
+    @Override
     public String getConnectionProperties() {
         return getPoolProperties().getConnectionProperties();
     }
 
+    @Override
     public Properties getDbProperties() {
         return PoolUtilities.cloneWithoutPassword(getPoolProperties().getDbProperties());
     }
 
+    @Override
     public String getDefaultCatalog() {
         return getPoolProperties().getDefaultCatalog();
     }
 
+    @Override
     public int getDefaultTransactionIsolation() {
         return getPoolProperties().getDefaultTransactionIsolation();
     }
 
+    @Override
     public String getDriverClassName() {
         return getPoolProperties().getDriverClassName();
     }
 
 
+    @Override
     public int getInitialSize() {
         return getPoolProperties().getInitialSize();
     }
 
+    @Override
     public String getInitSQL() {
         return getPoolProperties().getInitSQL();
     }
 
+    @Override
     public String getJdbcInterceptors() {
         return getPoolProperties().getJdbcInterceptors();
     }
 
+    @Override
     public int getMaxActive() {
         return getPoolProperties().getMaxActive();
     }
 
+    @Override
     public int getMaxIdle() {
         return getPoolProperties().getMaxIdle();
     }
 
+    @Override
     public int getMaxWait() {
         return getPoolProperties().getMaxWait();
     }
 
+    @Override
     public int getMinEvictableIdleTimeMillis() {
         return getPoolProperties().getMinEvictableIdleTimeMillis();
     }
 
+    @Override
     public int getMinIdle() {
         return getPoolProperties().getMinIdle();
     }
-    
+
+    @Override
     public long getMaxAge() {
         return getPoolProperties().getMaxAge();
-    }    
+    }
 
+    @Override
     public String getName() {
         return this.getPoolName();
     }
 
+    @Override
     public int getNumTestsPerEvictionRun() {
         return getPoolProperties().getNumTestsPerEvictionRun();
     }
@@ -248,31 +273,38 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
     /**
      * @return DOES NOT RETURN THE PASSWORD, IT WOULD SHOW UP IN JMX
      */
+    @Override
     public String getPassword() {
         return "Password not available as DataSource/JMX operation.";
     }
 
+    @Override
     public int getRemoveAbandonedTimeout() {
         return getPoolProperties().getRemoveAbandonedTimeout();
     }
 
 
+    @Override
     public int getTimeBetweenEvictionRunsMillis() {
         return getPoolProperties().getTimeBetweenEvictionRunsMillis();
     }
 
+    @Override
     public String getUrl() {
         return getPoolProperties().getUrl();
     }
 
+    @Override
     public String getUsername() {
         return getPoolProperties().getUsername();
     }
 
+    @Override
     public long getValidationInterval() {
         return getPoolProperties().getValidationInterval();
     }
 
+    @Override
     public String getValidationQuery() {
         return getPoolProperties().getValidationQuery();
     }
@@ -280,7 +312,8 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
     /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public String getValidatorClassName() {
         return getPoolProperties().getValidatorClassName();
     }
@@ -288,293 +321,351 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
     /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public Validator getValidator() {
         return getPoolProperties().getValidator();
     }
 
+    @Override
     public boolean isAccessToUnderlyingConnectionAllowed() {
         return getPoolProperties().isAccessToUnderlyingConnectionAllowed();
     }
 
+    @Override
     public Boolean isDefaultAutoCommit() {
         return getPoolProperties().isDefaultAutoCommit();
     }
 
+    @Override
     public Boolean isDefaultReadOnly() {
         return getPoolProperties().isDefaultReadOnly();
     }
 
+    @Override
     public boolean isLogAbandoned() {
         return getPoolProperties().isLogAbandoned();
     }
 
+    @Override
     public boolean isPoolSweeperEnabled() {
         return getPoolProperties().isPoolSweeperEnabled();
     }
 
+    @Override
     public boolean isRemoveAbandoned() {
         return getPoolProperties().isRemoveAbandoned();
     }
 
+    @Override
     public int getAbandonWhenPercentageFull() {
         return getPoolProperties().getAbandonWhenPercentageFull();
     }
 
+    @Override
     public boolean isTestOnBorrow() {
         return getPoolProperties().isTestOnBorrow();
     }
 
+    @Override
     public boolean isTestOnConnect() {
         return getPoolProperties().isTestOnConnect();
     }
 
+    @Override
     public boolean isTestOnReturn() {
         return getPoolProperties().isTestOnReturn();
     }
 
+    @Override
     public boolean isTestWhileIdle() {
         return getPoolProperties().isTestWhileIdle();
     }
 
 
+    @Override
     public Boolean getDefaultAutoCommit() {
         return getPoolProperties().getDefaultAutoCommit();
     }
 
+    @Override
     public Boolean getDefaultReadOnly() {
         return getPoolProperties().getDefaultReadOnly();
     }
 
+    @Override
     public InterceptorDefinition[] getJdbcInterceptorsAsArray() {
         return getPoolProperties().getJdbcInterceptorsAsArray();
     }
 
+    @Override
     public boolean getUseLock() {
         return getPoolProperties().getUseLock();
     }
 
+    @Override
     public boolean isFairQueue() {
         return getPoolProperties().isFairQueue();
     }
 
+    @Override
     public boolean isJmxEnabled() {
         return getPoolProperties().isJmxEnabled();
     }
 
+    @Override
     public boolean isUseEquals() {
         return getPoolProperties().isUseEquals();
     }
 
+    @Override
     public void setAbandonWhenPercentageFull(int percentage) {
         getPoolProperties().setAbandonWhenPercentageFull(percentage);
     }
 
+    @Override
     public void setAccessToUnderlyingConnectionAllowed(boolean accessToUnderlyingConnectionAllowed) {
         getPoolProperties().setAccessToUnderlyingConnectionAllowed(accessToUnderlyingConnectionAllowed);
     }
 
+    @Override
     public void setDbProperties(Properties dbProperties) {
         getPoolProperties().setDbProperties(dbProperties);
     }
 
+    @Override
     public void setDefaultReadOnly(Boolean defaultReadOnly) {
         getPoolProperties().setDefaultReadOnly(defaultReadOnly);
     }
 
+    @Override
     public void setMaxAge(long maxAge) {
         getPoolProperties().setMaxAge(maxAge);
     }
 
+    @Override
     public void setName(String name) {
         getPoolProperties().setName(name);
     }
 
+    @Override
     public String getPoolName() {
         return getPoolProperties().getName();
     }
-    
 
+
+    @Override
     public void setConnectionProperties(String connectionProperties) {
         getPoolProperties().setConnectionProperties(connectionProperties);
-        
+
     }
 
+    @Override
     public void setDefaultAutoCommit(Boolean defaultAutoCommit) {
         getPoolProperties().setDefaultAutoCommit(defaultAutoCommit);
     }
 
+    @Override
     public void setDefaultCatalog(String defaultCatalog) {
         getPoolProperties().setDefaultCatalog(defaultCatalog);
     }
 
+    @Override
     public void setDefaultTransactionIsolation(int defaultTransactionIsolation) {
         getPoolProperties().setDefaultTransactionIsolation(defaultTransactionIsolation);
     }
 
+    @Override
     public void setDriverClassName(String driverClassName) {
         getPoolProperties().setDriverClassName(driverClassName);
     }
-    
-    
+
+
+    @Override
     public void setFairQueue(boolean fairQueue) {
         getPoolProperties().setFairQueue(fairQueue);
     }
 
-    
+
+    @Override
     public void setInitialSize(int initialSize) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setInitSQL(String initSQL) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setJdbcInterceptors(String jdbcInterceptors) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setJmxEnabled(boolean jmxEnabled) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setLogAbandoned(boolean logAbandoned) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setMaxActive(int maxActive) {
         // TODO Auto-generated method stub
-        
+
     }
 
-     
+
+    @Override
     public void setMaxIdle(int maxIdle) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setMaxWait(int maxWait) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setMinEvictableIdleTimeMillis(int minEvictableIdleTimeMillis) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setMinIdle(int minIdle) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setNumTestsPerEvictionRun(int numTestsPerEvictionRun) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setPassword(String password) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setRemoveAbandoned(boolean removeAbandoned) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setRemoveAbandonedTimeout(int removeAbandonedTimeout) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setTestOnBorrow(boolean testOnBorrow) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setTestOnConnect(boolean testOnConnect) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setTestOnReturn(boolean testOnReturn) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setTestWhileIdle(boolean testWhileIdle) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setTimeBetweenEvictionRunsMillis(int timeBetweenEvictionRunsMillis) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setUrl(String url) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setUseEquals(boolean useEquals) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setUseLock(boolean useLock) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setUsername(String username) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setValidationInterval(long validationInterval) {
         // TODO Auto-generated method stub
-        
+
     }
 
-    
+
+    @Override
     public void setValidationQuery(String validationQuery) {
         // TODO Auto-generated method stub
-        
+
     }
-    
+
     /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setValidatorClassName(String className) {
         getPoolProperties().setValidatorClassName(className);
     }
@@ -582,69 +673,78 @@ public class ConnectionPool extends NotificationBroadcasterSupport implements Co
     /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public int getSuspectTimeout() {
-        return getPoolProperties().getSuspectTimeout(); 
+        return getPoolProperties().getSuspectTimeout();
     }
 
     /**
      * {@inheritDoc}
      */
-    
+
+    @Override
     public void setSuspectTimeout(int seconds) {
         //no op
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
+    @Override
     public void setDataSource(Object ds) {
         getPoolProperties().setDataSource(ds);
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
+    @Override
     public Object getDataSource() {
         return getPoolProperties().getDataSource();
     }
 
 
-    /** 
+    /**
      * {@inheritDoc}
      */
+    @Override
     public void setDataSourceJNDI(String jndiDS) {
         //noop
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
+    @Override
     public String getDataSourceJNDI() {
         return getPoolProperties().getDataSourceJNDI();
     }
-    
-    /** 
+
+    /**
      * {@inheritDoc}
      */
+    @Override
     public boolean isAlternateUsernameAllowed() {
         return getPoolProperties().isAlternateUsernameAllowed();
     }
-    
-    /** 
+
+    /**
      * {@inheritDoc}
      */
+    @Override
     public void setAlternateUsernameAllowed(boolean alternateUsernameAllowed) {
         //noop
     }
 
-    /** 
+    /**
      * {@inheritDoc}
      */
+    @Override
     public void setValidator(Validator validator) {
         //noop
     }
-    
-    
+
+
 
 }
