@@ -31,19 +31,26 @@ import javax.servlet.http.HttpSession;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotNull;
 import static org.junit.Assert.assertTrue;
+import static org.junit.Assert.fail;
 
 import org.junit.Test;
 
 import org.apache.catalina.Context;
+import org.apache.catalina.Wrapper;
 import org.apache.catalina.comet.CometEvent.EventType;
+import org.apache.catalina.connector.CometEventImpl;
+import org.apache.catalina.connector.Request;
+import org.apache.catalina.connector.Response;
 import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
+import org.apache.catalina.valves.TesterAccessLogValve;
+import org.apache.catalina.valves.ValveBase;
 
 public class TestCometProcessor extends TomcatBaseTest {
 
     @Test
-    public void testSimpleCometClient() throws Exception {
-        
+    public void testAsyncClose() throws Exception {
+
         if (!isCometSupported()) {
             return;
         }
@@ -52,24 +59,117 @@ public class TestCometProcessor extends TomcatBaseTest {
         Tomcat tomcat = getTomcatInstance();
         Context root = tomcat.addContext("", TEMP_DIR);
         Tomcat.addServlet(root, "comet", new SimpleCometServlet());
+        root.addServletMapping("/comet", "comet");
+        Tomcat.addServlet(root, "hello", new HelloWorldServlet());
+        root.addServletMapping("/hello", "hello");
+        root.getPipeline().addValve(new AsyncCometCloseValve());
+        tomcat.getConnector().setProperty("connectionTimeout", "5000");
+        tomcat.start();
+
+        // Create connection to Comet servlet
+        final Socket socket =
+            SocketFactory.getDefault().createSocket("localhost", getPort());
+        socket.setSoTimeout(5000);
+
+        final OutputStream os = socket.getOutputStream();
+        String requestLine = "POST http://localhost:" + getPort() +
+                "/comet HTTP/1.1\r\n";
+        os.write(requestLine.getBytes());
+        os.write("transfer-encoding: chunked\r\n".getBytes());
+        os.write("\r\n".getBytes());
+
+        InputStream is = socket.getInputStream();
+        ResponseReaderThread readThread = new ResponseReaderThread(is);
+        readThread.start();
+
+        // Wait for the comet request/response to finish
+        int count = 0;
+        while (count < 10 && !readThread.getResponse().endsWith("0\r\n\r\n")) {
+            Thread.sleep(500);
+            count++;
+        }
+
+        if (count == 10) {
+            fail("Comet request did not complete");
+        }
+
+        // Send a standard HTTP request on the same connection
+        requestLine = "GET http://localhost:" + getPort() +
+                "/hello HTTP/1.1\r\n";
+        os.write(requestLine.getBytes());
+        os.write("\r\n".getBytes());
+
+        // Check for the expected response
+        count = 0;
+        while (count < 10 && !readThread.getResponse().contains(
+                HelloWorldServlet.RESPONSE_TEXT)) {
+            Thread.sleep(500);
+            count++;
+        }
+
+        if (count == 10) {
+            fail("Non-comet request did not complete");
+        }
+
+        readThread.join();
+        os.close();
+        is.close();
+    }
+
+    @Test
+    public void testSimpleCometClient() throws Exception {
+        doSimpleCometTest(null);
+    }
+
+    @Test
+    public void testSimpleCometClientBeginFail() throws Exception {
+        doSimpleCometTest(SimpleCometServlet.FAIL_ON_BEGIN);
+    }
+
+    @Test
+    public void testSimpleCometClientReadFail() throws Exception {
+        doSimpleCometTest(SimpleCometServlet.FAIL_ON_READ);
+    }
+
+    @Test
+    public void testSimpleCometClientEndFail() throws Exception {
+        doSimpleCometTest(SimpleCometServlet.FAIL_ON_END);
+    }
+
+    private void doSimpleCometTest(String initParam) throws Exception {
+        if (!isCometSupported()) {
+            return;
+        }
+
+        // Setup Tomcat instance
+        Tomcat tomcat = getTomcatInstance();
+        Context root = tomcat.addContext("", TEMP_DIR);
+        Wrapper w = Tomcat.addServlet(root, "comet", new SimpleCometServlet());
+        if (initParam != null) {
+            w.addInitParameter(initParam, "true");
+        }
         root.addServletMapping("/", "comet");
+
+        TesterAccessLogValve alv = new TesterAccessLogValve();
+        root.getPipeline().addValve(alv);
+
         tomcat.start();
 
         // Create connection to Comet servlet
         final Socket socket =
             SocketFactory.getDefault().createSocket("localhost", getPort());
         socket.setSoTimeout(60000);
-        
+
         final OutputStream os = socket.getOutputStream();
         String requestLine = "POST http://localhost:" + getPort() +
                 "/ HTTP/1.1\r\n";
         os.write(requestLine.getBytes());
         os.write("transfer-encoding: chunked\r\n".getBytes());
         os.write("\r\n".getBytes());
-        
+
         PingWriterThread writeThread = new PingWriterThread(4, os);
         writeThread.start();
-        
+
         socket.setSoTimeout(25000);
         InputStream is = socket.getInputStream();
         ResponseReaderThread readThread = new ResponseReaderThread(is);
@@ -77,46 +177,61 @@ public class TestCometProcessor extends TomcatBaseTest {
         readThread.join();
         os.close();
         is.close();
-        
-        // Validate response
+
         String[] response = readThread.getResponse().split("\r\n");
-        assertEquals("HTTP/1.1 200 OK", response[0]);
-        assertEquals("Server: Apache-Coyote/1.1", response[1]);
-        assertTrue(response[2].startsWith("Set-Cookie: JSESSIONID="));
-        assertEquals("Content-Type: text/plain;charset=ISO-8859-1", response[3]);
-        assertEquals("Transfer-Encoding: chunked", response[4]);
-        assertTrue(response[5].startsWith("Date: "));
-        assertEquals("", response[6]);
-        assertEquals("7", response[7]);
-        assertEquals("BEGIN", response[8]);
-        assertEquals("", response[9]);
-        assertEquals("17", response[10]);
-        assertEquals("Client: READ: 4 bytes", response[11]);
-        assertEquals("", response[12]);
-        assertEquals("17", response[13]);
-        assertEquals("Client: READ: 4 bytes", response[14]);
-        assertEquals("", response[15]);
-        assertEquals("17", response[16]);
-        assertEquals("Client: READ: 4 bytes", response[17]);
-        assertEquals("", response[18]);
-        assertEquals("17", response[19]);
-        assertEquals("Client: READ: 4 bytes", response[20]);
-        assertEquals("", response[21]);
-        assertEquals("d", response[22]);
-        assertEquals("Client: END", response[23]);
-        assertEquals("", response[24]);
-        assertEquals("0", response[25]);
-        // Expect 26 lines
-        assertEquals(26, response.length);
+        if (initParam == null) {
+            // Normal response expected
+            // Validate response
+            assertEquals("HTTP/1.1 200 OK", response[0]);
+            assertEquals("Server: Apache-Coyote/1.1", response[1]);
+            assertTrue(response[2].startsWith("Set-Cookie: JSESSIONID="));
+            assertEquals("Content-Type: text/plain;charset=ISO-8859-1", response[3]);
+            assertEquals("Transfer-Encoding: chunked", response[4]);
+            assertTrue(response[5].startsWith("Date: "));
+            assertEquals("", response[6]);
+            assertEquals("7", response[7]);
+            assertEquals("BEGIN", response[8]);
+            assertEquals("", response[9]);
+            assertEquals("17", response[10]);
+            assertEquals("Client: READ: 4 bytes", response[11]);
+            assertEquals("", response[12]);
+            assertEquals("17", response[13]);
+            assertEquals("Client: READ: 4 bytes", response[14]);
+            assertEquals("", response[15]);
+            assertEquals("17", response[16]);
+            assertEquals("Client: READ: 4 bytes", response[17]);
+            assertEquals("", response[18]);
+            assertEquals("17", response[19]);
+            assertEquals("Client: READ: 4 bytes", response[20]);
+            assertEquals("", response[21]);
+            assertEquals("d", response[22]);
+            assertEquals("Client: END", response[23]);
+            assertEquals("", response[24]);
+            assertEquals("0", response[25]);
+            // Expect 26 lines
+            assertEquals(26, response.length);
+        } else {
+            // Failure expected only expected for the fail on begin
+            // Failure at any later stage and the response headers (including
+            // the 200 response code will already have been sent to the client
+            if (initParam == SimpleCometServlet.FAIL_ON_BEGIN) {
+                assertEquals("HTTP/1.1 500 Internal Server Error", response[0]);
+                alv.validateAccessLog(1, 500, 0, 1000);
+            } else {
+                assertEquals("HTTP/1.1 200 OK", response[0]);
+                alv.validateAccessLog(1, 200, 0, 9000);
+            }
+
+        }
     }
-    
+
     /**
      * Tests if the Comet connection is closed if the Tomcat connector is
      * stopped.
      */
     @Test
     public void testCometConnectorStop() throws Exception {
-        
+
         if (!isCometSupported()) {
             return;
         }
@@ -132,24 +247,24 @@ public class TestCometProcessor extends TomcatBaseTest {
         final Socket socket =
             SocketFactory.getDefault().createSocket("localhost", getPort());
         socket.setSoTimeout(10000);
-        
+
         final OutputStream os = socket.getOutputStream();
         String requestLine = "POST http://localhost:" + getPort() +
                 "/ HTTP/1.1\r\n";
         os.write(requestLine.getBytes());
         os.write("transfer-encoding: chunked\r\n".getBytes());
         os.write("\r\n".getBytes());
-        
+
         PingWriterThread writeThread = new PingWriterThread(100, os);
         writeThread.start();
 
         InputStream is = socket.getInputStream();
         ResponseReaderThread readThread = new ResponseReaderThread(is);
         readThread.start();
-        
+
         // Allow the first couple of PING messages to be written
         Thread.sleep(3000);
-        
+
         tomcat.getConnector().stop();
         // Allow the executor a chance to send the end event
         Thread.sleep(100);
@@ -194,6 +309,26 @@ public class TestCometProcessor extends TomcatBaseTest {
 
         private static final long serialVersionUID = 1L;
 
+        public static final String FAIL_ON_BEGIN = "failOnBegin";
+        public static final String FAIL_ON_READ = "failOnRead";
+        public static final String FAIL_ON_END = "failOnEnd";
+
+        private boolean failOnBegin = false;
+        private boolean failOnRead = false;
+        private boolean failOnEnd = false;
+
+
+        @Override
+        public void init() throws ServletException {
+            failOnBegin = Boolean.valueOf(getServletConfig().getInitParameter(
+                    FAIL_ON_BEGIN)).booleanValue();
+            failOnRead = Boolean.valueOf(getServletConfig().getInitParameter(
+                    FAIL_ON_READ)).booleanValue();
+            failOnEnd = Boolean.valueOf(getServletConfig().getInitParameter(
+                    FAIL_ON_END)).booleanValue();
+        }
+
+
         @Override
         public void event(CometEvent event) throws IOException,
                 ServletException {
@@ -205,9 +340,15 @@ public class TestCometProcessor extends TomcatBaseTest {
             session.setMaxInactiveInterval(30);
 
             if (event.getEventType() == EventType.BEGIN) {
+                if (failOnBegin) {
+                    throw new IOException("Fail on begin");
+                }
                 response.setContentType("text/plain");
                 response.getWriter().print("BEGIN" + "\r\n");
             } else if (event.getEventType() == EventType.READ) {
+                if (failOnRead) {
+                    throw new IOException("Fail on read");
+                }
                 InputStream is = request.getInputStream();
                 int count = 0;
                 while (is.available() > 0) {
@@ -217,6 +358,9 @@ public class TestCometProcessor extends TomcatBaseTest {
                 String msg = "READ: " + count + " bytes";
                 response.getWriter().print("Client: " + msg + "\r\n");
             } else if (event.getEventType() == EventType.END) {
+                if (failOnEnd) {
+                    throw new IOException("Fail on end");
+                }
                 String msg = "END";
                 response.getWriter().print("Client: " + msg + "\r\n");
                 event.close();
@@ -229,9 +373,9 @@ public class TestCometProcessor extends TomcatBaseTest {
     }
 
     private static class PingWriterThread extends Thread {
-        
-        private int pingCount;
-        private OutputStream os;
+
+        private final int pingCount;
+        private final OutputStream os;
         private volatile Exception e = null;
 
         public PingWriterThread(int pingCount, OutputStream os) {
@@ -262,8 +406,8 @@ public class TestCometProcessor extends TomcatBaseTest {
 
     private static class ResponseReaderThread extends Thread {
 
-        private InputStream is;
-        private StringBuilder response = new StringBuilder();
+        private final InputStream is;
+        private final StringBuilder response = new StringBuilder();
 
         public ResponseReaderThread(InputStream is) {
             this.is = is;
@@ -286,4 +430,41 @@ public class TestCometProcessor extends TomcatBaseTest {
             }
         }
     }
+
+    private static class AsyncCometCloseValve extends ValveBase {
+
+        @Override
+        public void invoke(Request request, Response response)
+                throws IOException, ServletException {
+
+            CometEventImpl event = new CometEventImpl(request, response);
+
+            getNext().invoke(request, response);
+
+            if (request.isComet()) {
+                Thread t = new AsyncCometCloseThread(event);
+                t.start();
+            }
+        }
+    }
+
+    private static class AsyncCometCloseThread extends Thread {
+
+        private final CometEvent event;
+
+        public AsyncCometCloseThread(CometEvent event) {
+            this.event = event;
+        }
+
+        @Override
+        public void run() {
+            try {
+                Thread.sleep(2000);
+                event.close();
+            } catch (Exception e) {
+                // Test should fail. Report what went wrong.
+                e.printStackTrace();
+            }
+        }
+    }
 }
